#========================================================================================
# package      DE_EPAGES::Core::API::Log
# state        public
#----------------------------------------------------------------------------------------
# description  This module provides an interface for logging. It's actually a wrapper
#               for the Log::Log4perl module.
#
# example      use DE_EPAGES::Core::API::Log qw( InitLog GetLog );
#               InitLog( '/etc/log4perl.conf' );
#               GetLog()->info( 'The logging system is initialized' );
#
# example      ; example appender log4perl.conf
#               log4perl.appender.error_log=DE_EPAGES::Core::API::Log4PerlFileAppender
#               log4perl.appender.error_log.filename=sub { join ('/', $ENV{'EPAGES_LOG'}, 'error.log'); }
#               log4perl.appender.error_log.layout=PatternLayout
#               log4perl.appender.error_log.layout.ConversionPattern=[%d PID:%P Type:%p %H:%Y] %c - logged at %F Line %L. %n%m%n
#               log4perl.appender.error_log.Threshold=ERROR
#               log4perl.category = ERROR, error_log
#
# example      Possible placeholders at ConversionPattern are:
#               %c Category of the logging event.
#               %C Fully qualified package (or class) name of the caller
#               %d Current date in yyyy/MM/dd hh:mm:ss format
#               %F File where the logging event occurred
#               %H Hostname
#               %l Fully qualified name of the calling method followed by the
#                  callers source the file name and line number between
#                  parentheses.
#               %L Line number within the file where the log statement was issued
#               %m The message to be logged
#               %M Method or function where the logging request was issued
#               %n Newline (OS-independent)
#               %p Priority of the logging event
#               %P pid of the current process
#               %r Number of milliseconds elapsed from program start to logging
#                  event
#               %x The elements of the NDC stack (see below)
#               %X{key} The entry 'key' of the MDC (see below)
#               %% A literal percent (%) sign
#
#========================================================================================
package DE_EPAGES::Core::API::Log;
use base qw( Exporter Log::Log4perl );

use strict;
our @EXPORT_OK = qw(
    GetAppender
    GetCommunicationLog
    GetCountLog
    GetLog
    InitLog
    LogCommunication
    LogDebug
    LogError
    LogWarnings
    TestLog
    SetLogLevel
    LogContext
);

use Carp;
use Data::Dumper qw ( Dumper );
use DE_EPAGES::Core::API::Error qw ( HandleError );
use DE_EPAGES::Core::API::Log4perl::NDCHash;
use Log::Log4perl::Level;
use Log::Log4perl::NDC;
use Scalar::Util qw ( blessed );

our $INIT=0; # true after InitLog()
our $ALLOW_LOG_DEBUG = 1; # allows to disable LogDebug() -> up to 5% performance gain
                          # add to log4perl.conf:
                          # log4perl.category.DE_EPAGES::Core::API::Log::LogDebug = FATAL

my %LOG_LEVELS = (
    'FATAL' => FATAL_INT,
    'ERROR' => ERROR_INT,
    'WARN'  => WARN_INT,
    'INFO'  => INFO_INT,
    'DEBUG' => DEBUG_INT,
);

use constant WATCH_CONFIG_INTERVAL => 10; # watch for changes in log4perl.conf every 10 seconds

#========================================================================================
# function     GetAppender
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Appender = GetAppender( $Name );
# example      my $Appender = GetAppender( 'error_log' );
#               if( defined $Appender ) {
#                   $Appender->log( message => 'An error occured while processing.' );
#               }
#----------------------------------------------------------------------------------------
# description  Returns a log appender by name. The appender must be specified in the
#               log4perl.conf file. Returns undef if the appender name is not found.
#               The appender object supports the following methods:
#               <ul><li>log( %params )</li>
#               </ul>
#               (The function should not die, because it may be used inside an error
#               handler)
#----------------------------------------------------------------------------------------
# input        $Name | log appender name | string
# return       $Appender | appender object, see perloc Log::Log4perl | object
#========================================================================================
sub GetAppender {
    my ($Name) = @_;

    my $Appenders = Log::Log4perl->appenders;
    return undef unless exists $Appenders->{$Name};
    return $Appenders->{$Name}{'appender'};
}

#========================================================================================
# function     GetLog
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Logger = GetLog();
# syntax       $Logger = GetLog( $Category );
# example      GetLog( 'WebInterface' )->fatal( 'The Internet was switched off.' );
#----------------------------------------------------------------------------------------
# description  Returns a logger for a category. If no category is specified, the
#               caller package and function name is used as category.
#               The logger object supports the following methods:
#               <ul><li>debug
#                   <li>info
#                   <li>warn
#                   <li>error
#                   <li>fatal
#               </ul>
#----------------------------------------------------------------------------------------
# input        $Category | logger category | string
# return       $Appender | logger object, see perloc Log::Log4perl | object
#========================================================================================
sub GetLog(;$) {
    my ( $Category ) = @_;

    unless ($INIT)  {
        InitLog();
        carp 'GetLog() called without InitLog()';
    }

    if (not defined $Category) {
        my $Counter = 1;
        do {
            $Category = (caller($Counter++))[3];
            $Category = 'main' unless defined $Category;
        } while ($Category eq '(eval)' or $Category eq 'DE_EPAGES::Core::API::Log::LogDebug' );
    }
    return Log::Log4perl->get_logger( $Category );
}

#========================================================================================
# function     InitLog
# state        public
#----------------------------------------------------------------------------------------
# syntax       InitLog();
# syntax       InitLog( $ConfigFile );
# example      InitLog( '/etc/log4perl.conf' );
#----------------------------------------------------------------------------------------
# description  Initializes the logging system. See `perldoc Log::Log4perl` for the
#               syntax of the configuration file.
#----------------------------------------------------------------------------------------
# input        $ConfigFile | configuration file name, see perldoc Log::Log4Perl for details
#                   (optional, default: $ENV{'LOG4PERL'} || $ENV{'EPAGES_CONFIG'}/log4perl.conf)
#                   | string
#========================================================================================
sub InitLog(;$) {
    my ($ConfigFile) = @_;
    unless( defined $ConfigFile ) {
        $ConfigFile = $ENV{'LOG4PERL'} || $ENV{'EPAGES_CONFIG'}.'/log4perl.conf';
    }

    Log::Log4perl->init_and_watch( $ConfigFile, WATCH_CONFIG_INTERVAL );
    $INIT = 1;
    $ALLOW_LOG_DEBUG = 0 unless GetLog(__PACKAGE__.'::LogDebug')->is_error;

    GetLog()->debug('InitLog: logging started');
    return;
}

#========================================================================================
# function     LogError
# state        public
#----------------------------------------------------------------------------------------
# syntax       LogError( $Error );
# example      eval { ... };
#               if( ExistsError() ) {
#                   LogError( GetError() );
#               }
#----------------------------------------------------------------------------------------
# description  Writes an error to the log file.
#----------------------------------------------------------------------------------------
# input        $Error | an error object | object
# return       1 | always 1 | boolean
#========================================================================================
sub LogError($) {
    my ($Error) = @_;

    if($Error->getNdcHash()) {
        LogContext($Error->getNdcHash(), sub{
            GetLog( $Error->subname() )->error( $Error->longMessage() );
        });
    } else {
        GetLog( $Error->subname() )->error( $Error->longMessage() );
    }
    return 1;
}

#========================================================================================
# function     LogWarnings
# state        public
#----------------------------------------------------------------------------------------
# syntax       LogWarnings( $aWarnings);
# example      LogWarnings( GetWarnings() );
#----------------------------------------------------------------------------------------
# description  Writes a warnings to the log file.
#----------------------------------------------------------------------------------------
# input        $aWarnings | warning objects | ref.array.object
# return       1 | always 1 | boolean
#========================================================================================
sub LogWarnings($) {
    my ($aWarnings) = @_;

    foreach my $Warning (@$aWarnings) {
        if( GetLog()->is_debug ) {
            GetLog( $Warning->subname )->warn( $Warning->longMessage );
        }
        else {
            GetLog( $Warning->subname )->warn( $Warning->message );
        }
    }
    return 1;
}

#========================================================================================
# function     LogDebug
# state        public
#----------------------------------------------------------------------------------------
# syntax       LogDebug( $Name, $rStructure);
# syntax       LogDebug( $Name, $rStructure, $Category );
# example      LogDebug( 'ProductInfo', \%ProductInfo, 'Product');
#----------------------------------------------------------------------------------------
# description  Dumps the data structure $rStructure to the log file if the log level for
#               category $Category (or current package) is 'DEBUG'.
#----------------------------------------------------------------------------------------
# input        $Name | name of data structure for identification in the log file | string
# input        $rStructure | the object | scalar (usually hash or array ref)
# input        $Category | debug category (default: calling package) | string
# return       nothing | nothing | nothing
#========================================================================================
sub LogDebug ($$;$) {

    return unless $ALLOW_LOG_DEBUG;

    my ($Name, $Structure, $Category) = @_;

    my $Log = GetLog($Category);
    return unless $Log->is_debug;

    local $Log::Log4perl::caller_depth = $Log::Log4perl::caller_depth + 1;
    $Log->debug(sub {
        local $Data::Dumper::Pad        = '';
        local $Data::Dumper::Varname    = $Name;
        local $Data::Dumper::Indent     = 1;
        local $Data::Dumper::Deepcopy   = 1;
        local $Data::Dumper::Sortkeys   = 1;
        local $Data::Dumper::Purity     = 0;
        local $Data::Dumper::Maxdepth   = 6;
        return Dumper($Structure);
    });
    return;
}

#========================================================================================
# function     GetCommunicationLog
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Logger = GetCommunicationLog( $Category );
# example      GetCommunicationLog( 'Ebay' )->fatal( 'The Internet was switched off.' );
#----------------------------------------------------------------------------------------
# description  Returns the logger for the "COMMUNICATION::$Category" category.
#               See also <function GetLog>.
#----------------------------------------------------------------------------------------
# input        $Category | (optional) logger category | string
# return       $Appender | logger object, see perloc Log::Log4perl | object
#========================================================================================
sub GetCommunicationLog(;$) {
    my ( $Category ) = @_;

    my $LogCategory = 'COMMUNICATION';
    $LogCategory .= "::$Category" if defined $Category;
    return GetLog( $LogCategory );
}

#========================================================================================
# function     GetCountLog
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Logger = GetCountLog( $Category );
# example      GetCountLog( 'Bla' )->fatal( 'negative count?' );
#----------------------------------------------------------------------------------------
# description  Returns the logger for the "COUNT::$Category" category. That category has
#               been created to address the need explained in EPG-18496, which is keeping
#               track of certain clicks which are relevant for business reasons. It might
#               also be used to keep track of other things. See also <function GetLog>.
#----------------------------------------------------------------------------------------
# input        $Category | (optional) logger category | string
# return       $Appender | logger object, see perloc Log::Log4perl | object
#========================================================================================
sub GetCountLog {
    my ( $Category ) = @_;
    return GetLog( $Category ? "COUNT::$Category" : 'COUNT' );
}

#========================================================================================
# function     LogCommunication
# state        public
#----------------------------------------------------------------------------------------
# syntax       LogCommunication( $Name, $rStructure, $Category );
# syntax       LogCommunication( $Name, $rStructure, $Category, $aFilter, $CallerDepth);
# example      LogCommunication( 'TransReturn', $TransReturn, 'WorldPay::PaymentCallback', ['callbackPW']);
#----------------------------------------------------------------------------------------
# description  Dumps the data structure $rStructure to the log file if the log level for
#               category $Category (or current package) is 'DEBUG'.
#----------------------------------------------------------------------------------------
# input        $Name | name of data structure for identification in the log file | string
# input        $rStructure | the object | scalar (usually hash or array ref)
# input        $Category | debug category | string
# input        $aFilter | (optional) filter (perl regular expression for
#                       s/$Filter[^\n]+\n/\n/gis) 
#                       OR
#                       (optional) a function pointer like
#                       sub {my $LogString = shift; $$LogString =~s/(?<=<xmlTag>)[^<]*?(?=<\/xmlTag>)/---replaced---/xgis;}
#                        | ref.array.string OR ref.code
# input        $CallerDepth | (optional) number of caller functions to remove from the
#                       stack trace | int
# input        $DataMaxDepth | maximum depth of perl structures to be logged out | int
# return       nothing | nothing | nothing
#========================================================================================
sub LogCommunication ($$$;$$$) {
    my ($Name, $Structure, $Category, $aFilter, $CallerDepth, $DataMaxDepth) = @_;
    $CallerDepth = defined $CallerDepth ? $CallerDepth + 1 : 1;
    local $Log::Log4perl::caller_depth = $Log::Log4perl::caller_depth + $CallerDepth;
    GetCommunicationLog($Category)->info(sub {
        local $Data::Dumper::Pad        = '';
        local $Data::Dumper::Varname    = $Name;
        local $Data::Dumper::Indent     = 1;
        local $Data::Dumper::Deepcopy   = 1;
        local $Data::Dumper::Sortkeys   = 1;
        local $Data::Dumper::Purity     = 0;
        local $Data::Dumper::Maxdepth   = $DataMaxDepth // 5;

        my $LogString = Dumper($Structure);
        if (defined $aFilter) {
            if( ref( $aFilter ) eq 'CODE' ){
                $aFilter->(\$LogString);
            }else{
                foreach my $Filter (@$aFilter) {
                    $LogString =~ s/$Filter.*?\n/\n/gis;
                }
            }
        }
        return $LogString;
    });
    return;
}

#========================================================================================
# function     TestLog
# state        public
#----------------------------------------------------------------------------------------
# syntax       TestLog();
# example      perl -e "use DE_EPAGES::Core::API::Log qw (TestLog); TestLog();"
#----------------------------------------------------------------------------------------
# description  Calls each log level (DEBUG, INFO, WARN, ERROR, FATAL) at log category
#               'BUGTRACKER::Log4perl'. This is useful to test the log4perl.conf.
#========================================================================================
sub TestLog () {
    my $Log = GetLog('BUGTRACKER::Log4perl');
    $Log->debug( 'test debug' );
    $Log->info( 'test info' );
    $Log->warn( 'test warn' );
    $Log->error( 'test error' );
    $Log->fatal( 'test fatal' );
    return;
}

#========================================================================================
# function     SetLogLevel
# state        public
#----------------------------------------------------------------------------------------
# syntax       SetLogLevel( $LevelString, $Sub, $Category );
# example      SetLogLevel( 'DEBUG', sub { GetLog->debug('hello'); });
#----------------------------------------------------------------------------------------
# description  Set temporary log level for category $Category (or current package).
#----------------------------------------------------------------------------------------
# input        $LevelString | name of level (DEBUG,WARN,ERROR,FATAL,INFO)| string
# input        $Sub | code block | ref.code
# input        $Category | debug category (default: calling package) | string
#========================================================================================
sub SetLogLevel ($$;$) {
    my ($LevelString, $Sub, $Category) = @_;
    local $Log::Log4perl::caller_depth = $Log::Log4perl::caller_depth + 1;
    my $Logger = GetLog($Category);
    my $Level = $Logger->level;
    $Logger->level($LOG_LEVELS{$LevelString});
    eval { $Sub->(); };
    my $Error = $@;
    $Logger->level($Level);
    die $Error if $Error;
}

#========================================================================================
# function     LogContext
# state        public
#----------------------------------------------------------------------------------------
# syntax       LogContext( $hContext, $cSub );
#----------------------------------------------------------------------------------------
# description  Adds context variables to NDC stack used by the log4perl framework.
#               In addition to simple NDC, this is not limited to simple strings, and can
#               be composed. Inside log4perl this is used as %x, and will use json
#               for serialization of the context hash.
#----------------------------------------------------------------------------------------
# example      LogContext({Store => $Store}, sub {
#                 my $iShops = LoadRootObject->child('Shops')->childrenIterator;
#                 while( my $Shop = <$iShops> ) {
#                   LogContext({Shop => $Shop->alias}, sub {
#                     # contains Store:X,Shop:Y in %x
#                     GetLog->debug('simple logging..')
#                   });
#                 }
#               });
#----------------------------------------------------------------------------------------
# input        $hContext | context hash | ref.hash
# input        $cSub | code to run | ref.sub
#========================================================================================
sub LogContext {
    my ($hContext, $cSub) = @_;

    # fetch top element from NDC stack.
    my $previousContext = Log::Log4perl::NDC->pop();

    # put regular elements back on NDC stack.
    if (defined($previousContext) && !(blessed ($previousContext) && $previousContext->can('context'))) {
        Log::Log4perl::NDC->push($previousContext);
        $previousContext = undef;
    }

    # push NDCHash onto stack.
    Log::Log4perl::NDC->push(DE_EPAGES::Core::API::Log4perl::NDCHash->new($hContext, $previousContext));

    eval {
        $cSub->();
    };
    HandleError(
        'Finally' => sub {
            my ($Error) = @_;
            # pop NDCHash from stack.
            my $NdcHash = Log::Log4perl::NDC->pop();
            if(defined $Error){
              if (blessed ($NdcHash) && $NdcHash->can('context')) {
                  $Error->addNdcHash($NdcHash->context());
              }
            }
            # put previous NDCHash back onto stack.
            if(defined($previousContext)) {
                Log::Log4perl::NDC->push($previousContext);
            }
        }
    );

    return;
}

1;
