#========================================================================================
# package      DE_EPAGES::Design::t::StyleHandler
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for the TLE functions of the package
#               DE_EPAGES::Design::API::TLE::StyleHandler
#========================================================================================
package DE_EPAGES::Design::t::StyleHandler;

use base DE_EPAGES::Core::API::Test;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Design::API::TLE::StyleHandler;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::TLE::API::BaseProcessor;
use Test::More;
use strict;

use constant WEBURLUNITYSSL => 'http://www.my-testshop.com';
use constant THEME_SPOTLIGHT => 'epages.spotlight';
use constant STYLE_STYLE => 'style';
use constant STYLE_PATTERN => 'pattern';
use constant THEME_NEUTRAL => 'epages.neutral';

sub test {
    _test_GetUnityThemeStyleURL();

    done_testing();
}

sub _test_GetUnityThemeStyleURL {

    # Build a wrapper function to use the TLE function "GetUnityThemeStyleURL" in this Perl code
    my $Processor = DE_EPAGES::TLE::API::BaseProcessor->new;
    my $StyleHandler = DE_EPAGES::Design::API::TLE::StyleHandler->new();
    $StyleHandler->register($Processor);
    my $GetUnityThemeStyleURL = sub {
        my ($Object, $ChangeAction) = @_;
        $Processor->callBlock('GetUnityThemeStyleURL', sub {[ $Object, $ChangeAction ]});
    };
    my $ShopContent;

    # simulate broken Unity connection
    my $Shop = _createBrokenShop();
    is($GetUnityThemeStyleURL->($Shop, 'PickupBasket'), undef,
        "No UnityThemeStyleURL constructed for Object \"Shop\" and ChangeAction \"PickupBasket\" in case of a broken Unity connection.");
    is($GetUnityThemeStyleURL->($Shop, 'SaveCustomerData'), undef,
        "No UnityThemeStyleURL constructed for Object \"Shop\" and ChangeAction \"SaveCustomerData\" in case of a broken Unity connection.");

    # test theme: epages.spotlight@dev, style: style
    $Shop = _createShop(THEME_SPOTLIGHT, STYLE_STYLE);
    $ShopContent = $Shop->sendUnityRequest('GET', '/shop');
    my $UnityThemeStyleURL = $ShopContent->{'checkoutStylesheetUrl'};

    is($GetUnityThemeStyleURL->($Processor, ''), undef,
        "No UnityThemeStyleURL constructed for Object \"Processor\" and empty ChangeAction.");
    is($GetUnityThemeStyleURL->($Processor, 'PickupBasket'), undef,
        "No UnityThemeStyleURL constructed for Object \"Processor\" and ChangeAction \"PickupBasket\".");
    is($GetUnityThemeStyleURL->($Shop, 'SaveCustomerData'), undef,
        "No UnityThemeStyleURL constructed for Object \"Shop\" and ChangeAction \"SaveCustomerData\".");
    is($GetUnityThemeStyleURL->($Shop, 'PickupBasket'), $UnityThemeStyleURL,
        "Theme \"" . THEME_SPOTLIGHT . "\", Style \"" . STYLE_STYLE
        . "\": UnityThemeStyleURL constructed properly for ChangeAction \"PickupBasket\".");
    is($GetUnityThemeStyleURL->($Shop, 'SaveCustomerData'), $UnityThemeStyleURL,
       "Theme \"" . THEME_SPOTLIGHT . "\", Style \"" . STYLE_STYLE
        . "\": UnityThemeStyleURL loaded properly for ChangeAction \"SaveCustomerData\".");
    is($GetUnityThemeStyleURL->($Shop, ''), $UnityThemeStyleURL,
       "Theme \"" . THEME_SPOTLIGHT . "\", Style \"" . STYLE_STYLE
        . "\": UnityThemeStyleURL loaded properly for ChangeAction \"\".");
    is($GetUnityThemeStyleURL->($Shop, undef), $UnityThemeStyleURL,
       "Theme \"" . THEME_SPOTLIGHT . "\", Style \"" . STYLE_STYLE
        . "\": UnityThemeStyleURL loaded properly for undef ChangeAction.");

    # change style => test theme: epages.spotlight@dev, style: pattern
    $Shop = _createShop(THEME_SPOTLIGHT, STYLE_PATTERN);

    #change the stored UnityThemeStyleURL using the change action PickupBasket
    $Shop = _createShop(THEME_SPOTLIGHT, STYLE_STYLE);
    $ShopContent = $Shop->sendUnityRequest('GET', '/shop');

    is($GetUnityThemeStyleURL->($Shop, 'PickupBasket'), $UnityThemeStyleURL,
        "Theme \"" . THEME_SPOTLIGHT . "\", Style \"" . STYLE_PATTERN
        . "\": UnityThemeStyleURL constructed properly for ChangeAction \"PickupBasket\".");
    is($GetUnityThemeStyleURL->($Shop, 'SaveCustomerData'), $UnityThemeStyleURL,
       "Theme \"" . THEME_SPOTLIGHT . "\", Style \"" . STYLE_PATTERN
        . "\": UnityThemeStyleURL loaded properly for ChangeAction \"SaveCustomerData\".");

    # change theme => test theme: epages.neutral@2, style: style
    $Shop = _createShop(THEME_NEUTRAL, STYLE_STYLE);

    #change the stored UnityThemeStyleURL using the change action PickupBasket
    $ShopContent = $Shop->sendUnityRequest('GET', '/shop');
    $UnityThemeStyleURL = $ShopContent->{'checkoutStylesheetUrl'};

    is($GetUnityThemeStyleURL->($Shop, 'PickupBasket'), $UnityThemeStyleURL,
        "Theme \"" . THEME_NEUTRAL . "\", Style \"" . STYLE_STYLE
        . "\": UnityThemeStyleURL constructed properly for ChangeAction \"PickupBasket\".");
    is($GetUnityThemeStyleURL->($Shop, 'SaveCustomerData'), $UnityThemeStyleURL,
       "Theme \"" . THEME_NEUTRAL . "\", Style \"" . STYLE_STYLE
        . "\": UnityThemeStyleURL loaded properly for ChangeAction \"SaveCustomerData\".");

    return;
}

sub _createShop {
    my ($ThemeName, $StyleName) = @_;

    my $hAttributes = {
        'WebUrlUnitySSL' => WEBURLUNITYSSL,
        'instanceOf' => 'Shop',
        'UnityThemeStylePath' => undef
    };
    my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Shop::API::Object::Shop', $hAttributes);

    $MockifyShop->addMockWithReturnValueAndParameterCheck('sendUnityRequest', {
        'theme' => {
            'name' => $ThemeName
        },
        'themeStyle' => {
            'name' => $StyleName
        }
    }, [{'string'=>'GET'}, {'string'=>'/shop'}]);
    return $MockifyShop->getMockObject();
}

sub _createBrokenShop {
    my ($ThemeName, $ThemeVersion, $StyleName) = @_;

    my $hAttributes = {
        'WebUrlUnitySSL' => WEBURLUNITYSSL,
        'instanceOf' => 'Shop',
        'UnityThemeStylePath' => undef
    };
    my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Shop::API::Object::Shop', $hAttributes);

    $MockifyShop->addMockWithReturnValueAndParameterCheck('sendUnityRequest', undef, [{'string'=>'GET'}, {'string'=>'/shop'}]);
    return $MockifyShop->getMockObject();
}

RunScript( Message => 'long', Sub=> sub {
    RunOnStore( Store => GetTestSite(), Sub => \&test );
});

1;
