#========================================================================================
# package      DE_EPAGES::GoogleSmartShopping::API::REST::GoogleSmartShoppingProductsExportDriver
# state        private
#----------------------------------------------------------------------------------------
# description  This module implements a csv driver to export object GoogleSmartShopping products
#
# example      use DE_EPAGES::GoogleSmartShopping::API::REST::GoogleSmartShoppingProductsExportDriver;
#
#               my $Driver = DE_EPAGES::GoogleSmartShopping::API::REST::GoogleSmartShoppingProductsExportDriver->new(
#                   Handler => $Handler
#               );
#               $Driver->exportREST;
#========================================================================================
package DE_EPAGES::GoogleSmartShopping::API::REST::GoogleSmartShoppingProductsExportDriver;
use base qw ( DE_EPAGES::ProductPortal::API::REST::PortalExportDriver Exporter);

use strict;
use DE_EPAGES::GoogleSmartShopping::API::ProductBatchService;
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::Log qw ( LogDebug );
use DE_EPAGES::Core::API::PerlTools qw ( fcmp isValid );
use DE_EPAGES::Core::API::Url qw ( UrlEncodeValue );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::GoogleSmartShopping::API::Constants qw (
    $ALLOWED_AMOUNTS
    $ALLOWED_EXCEPTION_ML_VALUES
    $ALLOWED_EXCEPTIONS_KG_VALUES
    $GOOGLE_VALID_UNIT_TYPES
    $GOOGLE_VALID_UNIT_CODES
    $SUPPORTED_LOCALE_IDS
    GOOGLE_DECIMAL_DIGITS
    GOOGLE_DECIMAL_FILL
    $GOOGLE_VALID_AREA_UNIT
    $GOOGLE_VALID_LENGTH_UNIT
    $GOOGLE_VALID_VOLUME_UNIT
    $GOOGLE_VALID_WEIGHT_UNIT
);
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath );
use DE_EPAGES::Object::API::Language qw ( GetCodeByLanguageID );
use DE_EPAGES::Product::API::Product qw ( GetUnityPrettyUrl );
use DE_EPAGES::Shop::API::Constants qw ( STOCK_IS_ORDERABLE );
use DE_EPAGES::Trigger::API::Trigger qw (TriggerHook);
use Scalar::Util qw ( blessed );
use Encode qw (decode_utf8);

use constant MAX_LENGTH_TITLE                  => 150;
use constant MAX_LENGTH_DESCRIPTION            => 10000;
use constant GENDER_ATTRIBUTE => 'Gender';
use constant AGE_GROUP_ATTRIBUTE => 'Age_group';
use constant COLOR_ATTRIBUTE => 'Color';
use constant SIZE_ATTRIBUTE => 'Size';
use constant REFERENCE_UNIT => 'RefUnit';
use constant REFERENCE_AMOUNT => 'RefAmount';
use constant MATERIAL_ATTRIBUTE => 'Material';
use constant PATTERN_ATTRIBUTE => 'Pattern';
our @ACCEPTED_WEIGHT_UNITS = qw (pound ounce gram kilogram);
our @PRODUCT_IDENTIFIERS = qw (Manufacturer UPCEAN ManufacturerSKU);

#========================================================================================
my @MAP_GSS_ATTRIBUTES = qw (
    offerId
    title
    description
    googleProductCategory
    productTypes
    link
    imageLink
    condition
    availability
    price
    brand
    gtin
    mpn
    identifierExists
    taxes
    unitPricingMeasure
    unitPricingBaseMeasure
    shipping
    shippingWeight
    gender
    ageGroup
    sizes
    color
    itemGroupId
    material
    pattern
    isBundle
    targetCountry
    contentLanguage
    channel
);

my $MaxLengthTitle = MAX_LENGTH_TITLE;

#========================================================================================
sub ErrorDefinition {
    return {
        'ValueTooLong'            => 'attribute ( #AttributeName ) for product ( #ProductID ) is too long (Maximum #MaxLength characters). Value has been cut.',
        'ProductNoDescription'    => 'product ( #ProductID ) has no description, can not export to GoogleSmartShopping.',
        'ProductNoGender' =>         'product ( #ProductID ) has no gender.',
        'ProductNoAgeGroup' =>       'product ( #ProductID ) has no age group.',
        'ProductNoColor' =>          'product ( #ProductID ) has no color.',
        'ProductNoSize' =>           'product ( #ProductID ) has no size.',
        'ProductReferenceAmountMisconfigured' => 'product ( #ProductID ) has reference amount misconfigurated.',
        'ProductNoMaterial' =>       'product ( #ProductID ) has no material attribute.',
        'ProductNoPattern' =>        'product ( #ProductID ) has no pattern attribute.',
        'ProductNoShipping' =>       'product ( #ProductID ) has no shipping available for the target country.',
    }
}

#========================================================================================
sub exportREST {
    my $self = shift;
    my ($ViewObject, $NoMonitor, $PortalConfig) = @_;

    $self->{'PortalLanguageID'} = $PortalConfig->get('LanguageID');
    $self->{'PortalLanguageCode'} = GetCodeByLanguageID( $self->{'PortalLanguageID'} );

    my $PortalSiteCountry = $ViewObject->get('PortalSite')->get('Country');
    $self->{'PortalSiteUppercaseCountry'} = uc( $PortalSiteCountry->{'Code2'} );
    $self->{'PortalShippingMethodCountries'} = $self->_getShippingMethodCountries($ViewObject->get('ShippingMethod'));

    $MaxLengthTitle = ($ViewObject->get('MaxTitleLength') > MAX_LENGTH_TITLE) ?
        MAX_LENGTH_TITLE : $ViewObject->get('MaxTitleLength');

    $self->{'ProductBatchService'} = DE_EPAGES::GoogleSmartShopping::API::ProductBatchService->new($PortalConfig->get('Shop'));

    $self->SUPER::exportREST([@MAP_GSS_ATTRIBUTES], $ViewObject, $NoMonitor);
    return;
}

##========================================================================================
# function     exportItem
# state        public
#----------------------------------------------------------------------------------------
# syntax       exportItem( $aItemData );
#----------------------------------------------------------------------------------------
# description  Creates an hash %ProductHash where 
#               the keys are MAP_GSS_ATTRIBUTES and the values are $aItemData
#----------------------------------------------------------------------------------------
# input        $aItemData | a product export values| refarray
#========================================================================================
sub exportItem {
    my $self = shift;
    my ($aItemData) = @_;

    my %ProductHash;

    @ProductHash{@MAP_GSS_ATTRIBUTES} = @{$aItemData};

    $self->{'ProductBatchService'}->ExportItem(\%ProductHash);
    return;
}

#========================================================================================
sub onStartExport {
    my $self = shift;

    $self->{'ProductBatchService'}->OnStartBatch();
    return;
}

#========================================================================================
sub onEndExport {
    my $self = shift;

    $self->{'ProductBatchService'}->OnEndBatch();
    return;
}

#========================================================================================
sub getCountryCode {
    my $self = shift;
    return lc($self->portalSite->get('Country')->{'Code2'});
}

#========================================================================================
sub isProductValidPortalExt1 {
    my $self = shift;
    my ($Product) = @_;

    $self->SUPER::isProductValidPortalExt1($Product);

    # test for description
    my $DescriptionAttribute = $self->descriptionAttribute;
    if ( defined $DescriptionAttribute ) {
        my $Alias = $DescriptionAttribute->alias;
        Error('ProductNoDescription', {'ProductID'=>$Product->id, 'Product'=>$Product}) unless defined $Product->getWithParent($Alias, $self->languageID );
    }

    # test for Shipping Country
    my $aProductShippingCountries = $self->_getProductShippingCountries($Product);

    unless ( $self->{'PortalSiteUppercaseCountry'} ~~ @{$aProductShippingCountries}) {
        Error('ProductNoShipping', {'ProductID'=>$Product->id, 'Product'=>$Product});
    }

    return;
}

#========================================================================================
sub isProductValidPortal {
    my $self = shift;
    my ($Product) = @_;

    $self->isProductValidPortalExt1($Product);
    $self->_isReferenceAttributesValid ($Product);

    return;
}

#========================================================================================
sub formatPrice {
    my $self = shift;
    my ($Price) = @_;

    my $CurrencyID = $self->currencyID;
    my $FormatAttributes = $self->portalSite->formatAttributes($CurrencyID);
    my $Formatter = DE_EPAGES::Core::API::Object::NumberFormat->new(
        decimal_point  => '.',
        thousands_sep  => ',',
        neg_format     => $FormatAttributes->{'NegFormatNumbers'},
        decimal_digits => GOOGLE_DECIMAL_DIGITS,
        decimal_fill   => GOOGLE_DECIMAL_FILL,
        int_curr_symbol => '',
        curr_pos_format => 'x ',
    );
    my $Value = substr($Formatter->format_price($Price), 0, -1);
    return {
        'value' => $Value,
        'currency' => $CurrencyID,
    };
}

#========================================================================================
sub getPrice {
    my $self = shift;
    my ($Product) = @_;

    my $Price = $self->productPrice($Product);
    $Price += $Product->getDeposit($self->currencyID, $self->taxModel, 0) // 0;
    $Price += $Product->getEcoParticipation($self->currencyID, $self->taxModel, 0) // 0;
    return $Price;
}

#========================================================================================

# export column product_type
sub productTypes {
    my $self = shift;
    my ($Product) = @_;

    my $ShopID = $Product->getSite()->id;
    foreach my $Category (@{$Product->getWithParent('Categories')}) {
        next if ($Category->instanceOf('CategorySpecialOffer')); # no special offer category
        next if ($Category->parent->id == $ShopID); # not the start page
        return [$Category->get('NameOrAlias', $self->languageID)];
    }
    return [];
}

# export column link
sub link {
    my $self = shift;
    my ($Product) = @_;

    my $Shop = $Product->getSite();
    my $URL;
    if ($Shop->isUnity()) {
        my $PortalLanguageID = $self->{'PortalLanguageID'};
        $URL = $self->_productUrlForUnity($Shop, $Product, $PortalLanguageID);
        LogDebug('GoogleSmartShopping Link URL: ', { 'Url' => $URL, 'PortalLanguageID' => $PortalLanguageID });
    } else {
        $URL = $self->productUrl($Product);
        LogDebug('GoogleSmartShopping Link URL: ', $URL);
    }

    return $URL;
}

# export column imageLink
sub imageLink {
    my $self = shift;
    my ($Product) = @_;

    my $imageUrl = $self->imageUrl($Product, $self->imageAttribute);
    return undef unless isValid($imageUrl);

    # EPG-19813: mask special chars in image file name
    my $index = rindex($imageUrl, '/');
    if ($index > -1) {
        $imageUrl = substr($imageUrl, 0, $index) . '/' . UrlEncodeValue(substr($imageUrl, $index + 1));
    }
    else {
        $imageUrl = UrlEncodeValue($imageUrl); # can this be possible?
    }
    return $imageUrl;
}

# export column price
sub price {
    my $self = shift;
    my ($Product) = @_;
    my $Price = $self->getPrice($Product);
    my $FormatedPrice = $self->formatPrice($Price);
    return $FormatedPrice;
}

# export column offerId
sub offerId {
    my $self = shift;
    my ($Product) = @_;

    my $LocaleID = $self->portalSite->get('LocaleID');

    return $Product->alias() . '_' . $LocaleID;
}

# export column title
sub title {
    my $self = shift;
    my ($Product) = @_;

    my $Value = $Product->getWithParent('Name', $self->languageID );

    if (length($Value) > $MaxLengthTitle) {
        Warning('ValueTooLong', {'ProductID'=>$Product->id, 'Product'=>$Product, 'AttributeName' => 'title', 'MaxLength' => $MaxLengthTitle});
        $Value = substr($Value, 0, $MaxLengthTitle);
    }
    $Value =~ s/\"/\'/g;
    return $Value;
}

# export column description
sub description {
    my $self = shift;
    my ($Product) = @_;

    my $DescriptionAttribute = $self->descriptionAttribute();
    if ( defined $DescriptionAttribute ) {
        my $Alias = $DescriptionAttribute->alias();
        my $Value = $Product->getWithParent($Alias, $self->languageID );

        if (defined $Value) {
            $Value = decode_utf8($Value);
            $Value = $self->removeHtmlTags($Value);
            if (length($Value) > MAX_LENGTH_DESCRIPTION) {
                Warning('ValueTooLong', {'ProductID'=>$Product->id, 'Product'=>$Product, 'AttributeName' => 'description', 'MaxLength' => MAX_LENGTH_DESCRIPTION});
                $Value = substr($Value, 0, MAX_LENGTH_DESCRIPTION);
            }
        }
        else {
            Error('ProductNoDescription', {'ProductID' => $Product->id});
        }
        $Value =~ s/\"/\'/g;
        return $Value;
    }
    return;
}

# export column brand
sub brand {
    my $self = shift;
    my ($Product) = @_;

    return $Product->getWithParent('Manufacturer');
}

# export column shipping_weight
sub shippingWeight {
    my $self = shift;
    my ($Product) = @_;

    my $Weight = $Product->getWithParent('Weight');
    return undef unless defined $Weight;

    my $WeightUnitName = undef;
    my $WeightUnit = $Product->getWithParent('WeightUnit');
    if (defined $WeightUnit) {
        # EPG-19587
        # Google does only accept lb, oz, g, kg
        # so we will calculate the weight to a unit Google accepts as follows:
        # mg => g
        # any unit other than lb, oz, g, kg, mg => kg
        my $WeightUnitAlias = $WeightUnit->alias;
        if ($WeightUnitAlias eq 'milligram') {
            $Weight /= 1000; ## no critic qw(ProhibitMagicNumbers)
            $WeightUnit = LoadObjectByPath('/Units/gram');
        }
        elsif (!($WeightUnitAlias ~~ @ACCEPTED_WEIGHT_UNITS)) {
            my $Factor = $WeightUnit->get('Factor');
            if (defined($Factor) && fcmp($Factor, 0) != 0) {
                $Weight = $Weight * $Factor / 1000; ## no critic qw(ProhibitMagicNumbers)
                $WeightUnit = LoadObjectByPath('/Units/gram/kilogram');
            }
        }
        $WeightUnitName = $WeightUnit->get('Abbreviation') // $WeightUnit->alias;
    }

    return {
        'value' => $Weight,
        'unit' => $WeightUnitName,
    };
}

# export column gtin
sub gtin {
    my $self = shift;
    my ($Product) = @_;

    return $Product->getWithParent('UPCEAN');
}

# condition
sub condition {
    my $self = shift;
    my ($Product) = @_;

    return 'new';
}

# shipping
sub shipping {
    my $self = shift;
    my ($Product) = @_;

    my $ShippingPrice = $self->shippingPrice($Product);
    my $hFormattedShippingPrice = $self->formatPrice($ShippingPrice);

    my $aProductShippingCountries = $self->_getProductShippingCountries($Product);

    my @GoogleFormattedShipping;

    foreach my $Country (@{$aProductShippingCountries}) {
        push(@GoogleFormattedShipping, {
            'country' => $Country,
            'price' => $hFormattedShippingPrice,
        });
    }

    return \@GoogleFormattedShipping;
}

# manufacturer product number
sub mpn {
    my $self = shift;
    my ($Product) = @_;

    return $Product->getWithParent('ManufacturerSKU');
}


sub googleProductCategory {
    my $self = shift;
    my ($Product) = @_;

    my $aPossibleAttributeNames = ['Google_Product_Category', 'google_product_category'];

    my $GoogleProductCategory;
    foreach my $AttributeName (@{$aPossibleAttributeNames}){
        if($Product->get('Class')->existsAttribute($AttributeName)){
            $GoogleProductCategory = $Product->getWithParent($AttributeName, $self->languageID());
            last;
        }
    }

    if ( !defined $GoogleProductCategory) {
        return undef;
    }

    # EPG-19668: only use the first line of attribute value and remove all quotes
    my $AmountOfCharactersBeforeLineBreak = index($GoogleProductCategory, "\n");
    if ($AmountOfCharactersBeforeLineBreak != -1) { # -1 means no match of index
        my $StartPosition = 0;
        $GoogleProductCategory = substr($GoogleProductCategory, $StartPosition, $AmountOfCharactersBeforeLineBreak);
    }
    $GoogleProductCategory =~ s/"//xg; # " # remove quotes
    return $GoogleProductCategory;
}

# availability
sub availability {
    my $self = shift;
    my ($Product) = @_;

    my $StockLevel = $Product->getWithParent('StockLevel');
    my $StockLevelAction = $Product->getSite()->get('ProductLowOnStockAction');

    # The stock level could be empty (if the merchant don't want to fill it) or above zero
    return 'in stock' unless isValid($StockLevel);
    return 'in stock' if $StockLevelAction == STOCK_IS_ORDERABLE;
    return 'in stock' if $StockLevel > 0;

    return 'out of stock';
}

# taxes
sub taxes {
    my $self = shift;
    my ($Product) = @_;

    return undef;
}

# EPG-22192
# The warnings are commented because we dont do wording now
# export column gender
sub gender {
    my $self = shift;
    my ($Product) = @_;

    # US only
    return '' unless $self->portalSite->get('LocaleID') ~~ $SUPPORTED_LOCALE_IDS;
    # test if gender set
    unless ($Product->get('Class')->existsAttribute(GENDER_ATTRIBUTE)) {
#        Warning('ProductNoGender', {
#            'ProductID'      => $Product->id,
#            'Product'        => $Product,
#            'AttributeAlias' => GENDER_ATTRIBUTE
#        });
        return undef;
    }

    my $Gender = $Product->getWithParent(GENDER_ATTRIBUTE, $self->languageID);
    unless(isValid($Gender)) {
#        Warning('ProductNoGender', {
#            'ProductID'      => $Product->id,
#            'Product'        => $Product,
#            'AttributeAlias' => GENDER_ATTRIBUTE
#        });
        return undef;
    }
    # We check the type of the attribute to get its value
    if (blessed($Gender) && $Gender->can('get')) {
        return $Gender->get('Alias');
    } elsif (ref $Gender eq 'ARRAY') {
        return undef;
    } elsif (! ref $Gender) {
        return $Gender;
    }

}
# export column age group
sub ageGroup {
    my $self = shift;
    my ($Product) = @_;


    return '' unless $self->portalSite->get('LocaleID') ~~ $SUPPORTED_LOCALE_IDS;
    # test if age group set
    unless ($Product->get('Class')->existsAttribute(AGE_GROUP_ATTRIBUTE)) {
#        Warning('ProductNoAgeGroup', {
#            'ProductID'      => $Product->id,
#            'Product'        => $Product,
#            'AttributeAlias' => AGE_GROUP_ATTRIBUTE
#        });
        return undef;
    }
    my $AgeGroup = $Product->getWithParent(AGE_GROUP_ATTRIBUTE, $self->languageID);
    unless(isValid($AgeGroup)) {
#        Warning('ProductNoAgeGroup', {
#            'ProductID'      => $Product->id,
#            'Product'        => $Product,
#            'AttributeAlias' => AGE_GROUP_ATTRIBUTE
#        });
        return undef;
    }

    # We check the type of the attribute to get its value
    if (blessed($AgeGroup) && $AgeGroup->can('get'))
    {
        return $AgeGroup->get('Alias');
    }
    elsif (ref $AgeGroup eq 'ARRAY')
    {
        return undef;
    }
    elsif (! ref $AgeGroup)
    {
    return $AgeGroup;
    }

}

# export column color
sub color {
    my $self = shift;
    my ($Product) = @_;


    return '' unless $self->portalSite->get('LocaleID') ~~ $SUPPORTED_LOCALE_IDS;

    # test if color set
    unless ($Product->get('Class')->existsAttribute(COLOR_ATTRIBUTE)) {
#        Warning('ProductNoColor', {
#            'ProductID'      => $Product->id,
#            'Product'        => $Product,
#            'AttributeAlias' => COLOR_ATTRIBUTE
#        });
        return undef;
    }
    my $Color = $Product->getWithParent(COLOR_ATTRIBUTE, $self->languageID);
    unless(isValid($Color)) {
#        Warning('ProductNoColor', {
#            'ProductID'      => $Product->id,
#            'Product'        => $Product,
#            'AttributeAlias' => COLOR_ATTRIBUTE
#        });
        return undef;
    }

    # We check the type of the attribute to get its value
    if (blessed($Color) && $Color->can('get'))
    {
    return $Color->get('Alias');
}
    elsif (ref $Color eq 'ARRAY')
    {
        return undef;
    }
    elsif (! ref $Color)
    {
        return $Color;
    }
}
# export column size
sub sizes {
    my $self = shift;
    my ($Product) = @_;

    return [] unless $self->portalSite->get('LocaleID') ~~ $SUPPORTED_LOCALE_IDS;

    # test if size set
    unless ($Product->get('Class')->existsAttribute(SIZE_ATTRIBUTE)) {
#        Warning('ProductNoSize', {
#            'ProductID'      => $Product->id,
#            'Product'        => $Product,
#            'AttributeAlias' => SIZE_ATTRIBUTE
#        });
        return [];
    }
    my $Size = $Product->getWithParent(SIZE_ATTRIBUTE, $self->languageID);
    unless(isValid($Size)) {
#        Warning('ProductNoSize', {
#            'ProductID'      => $Product->id,
#            'Product'        => $Product,
#            'AttributeAlias' => SIZE_ATTRIBUTE
#        });
        return [];
    }

    # We check the type of the attribute to get its value
    if (blessed($Size) && $Size->can('get')) {
        return [$Size->get('Alias')];
    } elsif (ref $Size eq 'ARRAY') {
        return [];
    } elsif (! ref $Size) {
        return [$Size];
    }
}
# export column itemGroupId, only for variant, with the alias of the super product
sub itemGroupId {
    my $self = shift;
    my ($Product) = @_;
    return undef unless $self->portalSite->get('LocaleID') ~~ $SUPPORTED_LOCALE_IDS;
    #Check if the product is a variant
    my $SuperProduct = $Product->getWithParent('SuperProduct');
    if (defined($SuperProduct)) {
        # We check the type of the attribute to get its value
        if (blessed($SuperProduct) && $SuperProduct->can('get')) {
            return $SuperProduct->get('Alias');
        }
        elsif (ref $SuperProduct eq 'ARRAY') {
            return undef;
        }
        elsif (! ref $SuperProduct) {
            return $SuperProduct;
        }
    } else {
        return undef;
    }
}

# export column material
sub material {
    my $self = shift;
    my ($Product) = @_;

    return undef unless $self->portalSite->get('LocaleID') ~~ $SUPPORTED_LOCALE_IDS;

    # test if size set
    unless ($Product->get('Class')->existsAttribute(MATERIAL_ATTRIBUTE)) {
        return undef;
    }
    my $Material = $Product->getWithParent(MATERIAL_ATTRIBUTE, $self->languageID);

    return undef unless(isValid($Material));

    # We check the type of the attribute to get its value
    if (blessed($Material) && $Material->can('get'))
    {
        return $Material->get('Alias');
    }
    elsif (ref $Material eq 'ARRAY')
    {
        return undef;
    }
    elsif (! ref $Material)
    {
        return $Material;
    }
}

# export column pattern
sub pattern {
    my $self = shift;
    my ($Product) = @_;

    return undef unless $self->portalSite->get('LocaleID') ~~ $SUPPORTED_LOCALE_IDS;

    # test if size set
    unless ($Product->get('Class')->existsAttribute(PATTERN_ATTRIBUTE)) {
        return undef;
    }
    my $Pattern = $Product->getWithParent(PATTERN_ATTRIBUTE, $self->languageID);
    return undef unless(isValid($Pattern));

    # We check the type of the attribute to get its value
    if (blessed($Pattern) && $Pattern->can('get'))
    {
        return $Pattern->get('Alias');
    }
    elsif (ref $Pattern eq 'ARRAY')
    {
        return undef;
    }
    elsif (! ref $Pattern)
    {
        return $Pattern;
    }
}

#Export identifier exist
sub identifierExists {
    my $self = shift;
    my ($Product) = @_;

    my $identifierExists = 'false';

    #at least 2 of the following 3 identifiers are required: 'brand', 'gtin', and 'mpn'
    my @Information = values(%{$Product->getWithParent(\@PRODUCT_IDENTIFIERS)});
    if ( scalar(grep {isValid($_)} @Information ) >= 2 )
    {
        $identifierExists='true';
    }

    return $identifierExists;
}

sub unitPricingMeasure {
    my $self = shift;
    my ($Product) = @_;

    my $hProductInfo = $Product->getWithParent( [qw( RefContentAmount RefUnit )] );

    my $UnitPricingMeasure;
    if ($self->_isValidUnitPricingMeasure($hProductInfo)) {
        $UnitPricingMeasure = $self->_getUnitPricingMeasure($hProductInfo);
    }
    return $UnitPricingMeasure;
}

sub unitPricingBaseMeasure{
    my $self = shift;
    my ($Product) = @_;

    my $hProductInfo = $Product->getWithParent( [qw( RefContentAmount RefAmount RefUnit )] );

    my $Value = '';
    my $UnitAbbreviation = '';

    if ($self->_isValidUnitPricingBaseMeasure($hProductInfo)) {
        $UnitAbbreviation = $hProductInfo->{RefUnit}->get( 'Abbreviation' );
        if ($hProductInfo->{RefUnit}->get('Code') eq 'MTK') {
            $UnitAbbreviation = 'sqm';
        }
        $Value = $hProductInfo->{RefAmount};

        #convert 750ml to 75cl
        if ('ml' eq $UnitAbbreviation && 750 == $Value) {
            $Value = 75;
            $UnitAbbreviation = 'cl';
        }
        return {
            'value' => $Value,
            'unit' => $UnitAbbreviation,
        };
    }
    return undef;
}

sub isBundle {
    my $self = shift;
    my ($Product) = @_;

    my $IsBundleProduct = $Product->get('IsBundleProduct');
    my $IsBundle = $IsBundleProduct ? 'true' : 'false';

    return $IsBundle;
}

sub targetCountry {
    my $self = shift;

   return $self->{'PortalSiteUppercaseCountry'};
}

sub contentLanguage {
    my $self = shift;

   return $self->{'PortalLanguageCode'};
}

sub channel {
    my $self = shift;

   return 'online';
}

#---------------------------------------------------------------------------
#  private methods
#---------------------------------------------------------------------------
sub _getProductShippingCountries {
    my $self = shift;
    my ($Product) = @_;

    my $PortalConfig = $self->object();
    my $ProductShippingMethod = $PortalConfig->shippingMethodForProduct($Product);

    if ($ProductShippingMethod->id() == $self->shippingMethod()->id()) {
        return $self->{'PortalShippingMethodCountries'};
    }

    return $self->_getShippingMethodCountries($ProductShippingMethod);
}

sub _getShippingMethodCountries {
    my $self = shift;
    my ($ShippingMethod) = @_;

    if (defined(my $RegionShippingMethod = $ShippingMethod->get('Region'))) {
        my @ShippingCountries;

        foreach my $CountryShippingMethod (@{$RegionShippingMethod->get('Countries')}) {
                push(@ShippingCountries, uc( $CountryShippingMethod->{'Code2'}));
        }
        return \@ShippingCountries;
    }

    return [$self->{'PortalSiteUppercaseCountry'}];
}

sub _isReferenceAmountValid {
    my $self = shift;
    my ($unitAbbreviation, $referenceAmount) = @_;

    return 1 if($referenceAmount ~~ $ALLOWED_AMOUNTS);

    #Additionally, 75cl, 50kg, and 1000kg are accepted.
    return 1 if ($unitAbbreviation eq 'kg' && $referenceAmount ~~ $ALLOWED_EXCEPTIONS_KG_VALUES);
    return 1 if ($unitAbbreviation eq 'ml' && $referenceAmount ~~ $ALLOWED_EXCEPTION_ML_VALUES);

    return 0;
}

sub _isReferenceAttributesValid {
    my $self    = shift;
    my ($Product) = @_;

    my $hProductInfo = $Product->getWithParent( [qw( RefContentAmount RefAmount RefUnit )] );

    #if the RefUnit and RefContentAmount attributes are null or empty, it is not an error
    if( isValid($hProductInfo->{RefUnit}) &&
        isValid($hProductInfo->{RefContentAmount}))
    {
        $self->_isProductReferenceAmountMisconfigured( $hProductInfo, $Product );
    }
    return;
}

sub _isProductReferenceUnitMisconfigured {
    my $self = shift;
    my ($hProductInfo, $Product) = @_;

    if (isValid($hProductInfo->{RefUnit}->get('Type'))){
        return ($hProductInfo->{RefUnit}->get('Code') ~~ $GOOGLE_VALID_UNIT_CODES ||
            $hProductInfo->{RefUnit}->get('Code') eq 'MLT') ? 1 : 0;
    }
    return 0;
}

sub _isProductReferenceAmountMisconfigured {
    my $self = shift;
    my ($hProductInfo, $Product) = @_;

    #if the RefAmount attribute is null or empty, it is not an error
    if (isValid($hProductInfo->{RefAmount}) && $self->_isProductReferenceUnitMisconfigured( $hProductInfo, $Product ))
    {
        unless( $self->_isReferenceAmountValid( $hProductInfo->{RefUnit}->get( 'Abbreviation' ), $hProductInfo->{RefAmount}))
        {
            Error('ProductReferenceAmountMisconfigured', {
                'ProductID'      => $Product->id,
                'Product'        => $Product,
                'AttributeAlias' => REFERENCE_AMOUNT
            });
        }
    }
}

sub _isValidGoogleWeightMeasurement {
    my $self = shift;
    my ($hProductInfo) = @_;
    return ($hProductInfo->{RefUnit}->get('Code') ~~ $GOOGLE_VALID_WEIGHT_UNIT) ? 1 : 0;
}

sub _isValidGoogleVolumeMeasurement {
    my $self = shift;
    my ($hProductInfo) = @_;
    return ($hProductInfo->{RefUnit}->get('Code') ~~ $GOOGLE_VALID_VOLUME_UNIT) ? 1 : 0;
}

sub _isValidGoogleAreaMeasurement {
    my $self = shift;
    my ($hProductInfo) = @_;
    return ($hProductInfo->{RefUnit}->get('Code') ~~ $GOOGLE_VALID_AREA_UNIT) ? 1 : 0;
}

sub _isValidGoogleLengthMeasurement {
    my $self = shift;
    my ($hProductInfo) = @_;
    return ($hProductInfo->{RefUnit}->get('Code') ~~ $GOOGLE_VALID_LENGTH_UNIT) ? 1 : 0;
}

sub _isValidUnitPricingBaseMeasure {
    my $self = shift;
    my ($hProductInfo) = @_;
    return ( isValid($hProductInfo->{RefContentAmount}) &&
        isValid($hProductInfo->{RefUnit}) &&
        isValid($hProductInfo->{RefAmount}) &&
        ($hProductInfo->{RefUnit}->get('Code') ~~ $GOOGLE_VALID_UNIT_CODES ||
            $hProductInfo->{RefUnit}->get('Code') eq 'MLT') &&
        ($hProductInfo->{RefUnit}->get('Type') ne 'area' ||
            $hProductInfo->{RefUnit}->get('Code') eq 'MTK')) ? 1 : 0;
}

sub _isValidUnitPricingMeasure {
    my $self = shift;
    my ($hProductInfo) = @_;
    return (isValid($hProductInfo->{RefContentAmount}) &&
            isValid($hProductInfo->{RefUnit}) &&
            ($hProductInfo->{RefUnit}->get('Type') ~~ $GOOGLE_VALID_UNIT_TYPES) ) ? 1 : 0;
}

sub _getAreaUnitPricing {
    my $self = shift;
    my ($hProductInfo) = @_;
    return ($self->_isValidGoogleAreaMeasurement($hProductInfo)) ?
        {'unit' => 'sqm', 'value' => $hProductInfo->{RefContentAmount}} :
        {'unit' => 'sqm', 'value' => $hProductInfo->{RefContentAmount} * $hProductInfo->{RefUnit}->get('Factor')};
}

sub _getMassUnitPricing {
    my $self = shift;
    my ($hProductInfo) = @_;
    return ($self->_isValidGoogleWeightMeasurement($hProductInfo)) ?
        {'unit' => $hProductInfo->{RefUnit}->get( 'Abbreviation' ), 'value' => $hProductInfo->{RefContentAmount}} :
        {'unit' => 'g', 'value' => $hProductInfo->{RefContentAmount} * $hProductInfo->{RefUnit}->get('Factor')};
}

sub _getVolumeUnitPricing {
    my $self = shift;
    my ($hProductInfo) = @_;
    return ($self->_isValidGoogleVolumeMeasurement($hProductInfo)) ?
        {'unit' => $hProductInfo->{RefUnit}->get( 'Abbreviation' ), 'value' => $hProductInfo->{RefContentAmount}} :
        {'unit' => 'l', 'value' => $hProductInfo->{RefContentAmount} * $hProductInfo->{RefUnit}->get('Factor')};
}

sub _getLengthUnitPricing {
    my $self = shift;
    my ($hProductInfo) = @_;
    return ($self->_isValidGoogleLengthMeasurement($hProductInfo)) ?
        {'unit' => $hProductInfo->{RefUnit}->get( 'Abbreviation' ), 'value' => $hProductInfo->{RefContentAmount}} :
        {'unit' => 'm', 'value' => $hProductInfo->{RefContentAmount} * $hProductInfo->{RefUnit}->get('Factor')};
}

sub _getUnitPricingMeasure {
    my $self = shift;
    my ($hProductInfo) = @_;

    my $UnitPricingMeasure;
    if ($hProductInfo->{RefUnit}->get('Type') eq 'area') {
        $UnitPricingMeasure = $self->_getAreaUnitPricing($hProductInfo);
    }
    elsif ($hProductInfo->{RefUnit}->get('Type') eq 'mass') {
        $UnitPricingMeasure = $self->_getMassUnitPricing($hProductInfo);
    }
    elsif ($hProductInfo->{RefUnit}->get('Type') eq 'volume') {
        $UnitPricingMeasure = $self->_getVolumeUnitPricing($hProductInfo);
    }
    elsif ($hProductInfo->{RefUnit}->get('Type') eq 'length') {
        $UnitPricingMeasure = $self->_getLengthUnitPricing($hProductInfo);
    }
    return $UnitPricingMeasure;
}

sub _productUrlForUnity {
    my $self = shift;
    my ($Shop, $Product, $PortalLanguageID) = @_;

    my $ShopLanguageID = $Shop->get('LanguageID');
    my $UseStoredPrettyUrl = 1;
    my $UrlWithoutExtensions = GetUnityPrettyUrl($Product, $ShopLanguageID, $PortalLanguageID, $UseStoredPrettyUrl);

    # check if there are required extensions to add
    my $PortalConfig = $self->object();

    my $hParams = {
        'PortalConfig' => $PortalConfig,
        'Url'          => $UrlWithoutExtensions
    };
    TriggerHook('PortalExportProductUrl', $hParams);

    return $hParams->{'Url'};
}

1;
