#========================================================================================
# package      DE_EPAGES::GoogleSmartShopping::t::unit::GoogleApiService
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for UI::Provider
#========================================================================================
package DE_EPAGES::GoogleSmartShopping::t::unit::GoogleApiService;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::GoogleSmartShopping::API::Constants qw ( EPAGESJ_BASE_URI );

use Test::Mockify::Sut;
use Test::Mockify::Matcher qw ( Object String HashRef ArrayRef );
use Test::Mockify::Verify qw ( WasCalled GetParametersFromMockifyCall );
use Test::Exception;
use Test::MockObject::Extends;
use Test::More;

use JSON::MaybeXS qw ( encode_json );
use HTTP::Status qw( HTTP_OK HTTP_INTERNAL_SERVER_ERROR );
use strict;

use constant AUTH_CODE => 'SOME_AUTH_CODE';
use constant ACCESS_TOKEN => 'SOME_ACCESS_TOKEN';
use constant OTHER_ACCESS_TOKEN => 'LOL_TOKEN';
use constant REFRESH_TOKEN => 'SOME_REFRESH_TOKEN';
use constant ACCOUNT_EMAIL => 'some_email@example.com';
use constant OTHER_ACCOUNT_EMAIL => 'wrong@email.com';
use constant ADMIN_REFRESH_TOKEN => 'SOME_ADMIN_REFRESH_TOKEN';
use constant ADMIN_ACCESS_TOKEN => 'SOME_ADMIN_ACCESS_TOKEN';

use constant GOOGLE_SITE_VERIFICATION_TOKEN => 'google12345.html';

use constant APP_CLIENT_ID => 'SomeClientId';
use constant APP_CLIENT_SECRET => 'SomeClientSecret';
use constant AUTH_TOKEN_URL => 'SomeAuthTokenUrl';
use constant APP_REDIRECT_URL => 'SomeAppRedirectUrl';
use constant GOOGLE_APP_URI => 'https://epages-de.github.io/google-app';
use constant STOREFRONT_WEBSITE_URL => 'https://shop.example.com';
use constant MULTI_CLIENT_ACCOUNT_ID => 389123456;
use constant MERCHANT_CENTER_ID => 458599058;

use constant EXPECTED_ACCESS_TOKEN_RESPONSE => { 'access_token' => ACCESS_TOKEN };
use constant OAUTH_RESPONSE_NO_ACCESS_TOKEN => { 'lol_token' => ACCESS_TOKEN };
use constant EXPECTED_GENERIC_ERROR_RESPONSE => {
    'error' => 'internal_server_error',
    'error_description' => 'Internal Server Error'
};

use constant EXPECTED_USERINFO_RESPONSE => { 'email' => ACCOUNT_EMAIL };
use constant EXPECTED_REFRESH_ERROR_RESPONSE => {
    'error' => 'invalid_grant',
    'error_description' => 'Token has been expired or revoked.'
};
use constant EXPECTED_USER_ACCOUNTINFO_RESPONSE => {
    "kind" => "content#accountsAuthInfoResponse",
    "accountIdentifiers" => [
        {
            "aggregatorId" => MULTI_CLIENT_ACCOUNT_ID
        }
    ]
};
use constant GENERIC_ERROR_MESSAGE => 'fail';

use constant EXPECTED_MERCHANT_CENTER_SUBACCOUNT_EPAGESJ_URL => "misc/google-smart-shopping/accounts/" . MULTI_CLIENT_ACCOUNT_ID . "/sub-accounts";
use constant EXPECTED_MERCHANT_CENTER_ACCOUNTS_RESPONSE => [{ 'name' => 'someMerchantCenterAccountName', 'id' => MERCHANT_CENTER_ID }];
use constant EXPECTED_MERCHANT_CENTER_ACCOUNTS_EPAGESJ_URL => 'misc/google-smart-shopping/accounts/merchant-center-accounts';
use constant EXPECTED_SITE_VERIFICATION_TOKEN_EPAGESJ_URL => 'misc/google-smart-shopping/accounts/site-verification-token';
use constant EXPECTED_SITE_VERIFICATION_EPAGESJ_URL => 'misc/google-smart-shopping/accounts/site-verification';
use constant ADS_MANAGER_ID => 333444;
use constant REFRESH_ERROR_DEFUNCT_TOKEN => {'error' => 'token_expired_or_revoked'};

use constant EXPECTED_SITE_VERIFICATION_TOKEN_RESPONSE => {'token' => GOOGLE_SITE_VERIFICATION_TOKEN};
use constant EXPECTED_SITE_VERIFICATION_RESPONSE => {};

use constant EXPECTED_MERCHANT_CENTER_ACCOUNT_STATUSES_EPAGESJ_RESPONSE => {
    "accountId"=> "414922848",
    "accountLevelIssues"=> [
        {
            "documentation"=> "https://support.google.com/merchants/answer/9849758",
            "id"=> "merchant_quality_low",
            "severity"=> "error",
            "title"=> "Account isn't eligible for enhanced free listings"
        },
        {
            "documentation"=> "https://support.google.com/merchants/answer/6159060",
            "id"=> "missing_ad_words_link",
            "severity"=> "error",
            "title"=> "No Google Ads account linked"
        }
    ],
    "kind"=> "content#accountStatus",
    "websiteClaimed"=> "false",
    "accountManagement"=> "manual"
};


use constant SUCCESSFUL_MERCHANT_CENTER_ACCOUNT_STATUSES_EPAGESJ_RESPONSE_HAS_EMPTY_PRODUCTS_ARRAY => {
    "accountId"=> "414922848",
    "accountLevelIssues"=> [
        {
            "documentation"=> "https://support.google.com/merchants/answer/9849758",
            "id"=> "merchant_quality_low",
            "severity"=> "error",
            "title"=> "Account isn't eligible for enhanced free listings"
        },
        {
            "documentation"=> "https://support.google.com/merchants/answer/6159060",
            "id"=> "missing_ad_words_link",
            "severity"=> "error",
            "title"=> "No Google Ads account linked"
        }
    ],
    "kind"=> "content#accountStatus",
    "products" => [],
    "websiteClaimed"=> "true",
    "accountManagement"=> "manual"
};
use constant EXPECTED_MERCHANT_CENTER_SUBACCOUNT_STATUSES_EPAGESJ_URL => 'misc/google-smart-shopping/accounts/'.MULTI_CLIENT_ACCOUNT_ID.'/account-statuses/'.MERCHANT_CENTER_ID;
use constant EXPECTED_MERCHANT_CENTER_STANDALONE_ACCOUNT_STATUSES_EPAGESJ_URL => 'misc/google-smart-shopping/accounts/'.MERCHANT_CENTER_ID.'/account-statuses/'.MERCHANT_CENTER_ID;

use constant TEST_ADDRESS_COUNTRY_NAME => "Germany";
use constant TEST_ADDRESS_COUNTRY_ISO_CODE => "DE";
use constant PHONE_NUMBER_VERIFICATION_ID => 'my-phone-number-verification-id';
use constant EXPECTED_PHONE_NUMBER_VERIFICATION_RESPONSE => {
'verificationId' => PHONE_NUMBER_VERIFICATION_ID
};
use constant PHONE_NUMBER_VERIFICATION_REQUEST => {
'phoneNumber'     => '152901820',
'languageCode'    => 'de_DE',
'phoneRegionCode' => 'DE'
};
use constant EXPECTED_PHONE_NUMBER_VERIFICATION_EPAGESJ_URL => EXPECTED_MERCHANT_CENTER_SUBACCOUNT_EPAGESJ_URL.'/'.MERCHANT_CENTER_ID.'/phone-verification';

use constant PHONE_NUMBER_VERIFICATION_CONFIRMATION_REQUEST => {
    'code' => 'my-sms-received-code'
};
use constant EXEPECTED_PHONE_NUMBER_VERIFICATION_CONFIRMATION_EPAGESJ_URL => EXPECTED_PHONE_NUMBER_VERIFICATION_EPAGESJ_URL.'/'.PHONE_NUMBER_VERIFICATION_ID;

use constant EXPECTED_PRODUCTS_LIST_RESPONSE => {
    "items" => [
    {
        "kind" => "content#product",
    "id" => "online:de:DE:1112_de_DE",
    "offerId" => "1112_de_DE",
    "source" => "feed",
    "identifierExists" => "true",
    "isBundle" => "false",
    "title" => "Join The Force! - Sticker",
    "description" => "Zeig allen, welche Seite du gewaehlt hast! Ideal fuer die Bildschirm Rueckseite Ihres Laptops oder Ihren Rimowa Koffer. Hochwertiger Sticker, farbiger Offsetdruck, matt ePages uuid => 99673562x-20211018",
    "link" => "https://merchandise.epagesdevs.epages.systems/p/join-the-force-sticker",
    "imageLink" => "http://merchandise.epagesdevs.epages.systems/WebRoot/Store/Shops/merchandise/6166/ED58/6D94/FCCE/619B/D580/9217/15A0/DSC-0003.JPG",
    "contentLanguage" => "de",
    "targetCountry" => "DE",
    "channel" => "online",
    "availability" => "in stock",
    "brand" => "ePages",
    "condition" => "new",
    "mpn" => "99673562x-20211018",
    "price" => {
    "value" => "0.19",
        "currency" => "EUR"
},
"productTypes" => [
    "Sticker"
],
    "shipping" => [
    {
        "price" => {
    "value" => "0.00",
        "currency" => "EUR"
},
"country" => "DE"
}
            ]
        },
        {
            "kind" => "content#product",
            "id" => "online:de:DE:1109_de_DE",
            "offerId" => "1109_de_DE",
            "source" => "feed",
            "identifierExists" => "true",
            "isBundle" => "false",
            "title" => "Small is powerful! - Flaschen / Dosenoeffner",
            "description" => "Wir verkaufen hier einen praktischen Flaschen- und Getraenkedosenoeffner mit einfarbingem Siebdruck des ePages Logos. Dank seines kleinen Formfaktors und des integrierten Schluesselrings ist er der ideale Begleiter fuer unterwegs. ePages uuid => 99675543x-20211018",
            "link" => "https://merchandise.epagesdevs.epages.systems/p/small-is-powerful-flaschen-dosenoeffner",
            "imageLink" => "http://merchandise.epagesdevs.epages.systems/WebRoot/Store/Shops/merchandise/615E/C542/6CD5/A08C/8444/D580/9217/3A04/DSC-0002.JPG",
            "contentLanguage" => "de",
            "targetCountry" => "DE",
            "channel" => "online",
            "availability" => "in stock",
            "brand" => "ePages",
            "condition" => "new",
            "mpn" => "99675543x-20211018",
            "price" => {
                "value" => "2.99",
                "currency" => "EUR"
            },
            "productTypes" => [
                "Tools"
            ],
            "shipping" => [
                {
                    "price" => {
                        "value" => "0.00",
                        "currency" => "EUR"
                    },
                    "country" => "DE"
                }
            ],
            "shippingWeight" => {
                "value" => 100,
                "unit" => "g"
            }
        },
        {
            "kind" => "content#product",
            "id" => "online:de:DE:1111_de_DE",
            "offerId" => "1111_de_DE",
            "source" => "feed",
            "identifierExists" => "true",
            "isBundle" => "false",
            "title" => "Simple is beautiful! - ePages Logo Keramik Tasse",
            "description" => "Die Tasse aus hochwertiger Gusskeramik kann bis zu 250ml Filterkaffee fassen, um Ihnen die noetige Energie fuer einen langen Arbeitstag zu liefern. Breite => 80mm Hoehe => 90mm Aufgrund von Produktionstoleranzen, koennen diese Masse um bis zu 5% schwanken. ePages uuid => 99675558x-20211018",
            "link" => "https://merchandise.epagesdevs.epages.systems/p/simple-is-beautiful-epages-logo-keramik-tasse",
            "imageLink" => "http://merchandise.epagesdevs.epages.systems/WebRoot/Store/Shops/merchandise/6166/B5EF/8B92/249B/D26B/D580/9217/4131/DSC-0006.JPG",
            "contentLanguage" => "de",
            "targetCountry" => "DE",
            "channel" => "online",
            "availability" => "in stock",
            "brand" => "ePages GmbH",
            "condition" => "new",
            "mpn" => "99675558x-20211018",
            "price" => {
                "value" => "5.99",
                "currency" => "EUR"
            },
            "productTypes" => [
                "Homeware"
            ],
            "shipping" => [
                {
                    "price" => {
                        "value" => "0.00",
                        "currency" => "EUR"
                    },
                    "country" => "DE"
                }
            ]
        },
        {
            "kind" => "content#product",
            "id" => "online:de:DE:1110_de_DE",
            "offerId" => "1110_de_DE",
            "source" => "feed",
            "identifierExists" => "true",
            "isBundle" => "false",
            "title" => "Ship It! - Baumwoll Poloshirt",
            "description" => "Beschreibung Baumwoll Poloshirt mit einfarbingem Flock Druck Zeigen Sie Ihre Begeisterung fuer eCommerce mit dem Ship it! - Poloshirt. Dem dritten und wichtigsten Credo unserer Firma. Groesse => LFarbe => PinkMaterial => Baumwolle Unisex ePages uuid => 99632162x-20211018",
            "link" => "https://merchandise.epagesdevs.epages.systems/p/ship-it-baumwoll-poloshirt",
            "imageLink" => "http://merchandise.epagesdevs.epages.systems/WebRoot/Store/Shops/merchandise/6166/B279/3C8E/9F12/623B/D580/9217/44FF/Selection-212.png",
            "contentLanguage" => "de",
            "targetCountry" => "DE",
            "channel" => "online",
            "ageGroup" => "adult",
            "availability" => "in stock",
            "brand" => "ePages GmbH",
            "color" => "pink",
            "condition" => "new",
            "gender" => "female",
            "material" => "Cotton",
            "mpn" => "99632162x-20211018",
            "pattern" => "none",
            "price" => {
                "value" => "5.00",
                "currency" => "EUR"
            },
            "productTypes" => [
                "Apparel"
            ],
            "shipping" => [
                {
                    "price" => {
                        "value" => "0.00",
                        "currency" => "EUR"
                    },
                    "country" => "DE"
                }
            ],
            "shippingWeight" => {
                "value" => 200,
                "unit" => "g"
            },
            "sizes" => [
                "S"
            ]
        }
    ]
};

use constant EXPECTED_PRODUCTS_STATUSES_RESPONSE => {
    'items' => [
        {
            'issues'     => [ {} ],
            'lastUpdate' => '2022-02-23T13:51:40Z',
            'productId'  => 'online:de:DE:36-rubberboots_de_DE',
            'status'     => 'disapproved'
        },
        {
            'issues'     => [ {} ],
            'lastUpdate' => '2022-02-25T18:25:28Z',
            'productId'  => 'online:de:DE:14-umbrella_de_DE',
            'status'     => 'disapproved'
        },
    ]
};
sub test {
    my $self = shift;

    $self->test_get_and_save_oauth_token_success();
    $self->test_get_and_save_oauth_token_success_existing_config();
    $self->test_get_and_save_oauth_token_failure();
    $self->test_get_and_save_oauth_token_failure_wrong_email();
    $self->test_get_and_save_oauth_token_failure_no_access_token();
    $self->test_revoke_token_success();
    $self->test_revoke_token_failure();
    $self->test_get_oauth_email_success();
    $self->test_get_oauth_email_failure();
    $self->test_get_user_account_info_success();
    $self->test_get_user_account_info_failure();
    $self->test_get_merchant_center_accounts_success();
    $self->test_get_merchant_center_accounts_no_accounts();
    $self->test_get_merchant_center_accounts_failure();
    $self->test_post_create_merchant_center_subaccount_success();
    $self->test_post_create_merchant_center_subaccount_failure();
    $self->test_delete_merchant_center_subaccount_success();
    $self->test_delete_merchant_center_subaccount_failure();
    $self->test_post_claim_website_success_no_mca();
    $self->test_post_claim_website_success_epages_mca();
    $self->test_post_claim_website_success_external_mca();
    $self->test_post_claim_website_failure();
    $self->test_get_site_verification_token_success();
    $self->test_get_site_verification_token_failure();
    $self->test_trigger_site_verification_success();
    $self->test_trigger_site_verification_failure();
    $self->test_get_merchant_center_account_statuses_subaccount_success();
    $self->test_get_merchant_center_account_statuses_standalone_merchant_account_success();
    $self->test_get_merchant_center_account_statuses_failure();
    $self->test_request_merchant_center_phone_verification_success();
    $self->test_confirm_merchant_center_phone_verification_success();
    $self->test_get_rated_product_statuses_no_products();
    $self->test_get_rated_product_statuses_empty_product_array();
    $self->test_get_rated_product_statuses_no_products_in_sales_country();
    $self->test_get_rated_product_statuses_low_quality_disapproved();
    $self->test_get_rated_product_statuses_low_quality_pending();
    $self->test_get_rated_product_statuses_low_quality_1st_case_9_from_20_active_shows_a_warning();
    $self->test_get_rated_product_statuses_low_quality_2nd_case_9_from_18_active_mo_warning();
    $self->test_get_rated_product_statuses_low_quality_3rd_case_19_from_100_active_shows_a_warning();
    $self->test_get_rated_product_statuses_low_quality_4th_case_20_from_100_active_mo_warning();
    $self->test_get_products_statuses_success();
    $self->test_list_products();
    $self->test_callProductsBatch();
    $self->test_delete_products();

    done_testing();
}

#----------------------------------------------------------------------------------------
# Tests
#----------------------------------------------------------------------------------------

sub test_get_and_save_oauth_token_success {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUserAgent = $self->_createUserAgent(HTTP_OK, EXPECTED_ACCESS_TOKEN_RESPONSE);
    my $AlreadyExistingConfig = 0;
    my $MockedShoppingConfig = $self->_createGoogleSmartShoppingConfig();
    my $MockedShop = $self->_createShop($MockedShoppingConfig, $AlreadyExistingConfig);
    my $SUT = $self->_createSUT({'UserAgent' => $MockedUserAgent, 'Shop' => $MockedShop});
    $SUT->{'GoogleSmartShoppingConfig'} = $MockedShoppingConfig;
    lives_ok(
        sub { $SUT->getAndSaveOAuthTokens(); },
        "$SubTestName - getOAuthTokens can be called without errors."
    );
    is(
        $SUT->{'Shop'}->get('GoogleSmartShoppingConfig')->get('AccessToken'),
        ACCESS_TOKEN,
        "$SubTestName - AccessToken correctly stored in GoogleSmartShoppingConfig object."
    );
}

sub test_get_and_save_oauth_token_success_existing_config {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUserAgent = $self->_createUserAgent(HTTP_OK, EXPECTED_ACCESS_TOKEN_RESPONSE);
    my $AlreadyExistingConfig = 1;
    my $MockedShoppingConfig = $self->_createGoogleSmartShoppingConfig();
    my $MockedShop = $self->_createShop($MockedShoppingConfig, $AlreadyExistingConfig);
    my $SUT = $self->_createSUT({'UserAgent' => $MockedUserAgent, 'Shop' => $MockedShop});
    $SUT->{'GoogleSmartShoppingConfig'} = $MockedShoppingConfig;
    lives_ok(
        sub { $SUT->getAndSaveOAuthTokens(); },
        "$SubTestName - getOAuthTokens can be called without errors."
    );
    is(
        $SUT->{'Shop'}->get('GoogleSmartShoppingConfig')->get('AccessToken'),
        ACCESS_TOKEN,
        "$SubTestName - AccessToken correctly stored in GoogleSmartShoppingConfig object."
    );
}

sub test_get_and_save_oauth_token_failure {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUserAgent = $self->_createUserAgent(HTTP_INTERNAL_SERVER_ERROR, EXPECTED_GENERIC_ERROR_RESPONSE);
    my $AlreadyExistingConfig = 0;
    my $MockedShoppingConfig = $self->_createGoogleSmartShoppingConfig();
    my $MockedShop = $self->_createShop($MockedShoppingConfig, $AlreadyExistingConfig);
    my $SUT = $self->_createSUT({ 'UserAgent' => $MockedUserAgent, 'Shop' => $MockedShop });
    eval {
        $SUT->getAndSaveOAuthTokens();
    };
    if (ExistsError()) {
        my $Error = GetError();
        is_deeply(
            $Error->vars->{'Response'},
            EXPECTED_GENERIC_ERROR_RESPONSE,
            "$SubTestName - getOAuthTokens throws error and saves Google API response in \$Error->vars->{'Response'} object."
        );
        is(
            $Error->vars->{'Status'},
            HTTP_INTERNAL_SERVER_ERROR,
            "$SubTestName - HTTP status code saved in \$Error->vars->{'Status'}"
        );
    } else {
        ok(0, "$SubTestName - getOAuthTokens should throw error given error response from Google's REST API.");
    }
}

sub test_get_and_save_oauth_token_failure_wrong_email {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUserAgent = $self->_createUserAgent(HTTP_OK, EXPECTED_ACCESS_TOKEN_RESPONSE);
    my $AlreadyExistingConfig = 1;
    my $MockedShoppingConfig = $self->_createGoogleSmartShoppingConfig({ AccountEmail => OTHER_ACCOUNT_EMAIL, AccessToken => OTHER_ACCESS_TOKEN });
    my $MockedShop = $self->_createShop($MockedShoppingConfig, $AlreadyExistingConfig);
    my $SUT = $self->_createSUT({
        'UserAgent'     => $MockedUserAgent,
        'Shop'          => $MockedShop,
        'DoGetResponse' => { 'email' => ACCOUNT_EMAIL }
    });
    eval {
        $SUT->getAndSaveOAuthTokens();
    };
    if (ExistsError()) {
        ok(1, "$SubTestName - getOAuthTokens should throw error if OAuth email doesn't match AccountEmail in GSSConfig.");
        is(
            $SUT->{'Shop'}->get('GoogleSmartShoppingConfig')->get('AccessToken'),
            OTHER_ACCESS_TOKEN,
            "$SubTestName - AccessToken should NOT be updated in GSSConfig object."
        );
        is(
            $SUT->{'Shop'}->get('GoogleSmartShoppingConfig')->get('AccountEmail'),
            OTHER_ACCOUNT_EMAIL,
            "$SubTestName - AccountEmail should NOT be updated in GSSConfig object."
        );
    } else {
        ok(0, "$SubTestName - getOAuthTokens should throw error given error response from Google's REST API.");
    }
}

sub test_get_and_save_oauth_token_failure_no_access_token {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUserAgent = $self->_createUserAgent(HTTP_OK, OAUTH_RESPONSE_NO_ACCESS_TOKEN);
    my $AlreadyExistingConfig = 0;
    my $MockedShoppingConfig = $self->_createGoogleSmartShoppingConfig();
    my $MockedShop = $self->_createShop($MockedShoppingConfig, $AlreadyExistingConfig);
    my $SUT = $self->_createSUT({ 'UserAgent' => $MockedUserAgent, 'Shop' => $MockedShop });
    eval {
        $SUT->getAndSaveOAuthTokens();
    };
    if (ExistsError()) {
        my $Error = GetError();
        is(
            $Error->vars->{'Status'},
            HTTP_INTERNAL_SERVER_ERROR,
            "$SubTestName - HTTP status code saved in \$Error->vars->{'Status'}"
        );
    } else {
        ok(0, "$SubTestName - getOAuthTokens should throw error given error response from Google's REST API.");
    }
}

sub test_revoke_token_success {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUserAgent = $self->_createUserAgent(HTTP_OK);
    my $SUT = $self->_createSUT({ 'UserAgent' => $MockedUserAgent });
    my $hResponse;
    lives_ok(
        sub { $hResponse = $SUT->revokeAccessToken(ACCESS_TOKEN);},
        "$SubTestName - Test revokeAccessToken lives ok."
    );
    is_deeply($hResponse, {} , "$SubTestName - Test revokeAccessToken success.");
}

sub test_revoke_token_failure {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedUserAgent = $self->_createUserAgent(HTTP_INTERNAL_SERVER_ERROR, EXPECTED_GENERIC_ERROR_RESPONSE);
    my $SUT = $self->_createSUT({ 'UserAgent' => $MockedUserAgent });
    my $hResponse;
    eval {
        $hResponse = $SUT->revokeAccessToken(ACCESS_TOKEN);
    };
    if (ExistsError()) {
        my $Error = GetError();
        is(
            $Error->vars->{'Response'},
            encode_json(EXPECTED_GENERIC_ERROR_RESPONSE),
            "$SubTestName - throw error if revoke fails."
        );
        is(
            $Error->vars->{'Status'},
            HTTP_INTERNAL_SERVER_ERROR,
            "$SubTestName - HTTP status code saved in \$Error->vars->{'Status'}"
        );
    } else {
        ok(0, "$SubTestName - revoke token should have thrown error in case of failure.");
    }
}

sub test_get_oauth_email_success {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedShoppingConfig = $self->_createGoogleSmartShoppingConfig();
    my $AlreadyExistingConfig = 1;
    my $MockedShop = $self->_createShop($MockedShoppingConfig, $AlreadyExistingConfig);
    my $SUT = $self->_createSUT({ 'Shop' => $MockedShop, 'DoGetResponse' => EXPECTED_USERINFO_RESPONSE });
    my $Email;
    lives_ok(
        sub { $Email = $SUT->getOAuthEmail(); },
        "$SubTestName - getOAuthEmail can be called without errors."
    );
    is( $Email, ACCOUNT_EMAIL, "$SubTestName - UserInfo email correctly returned." );
}

sub test_get_oauth_email_failure {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedShoppingConfig = $self->_createGoogleSmartShoppingConfig();
    my $AlreadyExistingConfig = 1;
    my $MockedShop = $self->_createShop($MockedShoppingConfig, $AlreadyExistingConfig);
    my $SUT = $self->_createSUT({ 'Shop' => $MockedShop, 'DoGetError' => GENERIC_ERROR_MESSAGE });
    my $Email;
    throws_ok(
        sub { $Email = $SUT->getOAuthEmail(); },
        qr/fail/,
        "$SubTestName - getOAuthEmail should not catch error in usual case."
    );
}

sub test_get_user_account_info_success {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $aResponse;
    my $SUT = $self->_createSUT({ 'DoGetResponse' => EXPECTED_USER_ACCOUNTINFO_RESPONSE });
    lives_ok(
        sub { $aResponse = $SUT->getUserAccountInfo(); },
        "$SubTestName - Varify that getMerchantCenterAccounts can be called without errors."
    );
}

sub test_get_user_account_info_failure {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedShoppingConfig = $self->_createGoogleSmartShoppingConfig();
    my $AlreadyExistingConfig = 1;
    my $MockedShop = $self->_createShop($MockedShoppingConfig, $AlreadyExistingConfig);
    my $MockedError = $self->_createError(HTTP_INTERNAL_SERVER_ERROR, EXPECTED_GENERIC_ERROR_RESPONSE);
    my $SUT = $self->_createSUT({ 'Shop' => $MockedShop, 'DoGetError' => $MockedError });
    my $aResponse;
    throws_ok(
        sub { $aResponse = $SUT->getUserAccountInfo(); },
        qr/Internal Server Error/,
        "$SubTestName - getUserAccountInfo should not catch generic error"
    );
}

sub test_get_merchant_center_accounts_success {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $aResponse;
    my $SUT = $self->_createSUT({ 'DoGetResponse' => EXPECTED_MERCHANT_CENTER_ACCOUNTS_RESPONSE });
    lives_ok(
        sub { $aResponse = $SUT->getMerchantCenterAccounts(); },
        "$SubTestName - Verify that getMerchantCenterAccounts can be called without errors."
    );
    ok(
        WasCalled($SUT, 'doGet'),
        $SubTestName . ' - Verify that doGet was called.'
    );
    my $aGetParams = GetParametersFromMockifyCall($SUT, 'doGet');
    is( @$aGetParams[0], EXPECTED_MERCHANT_CENTER_ACCOUNTS_EPAGESJ_URL, "$SubTestName - Verify doGet call correctly the epagesj endpoint." );
    is( $aResponse, EXPECTED_MERCHANT_CENTER_ACCOUNTS_RESPONSE, "$SubTestName - Varify that MerchantCenterAccounts list correctly returned." );
}

sub test_get_merchant_center_accounts_no_accounts {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $SUT = $self->_createSUT({ 'DoGetResponse' => [] });
    my $aResponse;
    lives_ok(
        sub { $aResponse = $SUT->getMerchantCenterAccounts(); },
        "$SubTestName - getMerchantCenterAccounts can be called without errors."
    );
    is_deeply( $aResponse, [], "$SubTestName - MerchantCenterAccounts empty list correctly returned." );
}

sub test_get_merchant_center_accounts_failure {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedShoppingConfig = $self->_createGoogleSmartShoppingConfig();
    my $AlreadyExistingConfig = 1;
    my $MockedShop = $self->_createShop($MockedShoppingConfig, $AlreadyExistingConfig);
    my $SUT = $self->_createSUT({ 'Shop' => $MockedShop, 'DoGetError' => GENERIC_ERROR_MESSAGE });
    my $aResponse;
    throws_ok(
        sub { $aResponse = $SUT->getMerchantCenterAccounts(); },
        qr/fail/,
        "$SubTestName - getMerchantCenterAccounts should not catch generic error"
    );
}

sub test_post_create_merchant_center_subaccount_success {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hConfigParams = {
        'MerchantCenterAccountID' => MERCHANT_CENTER_ID,
        'AccountEmail'            => ACCOUNT_EMAIL,
    };
    my $MockedShoppingConfig = $self->_createGoogleSmartShoppingConfig($hConfigParams);
    my $MockedShop = $self->_createShop($MockedShoppingConfig, 1);
    my $DoPostResponse = {
        'adultContent' => bless( do{\(my $o = 0)}, 'JSON::XS::Boolean' ),
        'emails' => [
            ACCOUNT_EMAIL
        ],
        'id' => MERCHANT_CENTER_ID,
        'name' => 'ShopNameOrAlias',
        'websiteUrl' => STOREFRONT_WEBSITE_URL
    };
    my $SUT = $self->_createSUT({ 'Shop' => $MockedShop, 'DoPostResponse' => $DoPostResponse });
    $SUT->{'GoogleSmartShopping_AdminRefreshToken'} = ADMIN_REFRESH_TOKEN;
    $SUT->{'GoogleSmartShopping_MerchantCenterMultiClientAccountID'} = MULTI_CLIENT_ACCOUNT_ID;
    $SUT->{'GoogleSmartShoppingConfig'} = $MockedShoppingConfig;
    my $AdultContent = 1;
    my $hResult;
    lives_ok(
        sub { $hResult = $SUT->createMerchantCenterSubAccount($AdultContent); },
        "$SubTestName - createMerchantCenterSubAccount can be called without errors."
    );
    my $aParams = GetParametersFromMockifyCall($SUT, 'doPost');
    my $URL = $aParams->[0];
    my $hHeaders = $aParams->[1];
    my $hPayload = $aParams->[2];
    is( $URL, EXPECTED_MERCHANT_CENTER_SUBACCOUNT_EPAGESJ_URL, "$SubTestName - verify that the correct epagesj endpoint is called." );
    is_deeply( $hHeaders, {'AdminToken' => ADMIN_ACCESS_TOKEN }, "$SubTestName - verify that the AdminToken header is passed." );
    is_deeply(
        $hPayload,
        {
            'adultContent' => 'true',
            'emails'       => [ ACCOUNT_EMAIL ],
            'name'         => 'ShopNameOrAlias',
            'websiteUrl'   => STOREFRONT_WEBSITE_URL,
        },
        "$SubTestName - verify that the correct payload is passed."
    );
    is( $hResult, $DoPostResponse, "$SubTestName - account correctly returned." );
    is( $SUT->{'GoogleSmartShoppingConfig'}->get('MerchantCenterAccountID'), MERCHANT_CENTER_ID, "$SubTestName - AccountID was inserted in Config." );
    is( $SUT->{'GoogleSmartShoppingConfig'}->get('MerchantCenterAccountName'), 'ShopNameOrAlias', "$SubTestName - Account Name was inserted in Config." );
    is( $SUT->{'GoogleSmartShoppingConfig'}->get('IsMerchantCenterSubAccount'), 1, "$SubTestName - IsMerchantCenterSubAccount was inserted in Config." );
    is( $SUT->{'GoogleSmartShoppingConfig'}->get('MerchantCenterMultiClientAccountID'), MULTI_CLIENT_ACCOUNT_ID, "$SubTestName - MerchantCenterMultiClientAccountID was inserted in Config." );
    ok( defined($SUT->{'GoogleSmartShoppingConfig'}->get('MerchantCenterSubAccountCreationTime')), "$SubTestName - SubAccountCreationTime was inserted in Config." );
}

sub test_post_create_merchant_center_subaccount_failure {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedShoppingConfig = $self->_createGoogleSmartShoppingConfig();
    my $MockedShop = $self->_createShop($MockedShoppingConfig, 1);
    my $SUT = $self->_createSUT({ 'Shop' => $MockedShop, 'DoPostError' => GENERIC_ERROR_MESSAGE });
    $SUT->{'GoogleSmartShopping_AdminRefreshToken'} = ADMIN_REFRESH_TOKEN;
    $SUT->{'GoogleSmartShopping_MerchantCenterMultiClientAccountID'} = MULTI_CLIENT_ACCOUNT_ID;
    $SUT->{'GoogleSmartShoppingConfig'} = $MockedShoppingConfig;
    my $hResult;
    throws_ok(
        sub { $hResult = $SUT->createMerchantCenterSubAccount('ConnectedGoogleAccount@Email', 1); },
        qr/fail/,
        "$SubTestName - createMerchantCenterSubAccount does not catch thrown error."
    );
}

sub test_delete_merchant_center_subaccount_success {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedShoppingConfig = $self->_createGoogleSmartShoppingConfig();
    my $MockedShop = $self->_createShop($MockedShoppingConfig, 1);
    my $SUT = $self->_createSUT({ 'Shop' => $MockedShop, 'DoDeleteResponse' => {} });
    $SUT->{'GoogleSmartShopping_AdminRefreshToken'} = ADMIN_REFRESH_TOKEN;
    $SUT->{'GoogleSmartShopping_MerchantCenterMultiClientAccountID'} = MULTI_CLIENT_ACCOUNT_ID;
    $SUT->{'GoogleSmartShoppingConfig'} = $MockedShoppingConfig;
    lives_ok(
        sub { $SUT->deleteMerchantCenterSubAccount(MERCHANT_CENTER_ID); },
        "$SubTestName - deleteMerchantCenterSubAccount can be called without errors."
    );
    my $aParams = GetParametersFromMockifyCall($SUT, 'doDelete');
    is_deeply(
        $aParams,
        [
            EPAGESJ_BASE_URI.'accounts/'.MULTI_CLIENT_ACCOUNT_ID.'/sub-accounts/'.MERCHANT_CENTER_ID,
            {'AdminToken' => ADMIN_ACCESS_TOKEN}
        ],
        "$SubTestName - Invalid token error was thrown."
    );
}

sub test_delete_merchant_center_subaccount_failure {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedShoppingConfig = $self->_createGoogleSmartShoppingConfig();
    my $MockedShop = $self->_createShop($MockedShoppingConfig, 1);
    my $SUT = $self->_createSUT({ 'Shop' => $MockedShop, 'DoDeleteError' => GENERIC_ERROR_MESSAGE });
    $SUT->{'GoogleSmartShopping_AdminRefreshToken'} = ADMIN_REFRESH_TOKEN;
    $SUT->{'GoogleSmartShopping_MerchantCenterMultiClientAccountID'} = MULTI_CLIENT_ACCOUNT_ID;
    $SUT->{'GoogleSmartShoppingConfig'} = $MockedShoppingConfig;
    throws_ok(
        sub { $SUT->deleteMerchantCenterSubAccount(MERCHANT_CENTER_ID); },
        qr/fail/,
        "$SubTestName - deleteMerchantCenterSubAccount does not catch a thrown error."
    );
}

sub test_post_claim_website_success_no_mca {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        MerchantCenterAccountID => MERCHANT_CENTER_ID,
        IsMerchantCenterSubAccount => 0,
    });
    my $MockedShop = $self->_createShop($MockedShoppingConfig, 1);
    my $SUT = $self->_createSUT({ 'Shop' => $MockedShop, 'DoPostResponse' => {} });
    $SUT->{'GoogleSmartShopping_AdminRefreshToken'} = ADMIN_REFRESH_TOKEN;
    $SUT->{'GoogleSmartShopping_MerchantCenterMultiClientAccountID'} = MULTI_CLIENT_ACCOUNT_ID;
    $SUT->{'GoogleSmartShoppingConfig'} = $MockedShoppingConfig;
    lives_ok(
        sub { $SUT->claimWebsite(); },
        "$SubTestName - claimWebsite can be called without errors."
    );
    ok(
        WasCalled($SUT, 'doPost'),
        $SubTestName . ' - Verify that doPost was called.'
    );
    my $aPostParams = GetParametersFromMockifyCall($SUT, 'doPost');
    is_deeply(
        $aPostParams,
        [
            EPAGESJ_BASE_URI.'accounts/claim-website',
            {},
            {
                merchantAccountId => MERCHANT_CENTER_ID,
                managerAccountId => MERCHANT_CENTER_ID,
            }
        ],
        "$SubTestName - Verify doPost claimWebsite call correctly the epagesj endpoint."
    );
}

sub test_post_claim_website_success_epages_mca {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        MerchantCenterAccountID => MERCHANT_CENTER_ID,
        MerchantCenterMultiClientAccountID => MULTI_CLIENT_ACCOUNT_ID,
        IsMerchantCenterSubAccount => 1,
    });
    my $MockedShop = $self->_createShop($MockedShoppingConfig, 1);
    my $SUT = $self->_createSUT({ 'Shop' => $MockedShop, 'DoPostResponse' => {} });
    $SUT->{'GoogleSmartShopping_AdminRefreshToken'} = ADMIN_REFRESH_TOKEN;
    $SUT->{'GoogleSmartShopping_MerchantCenterMultiClientAccountID'} = MULTI_CLIENT_ACCOUNT_ID;
    $SUT->{'GoogleSmartShoppingConfig'} = $MockedShoppingConfig;
    lives_ok(
        sub { $SUT->claimWebsite(); },
        "$SubTestName - claimWebsite can be called without errors."
    );
    ok(
        WasCalled($SUT, 'doPost'),
        $SubTestName . ' - Verify that doPost was called.'
    );
    my $aPostParams = GetParametersFromMockifyCall($SUT, 'doPost');
    is_deeply(
        $aPostParams,
        [
            EPAGESJ_BASE_URI.'accounts/claim-website',
            {},
            {
                merchantAccountId => MERCHANT_CENTER_ID,
                managerAccountId => MULTI_CLIENT_ACCOUNT_ID,
            }
        ],
        "$SubTestName - Verify doPost claimWebsite with epagesMultiaccountId call correctly the epagesj endpoint."
    );
}

sub test_post_claim_website_success_external_mca {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        MerchantCenterAccountID => MERCHANT_CENTER_ID,
        MerchantCenterMultiClientAccountID => 5353572727,
        IsMerchantCenterSubAccount => 0,
    });
    my $MockedShop = $self->_createShop($MockedShoppingConfig, 1);
    my $SUT = $self->_createSUT({ 'Shop' => $MockedShop, 'DoPostResponse' => {} });
    $SUT->{'GoogleSmartShopping_AdminRefreshToken'} = ADMIN_REFRESH_TOKEN;
    $SUT->{'GoogleSmartShopping_MerchantCenterMultiClientAccountID'} = MULTI_CLIENT_ACCOUNT_ID;
    $SUT->{'GoogleSmartShoppingConfig'} = $MockedShoppingConfig;
    lives_ok(
        sub { $SUT->claimWebsite(); },
        "$SubTestName - claimWebsite can be called without errors."
    );
    ok(
        WasCalled($SUT, 'doPost'),
        $SubTestName . ' - Verify that doPost was called.'
    );
    my $aPostParams = GetParametersFromMockifyCall($SUT, 'doPost');
    is_deeply(
        $aPostParams,
        [
            EPAGESJ_BASE_URI.'accounts/claim-website',
            {},
            {
                merchantAccountId => MERCHANT_CENTER_ID,
                managerAccountId => 5353572727,
            }
        ],
        "$SubTestName - Verify doPost claimWebsite with outside MultiAccountId call correctly the epagesj endpoint."
    );
}

sub test_post_claim_website_failure {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        MerchantCenterAccountID => MERCHANT_CENTER_ID,
        IsMerchantCenterSubAccount => 0,
    });
    my $MockedShop = $self->_createShop($MockedShoppingConfig, 1);
    my $SUT = $self->_createSUT({ 'Shop' => $MockedShop, 'DoPostError' => GENERIC_ERROR_MESSAGE });
    $SUT->{'GoogleSmartShopping_AdminRefreshToken'} = ADMIN_REFRESH_TOKEN;
    $SUT->{'GoogleSmartShopping_MerchantCenterMultiClientAccountID'} = MULTI_CLIENT_ACCOUNT_ID;
    $SUT->{'GoogleSmartShoppingConfig'} = $MockedShoppingConfig;
    throws_ok(
        sub { $SUT->claimWebsite(); },
        qr/fail/,
        "$SubTestName - claimWebsite does not catch thrown error from ApiClient doPost."
    );
}

sub test_get_site_verification_token_success {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $SUT = $self->_createSUT({ 'DoPostResponse' => EXPECTED_SITE_VERIFICATION_TOKEN_RESPONSE });
    my $hSiteVerificationToken;
    lives_ok(
        sub { $hSiteVerificationToken = $SUT->getSiteVerificationToken(STOREFRONT_WEBSITE_URL); },
        "$SubTestName - getSiteVerificationToken can be called without errors."
    );
    my $aGetParams = GetParametersFromMockifyCall($SUT, 'doPost');
    is( @$aGetParams[0], EXPECTED_SITE_VERIFICATION_TOKEN_EPAGESJ_URL,
        "$SubTestName - Verify doPost calls correctly the epagesj site verification endpoint." );
    is( $hSiteVerificationToken->{'token'}, GOOGLE_SITE_VERIFICATION_TOKEN, "$SubTestName -  site verification token correctly returned." );
}

sub test_get_site_verification_token_failure {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $SUT = $self->_createSUT({ 'DoPostError' => GENERIC_ERROR_MESSAGE });
    my $hSiteVerificationToken;
    throws_ok(
        sub { $hSiteVerificationToken = $SUT->getSiteVerificationToken(STOREFRONT_WEBSITE_URL); },
        qr/fail/,
        "$SubTestName - getSiteVerificationToken doesn't catch error thrown by DoPost."
    );
}

sub test_trigger_site_verification_success {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $SUT = $self->_createSUT({ 'DoPostResponse' => EXPECTED_SITE_VERIFICATION_RESPONSE });
    my $hResponse;
    lives_ok(
        sub { $hResponse = $SUT->triggerSiteVerification(STOREFRONT_WEBSITE_URL); },
        "$SubTestName - triggerSiteVerification can be called without errors."
    );
    my $aGetParams = GetParametersFromMockifyCall($SUT, 'doPost');
    is( @$aGetParams[0], EXPECTED_SITE_VERIFICATION_EPAGESJ_URL,
        "$SubTestName - Verify doPost calls correctly the epagesj site verification endpoint." );
    is_deeply( $hResponse, {}, "$SubTestName - Verify that site verification returns empty response." );
}

sub test_trigger_site_verification_failure {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $SUT = $self->_createSUT({ 'DoPostError' => GENERIC_ERROR_MESSAGE });
    my $hResponse;
    throws_ok(
        sub { $hResponse = $SUT->triggerSiteVerification(STOREFRONT_WEBSITE_URL); },
        qr/fail/,
        "$SubTestName - triggerSiteVerification can be called without errors."
    );
}

sub test_get_merchant_center_account_statuses_subaccount_success {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        MerchantCenterAccountID => MERCHANT_CENTER_ID,
        MerchantCenterMultiClientAccountID => 389123456,
        IsMerchantCenterSubAccount => 0,
    });

    my $MockedShop = $self->_createShop($MockedShoppingConfig, 1);

    my $SUT = $self->_createSUT({ 'Shop' => $MockedShop, 'DoGetResponse' => EXPECTED_MERCHANT_CENTER_ACCOUNT_STATUSES_EPAGESJ_RESPONSE });
    $SUT->{'GoogleSmartShoppingConfig'} = $MockedShoppingConfig;

    my $hResponse;
    lives_ok(
        sub { $hResponse = $SUT->getMerchantCenterAccountStatuses(); },
        "$SubTestName - Varify that getMerchantCenterAccounts can be called without errors."
    );
    ok(WasCalled($SUT, 'doGet'), $SubTestName . ' - Verify that doGet was called.');
    ok(WasCalled($SUT, '_createAdminToken'), $SubTestName . ' - Verify that _createAdminToken was called.');
    my $ExpectedAccountStatus = {
        "documentation" => "https://support.google.com/merchants/answer/6159060",
        "id" => "missing_ad_words_link",
        "severity" => "error",
        "title" => "No Google Ads account linked"
    };
    my $aGetParams = GetParametersFromMockifyCall($SUT, 'doGet');
    is_deeply(
        $aGetParams,
        [ EXPECTED_MERCHANT_CENTER_SUBACCOUNT_STATUSES_EPAGESJ_URL, {'AdminToken' => ADMIN_ACCESS_TOKEN} ],
        "$SubTestName - Verify doGet correctly calls the epagesj mc account statuses endpoint with AdminToken header included."
    );
    is_deeply( $hResponse->{'accountLevelIssues'}[1], $ExpectedAccountStatus, "$SubTestName - Varify that MerchantCenter account statuses correctly returned." );
}

sub test_get_merchant_center_account_statuses_standalone_merchant_account_success {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        MerchantCenterAccountID => MERCHANT_CENTER_ID,
        MerchantCenterMultiClientAccountID => 458599058,
        IsMerchantCenterSubAccount => 0,
    });

    my $MockedShop = $self->_createShop($MockedShoppingConfig, 1);

    my $SUT = $self->_createSUT({ 'Shop' => $MockedShop, 'DoGetResponse' => EXPECTED_MERCHANT_CENTER_ACCOUNT_STATUSES_EPAGESJ_RESPONSE });
    $SUT->{'GoogleSmartShoppingConfig'} = $MockedShoppingConfig;

    my $hResponse;
    lives_ok(
        sub { $hResponse = $SUT->getMerchantCenterAccountStatuses(); },
        "$SubTestName - Varify that getMerchantCenterAccounts can be called without errors."
    );
    ok(WasCalled($SUT, 'doGet'), $SubTestName . ' - Verify that doGet was called.');
    ok(!WasCalled($SUT, '_createAdminToken'), $SubTestName . ' - Verify that _createAdminToken was NOT called.');
    my $ExpectedAccountStatus = {
        "documentation" => "https://support.google.com/merchants/answer/6159060",
        "id" => "missing_ad_words_link",
        "severity" => "error",
        "title" => "No Google Ads account linked"
    };
    my $aGetParams = GetParametersFromMockifyCall($SUT, 'doGet');
    is( @$aGetParams[0], EXPECTED_MERCHANT_CENTER_STANDALONE_ACCOUNT_STATUSES_EPAGESJ_URL, "$SubTestName - Verify doGet call correctly the epagesj mc account statuses endpoint." );
    is_deeply( $hResponse->{'accountLevelIssues'}[1], $ExpectedAccountStatus, "$SubTestName - Verify that MerchantCenter account statuses correctly returned." );
}

sub test_get_merchant_center_account_statuses_failure {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        MerchantCenterAccountID => MERCHANT_CENTER_ID,
        MerchantCenterMultiClientAccountID => 458599058,
        IsMerchantCenterSubAccount => 0,
    });

    my $MockedShop = $self->_createShop($MockedShoppingConfig, 1);

    my $SUT = $self->_createSUT({ 'Shop' => $MockedShop,  'DoGetError' => GENERIC_ERROR_MESSAGE });
    $SUT->{'GoogleSmartShoppingConfig'} = $MockedShoppingConfig;

    my $hResponse;
    throws_ok(
        sub { $hResponse = $SUT->getMerchantCenterAccountStatuses(); },
        qr/fail/,
        "$SubTestName - Verify that getMerchantCenterAccountStatuses does not catch error thrown in DoGet."
    );
}

sub test_get_rated_product_statuses_no_products {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        MerchantCenterAccountID => MERCHANT_CENTER_ID,
        MerchantCenterMultiClientAccountID => 458599058,
        IsMerchantCenterSubAccount => 0,
    });
    my $MockedShop = $self->_createShop($MockedShoppingConfig, 1);

    my $SUT = $self->_createSUT({ 'Shop' => $MockedShop,  'getMerchantCenterAccountStatusesResponse' => EXPECTED_MERCHANT_CENTER_ACCOUNT_STATUSES_EPAGESJ_RESPONSE });
    my $hResponse;

    lives_ok(
        sub { $hResponse = $SUT->getRatedProductsStatistics(); },
        "$SubTestName - Varify that getRatedProductsStatistics can be called without errors."
    );

    my $ExpectedProductsStatistics = {
        TooManyPending      => 0,
        TooManyDisapproved  => 0,
        IsNotEnoughActive     => 0,
        active              => 0,
        pending             => 0,
        disapproved         => 0,
        total               => 0,
    };

    is_deeply($hResponse,$ExpectedProductsStatistics,"$SubTestName - Verify DataQuality is rated low");

}

sub test_get_rated_product_statuses_empty_product_array {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];
    my $SUT = $self->_createSUT({ 'getMerchantCenterAccountStatusesResponse' => SUCCESSFUL_MERCHANT_CENTER_ACCOUNT_STATUSES_EPAGESJ_RESPONSE_HAS_EMPTY_PRODUCTS_ARRAY });
    my $hResponse;

    lives_ok(
        sub { $hResponse = $SUT->getRatedProductsStatistics(); },
        "$SubTestName - Varify that getRatedProductsStatistics can be called without errors."
    );

    my $ExpectedProductsStatistics = {
        total => 0,
    };
    is_deeply($hResponse->{'total'},$ExpectedProductsStatistics->{'total'},"$SubTestName - Verify Products were recognized as NOT uploaded");

}

sub test_get_rated_product_statuses_no_products_in_sales_country {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $aProducts = [
        {
            "channel"         => "online",
            "country"         => "AT",
            "destination"     => "Shopping",
            "itemLevelIssues" => {
                "code"          => "policy_enforcement_account_disapproval",
                "description"   => "Suspended account for policy violation",
                "detail"        => "Remove products that violate our policies, or request a manual review",
                "documentation" => "https://support.google.com/merchants/answer/2948694",
                "numItems"      => 4,
                "resolution"    => "merchant_action",
                "servability"   => "disapproved"
            },
            "statistics"      => {
                "active"      => 0,
                "disapproved" => 0,
                "expiring"    => 4,
                "pending"     => 0
            }},
    ];

    my $MockedShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        MerchantCenterAccountID => MERCHANT_CENTER_ID,
        MerchantCenterMultiClientAccountID => 458599058,
        IsMerchantCenterSubAccount => 0,
    });
    my $MockedShop = $self->_createShop($MockedShoppingConfig, 1);

    my $SUT = $self->_createSUT({ 'Shop' => $MockedShop,  'getMerchantCenterAccountStatusesResponse' => $self->_createAccountStatusesResponse($aProducts) });
    my $hResponse = undef;

    lives_ok(
        sub { $hResponse = $SUT->getRatedProductsStatistics(); },
        "$SubTestName - Verify that getRatedProductsStatistics can be called without errors."
    );

    ok(defined($hResponse), "$SubTestName - Verify response is defined");

    is($hResponse->{'total'},0, "$SubTestName - Verify that product quality check returns no Products uploaded");
}

sub test_get_rated_product_statuses_low_quality_disapproved {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $aProducts = [
        {
        "channel"         => "online",
        "country"         => "DE",
        "destination"     => "Shopping",
        "itemLevelIssues" => {
            "code"          => "policy_enforcement_account_disapproval",
            "description"   => "Suspended account for policy violation",
            "detail"        => "Remove products that violate our policies, or request a manual review",
            "documentation" => "https://support.google.com/merchants/answer/2948694",
            "numItems"      => 5,
            "resolution"    => "merchant_action",
            "servability"   => "disapproved"
        },
        "statistics"      => {
            "active"      => 2,
            "disapproved" => 5,
            "expiring"    => 0,
            "pending"     => 0
        }},
        {
        "channel"         => "online",
        "country"         => "AT",
        "destination"     => "Shopping",
        "itemLevelIssues" => {
            "code"          => "policy_enforcement_account_disapproval",
            "description"   => "Suspended account for policy violation",
            "detail"        => "Remove products that violate our policies, or request a manual review",
            "documentation" => "https://support.google.com/merchants/answer/2948694",
            "numItems"      => 4,
            "resolution"    => "merchant_action",
            "servability"   => "disapproved"
        },
        "statistics"      => {
            "active"      => 0,
            "disapproved" => 0,
            "expiring"    => 4,
            "pending"     => 0
        }},
    ];

    my $MockedShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        MerchantCenterAccountID => MERCHANT_CENTER_ID,
        MerchantCenterMultiClientAccountID => 458599058,
        IsMerchantCenterSubAccount => 0,
    });
    my $MockedShop = $self->_createShop($MockedShoppingConfig, 1);

    my $SUT = $self->_createSUT({ 'Shop' => $MockedShop,  'getMerchantCenterAccountStatusesResponse' => $self->_createAccountStatusesResponse($aProducts) });
    my $hResponse = undef;

    lives_ok(
        sub { $hResponse = $SUT->getRatedProductsStatistics(); },
        "$SubTestName - Verify that getRatedProductsStatistics can be called without errors."
    );

    ok(defined($hResponse), "$SubTestName - Verify response is defined");

    is($hResponse->{'TooManyDisapproved'},1, "$SubTestName - Verify that product quality check returns too many disapproved");
}

sub test_get_rated_product_statuses_low_quality_pending {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $aProducts = [
        {
            "channel"         => "online",
            "country"         => "DE",
            "destination"     => "Shopping",
            "itemLevelIssues" => {
                "code"          => "policy_enforcement_account_disapproval",
                "description"   => "Suspended account for policy violation",
                "detail"        => "Remove products that violate our policies, or request a manual review",
                "documentation" => "https://support.google.com/merchants/answer/2948694",
                "numItems"      => 4,
                "resolution"    => "merchant_action",
                "servability"   => "disapproved"
            },
            "statistics"      => {
                "active"      => 0,
                "disapproved" => 0,
                "expiring"    => 3,
                "pending"     => 4
            }},
    ];
    my $MockedShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        MerchantCenterAccountID => MERCHANT_CENTER_ID,
        MerchantCenterMultiClientAccountID => 458599058,
        IsMerchantCenterSubAccount => 0,
    });
    my $MockedShop = $self->_createShop($MockedShoppingConfig, 1);

    my $SUT = $self->_createSUT({ 'Shop' => $MockedShop,  'getMerchantCenterAccountStatusesResponse' => $self->_createAccountStatusesResponse($aProducts) });    my $hResponse = undef;

    lives_ok(
        sub { $hResponse = $SUT->getRatedProductsStatistics(); },
        "$SubTestName - Verify that getRatedProductsStatistics can be called without errors."
    );

    ok(defined($hResponse), "$SubTestName - Verify response is defined");

    is($hResponse->{'TooManyPending'}, 1, "$SubTestName - Verify that product quality check returns too many pending");
}

sub test_get_rated_product_statuses_low_quality_1st_case_9_from_20_active_shows_a_warning {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $aProducts = [
        {
            "channel"         => "online",
            "country"         => "DE",
            "destination"     => "Shopping",
            "itemLevelIssues" => {
                "code"          => "policy_enforcement_account_disapproval",
                "description"   => "Suspended account for policy violation",
                "detail"        => "Remove products that violate our policies, or request a manual review",
                "documentation" => "https://support.google.com/merchants/answer/2948694",
                "numItems"      => 4,
                "resolution"    => "merchant_action",
                "servability"   => "disapproved"
            },
            # less_10_active_less_half_active, 9/20 active --> warning
            "statistics"      => {
                "active"      => 9,
                "disapproved" => 5,
                "expiring"    => 5,
                "pending"     => 1
            }},
    ];
    my $MockedShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        MerchantCenterAccountID => MERCHANT_CENTER_ID,
        MerchantCenterMultiClientAccountID => 458599058,
        IsMerchantCenterSubAccount => 0,
    });
    my $MockedShop = $self->_createShop($MockedShoppingConfig, 1);

    my $SUT = $self->_createSUT({ 'Shop' => $MockedShop,  'getMerchantCenterAccountStatusesResponse' => $self->_createAccountStatusesResponse($aProducts) });

    my $hResponse = undef;
    lives_ok(
        sub { $hResponse = $SUT->getRatedProductsStatistics(); },
        "$SubTestName - Verify that getRatedProductsStatistics can be called without errors."
    );

    ok(defined($hResponse), "$SubTestName - Verify response is defined");

    is($hResponse->{'IsNotEnoughActive'}, 1, "$SubTestName - Verify that when less than 10 active and less than half active shows a warning");
}

sub test_get_rated_product_statuses_low_quality_2nd_case_9_from_18_active_mo_warning {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $aProducts = [
        {
            "channel"         => "online",
            "country"         => "DE",
            "destination"     => "Shopping",
            "itemLevelIssues" => {
                "code"          => "policy_enforcement_account_disapproval",
                "description"   => "Suspended account for policy violation",
                "detail"        => "Remove products that violate our policies, or request a manual review",
                "documentation" => "https://support.google.com/merchants/answer/2948694",
                "numItems"      => 4,
                "resolution"    => "merchant_action",
                "servability"   => "disapproved"
            },
            # less_10_active_half_or_more_active, 9/18 active --> no warning
            "statistics"      => {
                "active"      => 9,
                "disapproved" => 5,
                "expiring"    => 3,
                "pending"     => 1
            }},
    ];
    my $MockedShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        MerchantCenterAccountID => MERCHANT_CENTER_ID,
        MerchantCenterMultiClientAccountID => 458599058,
        IsMerchantCenterSubAccount => 0,
    });
    my $MockedShop = $self->_createShop($MockedShoppingConfig, 1);

    my $SUT = $self->_createSUT({ 'Shop' => $MockedShop,  'getMerchantCenterAccountStatusesResponse' => $self->_createAccountStatusesResponse($aProducts) });

    my $hResponse = undef;
    lives_ok(
        sub { $hResponse = $SUT->getRatedProductsStatistics(); },
        "$SubTestName - Verify that getRatedProductsStatistics can be called without errors."
    );

    ok(defined($hResponse), "$SubTestName - Verify response is defined");

    is($hResponse->{'IsNotEnoughActive'}, 0, "$SubTestName - Verify that when less than 10 active and half or more active there is NO warning");
}

sub test_get_rated_product_statuses_low_quality_3rd_case_19_from_100_active_shows_a_warning {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $aProducts = [
        {
            "channel"         => "online",
            "country"         => "DE",
            "destination"     => "Shopping",
            "itemLevelIssues" => {
                "code"          => "policy_enforcement_account_disapproval",
                "description"   => "Suspended account for policy violation",
                "detail"        => "Remove products that violate our policies, or request a manual review",
                "documentation" => "https://support.google.com/merchants/answer/2948694",
                "numItems"      => 4,
                "resolution"    => "merchant_action",
                "servability"   => "disapproved"
            },
            # more_10_active_less_20pc_active, 19/100 active --> warning
            "statistics"      => {
                "active"      => 19,
                "disapproved" => 10,
                "expiring"    => 10,
                "pending"     => 61
            }},
    ];
    my $MockedShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        MerchantCenterAccountID => MERCHANT_CENTER_ID,
        MerchantCenterMultiClientAccountID => 458599058,
        IsMerchantCenterSubAccount => 0,
    });
    my $MockedShop = $self->_createShop($MockedShoppingConfig, 1);

    my $SUT = $self->_createSUT({ 'Shop' => $MockedShop,  'getMerchantCenterAccountStatusesResponse' => $self->_createAccountStatusesResponse($aProducts) });

    my $hResponse = undef;
    lives_ok(
        sub { $hResponse = $SUT->getRatedProductsStatistics(); },
        "$SubTestName - Verify that getRatedProductsStatistics can be called without errors."
    );

    ok(defined($hResponse), "$SubTestName - Verify response is defined");

    is($hResponse->{'IsNotEnoughActive'}, 1, "$SubTestName - Verify that when >10 active and <20% active then shows a warning");
}

sub test_get_rated_product_statuses_low_quality_4th_case_20_from_100_active_mo_warning {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $aProducts = [
        {
            "channel"         => "online",
            "country"         => "DE",
            "destination"     => "Shopping",
            "itemLevelIssues" => {
                "code"          => "policy_enforcement_account_disapproval",
                "description"   => "Suspended account for policy violation",
                "detail"        => "Remove products that violate our policies, or request a manual review",
                "documentation" => "https://support.google.com/merchants/answer/2948694",
                "numItems"      => 4,
                "resolution"    => "merchant_action",
                "servability"   => "disapproved"
            },
            # 10_or_more_active_more_20pc_active, 20/100 active --> no warning
            "statistics"      => {
                "active"      => 20,
                "disapproved" => 10,
                "expiring"    => 10,
                "pending"     => 60
            }},
    ];
    my $MockedShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        MerchantCenterAccountID => MERCHANT_CENTER_ID,
        MerchantCenterMultiClientAccountID => 458599058,
        IsMerchantCenterSubAccount => 0,
    });
    my $MockedShop = $self->_createShop($MockedShoppingConfig, 1);

    my $SUT = $self->_createSUT({ 'Shop' => $MockedShop,  'getMerchantCenterAccountStatusesResponse' => $self->_createAccountStatusesResponse($aProducts) });

    my $hResponse = undef;
    lives_ok(
        sub { $hResponse = $SUT->getRatedProductsStatistics(); },
        "$SubTestName - Verify that getRatedProductsStatistics can be called without errors."
    );

    ok(defined($hResponse), "$SubTestName - Verify response is defined");

    is($hResponse->{'IsNotEnoughActive'}, 0, "$SubTestName - Verify that when >10 active and >=20% active then there is NO warning");
}

sub test_request_merchant_center_phone_verification_success {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedShoppingConfig = $self->_createGoogleSmartShoppingConfig({
            'MerchantCenterAccountID' => MERCHANT_CENTER_ID,
            'MerchantCenterMultiClientAccountID' => MULTI_CLIENT_ACCOUNT_ID
    });
    my $MockedShop = $self->_createShop($MockedShoppingConfig, 1);
    my $SUT = $self->_createSUT({ 'Shop' => $MockedShop, 'DoPostResponse' => EXPECTED_PHONE_NUMBER_VERIFICATION_RESPONSE });
    $SUT->{'GoogleSmartShoppingConfig'} = $MockedShoppingConfig;
    my $hResponse;
    lives_ok(
        sub { $hResponse = $SUT->requestMerchantCenterPhoneVerification(PHONE_NUMBER_VERIFICATION_REQUEST); },
        "$SubTestName - requestMerchantCenterPhoneVerification can be called without errors."
    );
    my $aGetParams = GetParametersFromMockifyCall($SUT, 'doPost');
    is(
        @$aGetParams[0],
        EXPECTED_PHONE_NUMBER_VERIFICATION_EPAGESJ_URL,
        "$SubTestName - Verify doPost calls correctly the epagesj phone number verification endpoint."
    );
    is_deeply(
        $hResponse,
        EXPECTED_PHONE_NUMBER_VERIFICATION_RESPONSE,
        "$SubTestName - Verify that phone number verification returns verification id response."
    );
}

sub test_confirm_merchant_center_phone_verification_success {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedShoppingConfig = $self->_createGoogleSmartShoppingConfig({
            'MerchantCenterAccountID' => MERCHANT_CENTER_ID,
            'MerchantCenterMultiClientAccountID' => MULTI_CLIENT_ACCOUNT_ID
    });
    my $MockedShop = $self->_createShop($MockedShoppingConfig, 1);
    my $SUT = $self->_createSUT({ 'Shop' => $MockedShop, 'DoPutResponse' => {} });
    $SUT->{'GoogleSmartShoppingConfig'} = $MockedShoppingConfig;
    my $hResponse;
    lives_ok(
        sub {
            $hResponse = $SUT->confirmMerchantCenterPhoneVerification(
                PHONE_NUMBER_VERIFICATION_ID, PHONE_NUMBER_VERIFICATION_CONFIRMATION_REQUEST
            );
        },
        "$SubTestName - confirmMerchantCenterPhoneVerification can be called without errors."
    );
    my $aGetParams = GetParametersFromMockifyCall($SUT, 'doPut');
    is(
        @$aGetParams[0],
        EXEPECTED_PHONE_NUMBER_VERIFICATION_CONFIRMATION_EPAGESJ_URL,
        "$SubTestName - Verify doPut calls correctly the epagesj phone number verification confirmation endpoint."
    );
}

sub test_get_products_statuses_success {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        'MerchantCenterAccountID' => MERCHANT_CENTER_ID,
    });
    my $MockedShop = $self->_createShop($MockedShoppingConfig, 1);
    my $SUT = $self->_createSUT({ 'Shop' => $MockedShop, 'DoGetResponse' => EXPECTED_PRODUCTS_STATUSES_RESPONSE });
    $SUT->{'GoogleSmartShoppingConfig'} = $MockedShoppingConfig;
    my $Page = 1;
    my $ResultsPerPage = 20;
    my $aResponse = undef;

    lives_ok(
        sub {
            $aResponse = $SUT->productsStatuses($Page, $ResultsPerPage);
        },
        "$SubTestName - productsStatuses can be called without errors."
    );
    is_deeply(
        $aResponse,
        EXPECTED_PRODUCTS_STATUSES_RESPONSE,
        "$SubTestName - Verify method returns product statuses."
    );
}

sub test_list_products {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        'MerchantCenterAccountID' => MERCHANT_CENTER_ID,
    });
    my $MockedShop = $self->_createShop($MockedShoppingConfig, 1);
    my $SUT = $self->_createSUT({ 'Shop' => $MockedShop, 'DoGetResponse' => EXPECTED_PRODUCTS_LIST_RESPONSE });
    $SUT->{'GoogleSmartShoppingConfig'} = $MockedShoppingConfig;
    my $aResponse = undef;

    lives_ok(
        sub {
            $aResponse = $SUT->getFirstProduct();
        },
        "$SubTestName - listProducts can be called without errors."
    );

    ok(defined($aResponse), "$SubTestName - Verify response is defined");
    is($aResponse, EXPECTED_PRODUCTS_LIST_RESPONSE->{'items'}->[0],
        "$SubTestName - Verify method returns first entry in array from field items in aResponse.");

    is($aResponse->{price}, EXPECTED_PRODUCTS_LIST_RESPONSE->{'items'}->[0]->{'price'},
        "$SubTestName - Verify method return contains field price.")
}


sub test_callProductsBatch {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hAwesomePayload = {
        'intro' => 'awesome',
        'body' => 'wonder',
        'conclusion' => 'egregious',
    };
    my $MockedShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        'MerchantCenterAccountID' => MERCHANT_CENTER_ID,
    });
    my $MockedShop = $self->_createShop($MockedShoppingConfig, 1);

    my $SUT = $self->_createSUT({ 'Shop' => $MockedShop, 'DoPostResponse' => EXPECTED_MERCHANT_CENTER_ACCOUNTS_RESPONSE });

    $SUT->{'GoogleSmartShoppingConfig'} = $MockedShoppingConfig;

    my $aResponse;
    lives_ok(
        sub { $aResponse = $SUT->callProductsBatch($hAwesomePayload); },
        "$SubTestName - Verify that callProductsBatch can be called without errors."
    );

    ok(
        WasCalled($SUT, 'doPost'),
        $SubTestName . ' - Verify that doPost was called.'
    );

    my $aGetParams = GetParametersFromMockifyCall($SUT, 'doPost');

    is( 
        @$aGetParams[0],
        sprintf(
            '%saccounts/%s/products/batch',
            EPAGESJ_BASE_URI,
            MERCHANT_CENTER_ID,
        ),
        "$SubTestName - Verify doPost call correctly the epagesj endpoint." );

    is( $aResponse, EXPECTED_MERCHANT_CENTER_ACCOUNTS_RESPONSE, "$SubTestName - Varify that callProductsBatch returns the response from doPost." );
}

sub test_delete_products {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        'MerchantCenterAccountID' => MERCHANT_CENTER_ID,
    });
    my $MockedShop = $self->_createShop($MockedShoppingConfig, 1);

    my $ArrayOfProductsMarkedForDeletion = [
        'online:de:DE:53-Rustbook_de_DE',
        'online:de:DE:54-Programmer-Socks_de_DE',
        'online:de:DE:56-Lenzetto_de_DE',
    ];

    my $hExpectedResponse = {
        'entries' => [
            {
                'batchId' => 1,
                'kind'    => 'content#productsCustomBatchResponseEntry'
            }
        ],
        'kind' => 'content#productsCustomBatchResponse'
    };


    my $SUT = $self->_createSUT({ 'Shop' => $MockedShop, 'expectedCallProductsBatchResponse' => $hExpectedResponse});

    my $aResponse = undef;

    lives_ok(
        sub {
            $aResponse = $SUT->deleteProducts($ArrayOfProductsMarkedForDeletion);
        },
        "$SubTestName - deleteProducts can be called without errors."
    );

    ok(
        WasCalled($SUT,'callProductsBatch'),
        "$SubTestName - callProductsBatch was called"
    );
}

#----------------------------------------------------------------------------------------
# Helpers
#----------------------------------------------------------------------------------------

sub _createSUT {
    my $self = shift;
    my ($hSUTParams) = @_;

    my $hServiceParams = { 'oauth_code' => AUTH_CODE, 'Shop' => $hSUTParams->{'Shop'} // undef };

    my $Mockify = Test::Mockify::Sut->new('DE_EPAGES::GoogleSmartShopping::API::GoogleApiService', [$hServiceParams]);

    $Mockify->spy('_createDeleteProductsPayload')->when(ArrayRef());

    $Mockify->mockImported('DE_EPAGES::Object::API::Factory', 'LoadRootObject')
        ->whenAny()
        ->thenReturn($self->_createSystem());

    $Mockify->mockImported('DE_EPAGES::GoogleSmartShopping::API::Tools', 'LogGoogleSmartShopping')
        ->whenAny()
        ->thenReturnUndef();

    if (defined($hSUTParams->{'UserAgent'})) {
        $Mockify->mockStatic('LWP::UserAgent::new')
            ->whenAny()
            ->thenReturn($hSUTParams->{'UserAgent'});
    }

    if (defined($hSUTParams->{'DoGetError'})) {
        $Mockify->mock('doGet')
            ->whenAny()
            ->thenThrowError($hSUTParams->{'DoGetError'});
    } elsif (defined($hSUTParams->{'DoGetResponse'})) {
        $Mockify->mock('doGet')
            ->whenAny()
            ->thenReturn($hSUTParams->{'DoGetResponse'});
    }

    if (defined($hSUTParams->{'DoPostError'})) {
        $Mockify->mock('doPost')
            ->whenAny()
            ->thenThrowError($hSUTParams->{'DoPostError'});
    } elsif (defined($hSUTParams->{'DoPostResponse'})) {
        $Mockify->mock('doPost')
            ->whenAny()
            ->thenReturn($hSUTParams->{'DoPostResponse'});
    }

    if (defined($hSUTParams->{'DoPutError'})) {
        $Mockify->mock('doPut')
            ->whenAny()
            ->thenThrowError($hSUTParams->{'DoPutError'});
    } elsif (defined($hSUTParams->{'DoPutResponse'})) {
        $Mockify->mock('doPut')
            ->whenAny()
            ->thenReturn($hSUTParams->{'DoPutResponse'});
    }

    if (defined($hSUTParams->{'DoDeleteError'})) {
        $Mockify->mock('doDelete')
            ->whenAny()
            ->thenThrowError($hSUTParams->{'DoDeleteError'});
    } elsif (defined($hSUTParams->{'DoDeleteResponse'})) {
        $Mockify->mock('doDelete')
            ->whenAny()
            ->thenReturn($hSUTParams->{'DoDeleteResponse'});
    }

    if (defined($hSUTParams->{'MockedError'})) {
        $Mockify->mockImported('DE_EPAGES::Core::API::Error', 'GetError')
            ->whenAny()
            ->thenReturn($hSUTParams->{'MockedError'});
    }
    if (defined($hSUTParams->{'getMerchantCenterAccountStatusesResponse'})) {
        $Mockify->mock('getMerchantCenterAccountStatuses')
            ->whenAny()
            ->thenReturn($hSUTParams->{'getMerchantCenterAccountStatusesResponse'});
    }
    if (defined($hSUTParams->{'listProductsResponse'})) {
        $Mockify->mock('listProducts')
            ->whenAny()
            ->thenReturn($hSUTParams->{'listProductsResponse'});
    }
    if (defined($hSUTParams->{'getFirstProductResponse'})) {
        $Mockify->mock('getFirstProduct')
            ->whenAny()
            ->thenReturn($hSUTParams->{'getFirstProductResponse'});
    }

    if (defined($hSUTParams->{'expectedCallProductsBatchResponse'})) {
        $Mockify->mock('callProductsBatch')
            ->when(HashRef())
            ->thenReturn($hSUTParams->{'expectedCallProductsBatchResponse'});
    }

    $Mockify->mock('GetAccessToken')
        ->when(String(ADMIN_REFRESH_TOKEN))
        ->thenReturn(ADMIN_ACCESS_TOKEN);

    $Mockify->mock('_createAdminToken')
        ->whenAny()
        ->thenReturn(ADMIN_ACCESS_TOKEN);

    $Mockify->mock('loadGoogleSettings')
        ->when()
        ->thenReturnUndef();

    return $Mockify->getMockObject();
}

sub _createShop {
    my $self = shift;
    my ($GoogleSmartShoppingConfig, $AlreadyExistingConfig) = @_;
    my $MockedAddress = $self->_createAddress();

    my $hAttributes = {
        'ID' => undef,
        'GoogleSmartShoppingConfig' => $GoogleSmartShoppingConfig,
        'Address'                   => $MockedAddress,
        'LanguageID'                => 1,
        'NameOrAlias'               => 'ShopNameOrAlias',
        'WebUrlUnitySSL'            => STOREFRONT_WEBSITE_URL,
    };
    my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);
    $MockifyShop->addMockWithReturnValueAndParameterCheck(
        'existsChild', $AlreadyExistingConfig, [ {'string' => 'GoogleSmartShoppingConfig'} ]
    );
    $MockifyShop->addMockWithReturnValueAndParameterCheck(
        'insertChild', $GoogleSmartShoppingConfig, [ {'string' => 'GoogleSmartShoppingConfig'}, 'hashref' ]
    );

    return $MockifyShop->getMockObject();
}

sub _createSystem {
    my $self = shift;

    my $hAttributes = {
        'GoogleSmartShopping_AppClientID' => APP_CLIENT_ID,
        'GoogleSmartShopping_AppClientSecret' => APP_CLIENT_SECRET,
        'GoogleSmartShopping_AuthTokenURL' => AUTH_TOKEN_URL,
        'GoogleSmartShopping_AppRedirectURL' => APP_REDIRECT_URL,
        'GoogleSmartShopping_GoogleAppURL' => GOOGLE_APP_URI,
        'GoogleSmartShopping_AdminRefreshToken' => ADMIN_REFRESH_TOKEN,
        'GoogleSmartShopping_MerchantCenterMultiClientAccountID' => MULTI_CLIENT_ACCOUNT_ID,
        'GoogleSmartShopping_AdsManagerAccountId' => ADS_MANAGER_ID,
    };

    my $MockifySystem = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Object::API::Object::Site', $hAttributes
    );

    return $MockifySystem->getMockObject();
}

sub _createError {
    my $self = shift;
    my ($Status, $hResponse) = @_;

    my $Error = DE_EPAGES::Core::API::Error->new(
        Code => 'Error',
        Vars => {
            'Status' => $Status,
            'Response' => encode_json($hResponse)
        }
    );
    return $Error;
}

sub _createUserAgent {
    my $self = shift;
    my ($Code, $hResponse) = @_;

    my $Response = $hResponse // {};
    my $MockifyUA = Test::Mockify->new('LWP::UserAgent');
    $MockifyUA->mock('request')->when(Object('HTTP::Request'))->thenReturn($self->_createResponse($Code, $Response));
    return $MockifyUA->getMockObject();
}

sub _createResponse {
    my $self = shift;
    my ($Code, $hResponse) = @_;

    my $MockifyResponse = Test::Mockify->new('HTTP::Response');
    $MockifyResponse->mock('code')->when()->thenReturn($Code);
    my $Response = encode_json($hResponse);
    $MockifyResponse->mock('decoded_content')->when()->thenReturn($Response);
    return $MockifyResponse->getMockObject();
}

sub _createGoogleSmartShoppingConfig {
    my $self = shift;
    my ($hConfigAttributes) = @_;
 
    my $hAttributes = {
        AccountEmail                         => $hConfigAttributes->{'AccountEmail'} // undef,
        MerchantCenterSubAccountCreationTime => undef,
        IsMerchantCenterSubAccount           => $hConfigAttributes->{'IsMerchantCenterSubAccount'} // undef,
        MerchantCenterAccountID              => $hConfigAttributes->{'MerchantCenterAccountID'} // undef,
        MerchantCenterMultiClientAccountID   => $hConfigAttributes->{'MerchantCenterMultiClientAccountID'} // undef,
        MerchantCenterAccountName            => undef,
        AccessToken                          => $hConfigAttributes->{'AccessToken'} // undef,
    };

    my $MockifyConfig = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::GoogleSmartShopping::API::Object::GoogleSmartShoppingConfig', $hAttributes
    );

    return $MockifyConfig->getMockObject();
}

sub _createAccountStatusesResponse {
    my $self = shift;
    my ($aProducts) = @_;


    my $hResponse = {
        "accountId"=> "414922848",
        "accountLevelIssues"=> [
            {
                "documentation"=> "https://support.google.com/merchants/answer/9849758",
                "id"=> "merchant_quality_low",
                "severity"=> "error",
                "title"=> "Account isn't eligible for enhanced free listings"
            },
            {
                "documentation"=> "https://support.google.com/merchants/answer/6159060",
                "id"=> "missing_ad_words_link",
                "severity"=> "error",
                "title"=> "No Google Ads account linked"
            }
        ],
        "kind"=> "content#accountStatus",
        "products" => (),
        "websiteClaimed"=> "true",
        "accountManagement"=> "manual"
    };

    $hResponse->{'products'} = $aProducts;
    return $hResponse;
}

#----------------------------------------------------------------------------------------

sub _createAddress {
    my $self = shift;

    my $hParameterList = {
        'Country'   => {'Code2' => TEST_ADDRESS_COUNTRY_ISO_CODE} ,
    };

    my $MockAddress = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Address::API::Object::Address',
        $hParameterList
    );

    return $MockAddress->getMockObject();
}


__PACKAGE__->runScript();

1;