#========================================================================================
# §package      DE_EPAGES::GoogleSmartShopping::t::unit::UI_Campaigns
# §base         DE_EPAGES::Core::API::Test
# §state        public
#----------------------------------------------------------------------------------------
# §description  Unit test for the GoogleSmartShopping Ads Campaign creation
#========================================================================================
package DE_EPAGES::GoogleSmartShopping::t::unit::UI_Campaigns;
use base DE_EPAGES::Core::API::Test;

use strict;

use DE_EPAGES::Core::API::Object::DateTimeFormatter;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;

use Test::Mockify;
use Test::Mockify::Matcher qw ( String HashRef Number Object );
use Test::Mockify::Sut;
use Test::Mockify::Verify qw ( WasCalled GetParametersFromMockifyCall );
use Test::More;

use constant SHOP_NAME => 'Spotlight';
use constant SHOP_URL => 'https://www.spotlight.com';

use constant MERCHANT_CENTER_ID => 503220055;
use constant MULTI_CLIENT_ACCOUNT_ID => 398123456;
use constant CONNECTED_GOOGLE_ADS_ID => 1133146331;
use constant ADS_CAMPAIGN_DAILY_BUDGET_INPUT => "12,35";
use constant ADS_CAMPAIGN_DAILY_BUDGET_EXPECTED_OUTPUT => 12.35;
use constant ADS_CAMPAIGN_DAILY_BUDGET => 12.35;
use constant ADS_CAMPAIGN_DAILY_BUDGET_CURRENCY => 'EUR';
use constant ADS_CAMPAIGN_ID => 14886962060;
use constant ADS_CAMPAIGN_NAME => 'Spotlight';
use constant MOCKED_CAMPAIGN_SERVICE_RESPONSE_PAST_DATE => [
    {
        'id' => ADS_CAMPAIGN_ID,
        'dailyBudget' => {
            'amount' => 128,
            'currency' => 'EUR',
            'id' => 9606674338,
            'name' => 'Spotlight-Local-Zwei Campaign'
        },
        'name' => 'Spotlight-Local-Zwei Campaign',
        'resourceName' => 'customers/2080416696/campaigns/14886962060',
        'startDate' => '2021-10-07',
        'status' => 'PAUSED'
    }
];
use constant MOCKED_CAMPAIGN_SERVICE_RESPONSE_FUTURE_DATE => [
    {
        'id' => ADS_CAMPAIGN_ID,
        'dailyBudget' => {
            'amount' => 128,
            'currency' => 'EUR',
            'id' => 9606674338,
            'name' => 'Spotlight-Local-Zwei Campaign'
        },
        'name' => 'Spotlight-Local-Zwei Campaign',
        'resourceName' => 'customers/2080416696/campaigns/14886962060',
        'startDate' => '2121-10-07',
        'status' => 'PAUSED'
    }
];
use constant EXPECTED_GET_CAMPAIGNS_RESPONSE_FUTURE_DATE => [
    {
        'id' => ADS_CAMPAIGN_ID,
        'dailyBudget' => {
            'amount' => 128,
            'currency' => 'EUR',
            'id' => 9606674338,
            'name' => 'Spotlight-Local-Zwei Campaign'
        },
        'name' => 'Spotlight-Local-Zwei Campaign',
        'resourceName' => 'customers/2080416696/campaigns/14886962060',
        'startDate' => '2121-10-07T00:00:00',
        'status' => 'PAUSED',
        'isFuture' => '1'
    }
];

use constant CAMPAIGN_CONVERSION_AND_REMARKETING_ID => 'AW-10865069576';
use constant CAMPAIGN_CONVERSION_LABEL => CAMPAIGN_CONVERSION_AND_REMARKETING_ID . '/4VHQCJi636cDEIic77wo';
use constant MOCKED_CAMPAIGN_CONVERSION_ACTION_SERVICE_RESPONSE => {
    'id'                => 888659224,
    'name'              => 'Spotlight - 16507745941 Conversion Action',
    'awConversionId'    => CAMPAIGN_CONVERSION_AND_REMARKETING_ID,
    'awConversionLabel' => CAMPAIGN_CONVERSION_LABEL,
};
use constant MOCKED_CAMPAIGN_REMARKETING_ACTION_SERVICE_RESPONSE => {
    'id'              => 888659185,
    'name'            => 'Spotlight - 16507745941 Remarketing Action',
    'awRemarketingId' => CAMPAIGN_CONVERSION_AND_REMARKETING_ID,
};

use constant ADS_CAMPAIGN_RESPONSE_ERROR => 'UNAUTHORIZED';

sub test {
    my $self = shift;
    $self->test_UnityViewGoogleSmartShoppingCampaigns_Success_with_Campaign();
    $self->test_UnityViewGoogleSmartShoppingCampaigns_Success_no_Campaign_with_Products();
    $self->test_UnityViewGoogleSmartShoppingCampaigns_Success_no_Campaign_no_Products();
    $self->test_UnityViewGoogleSmartShoppingCampaigns_Success_no_Campaign_no_ActiveProducts();
    $self->test_UnityViewGoogleSmartShoppingCampaigns_Success_Campaign_with_Products();
    $self->test_CreateGoogleAdsCampaign_Success();
    $self->test_CreateGoogleAdsCampaign_Fail_ConversionAction();
    $self->test_CreateGoogleAdsCampaign_Fail_RemarketingAction();
    $self->test_GetGoogleAdsCampaign_SuccessPast();
    $self->test_GetGoogleAdsCampaign_SuccessFuture();
    $self->test_GetGoogleAdsCampaign_Failure_NoCampaignFound();
    $self->test_SwitchGoogleAdsCampaignStatus_to_enabled_Success();
    $self->test_SwitchGoogleAdsCampaignStatus_to_paused_Success();
    $self->test_SwitchGoogleAdsCampaignStatus_Error();
    $self->test_UpdateGoogleAdsCampaignBudget_Success();
    $self->test_DeleteAdsCampaign_Success();
    $self->test_DeleteAdsCampaign_Failure_CampaignID();
    $self->test_DeleteAdsCampaign_Failure_Api();

    done_testing();
}


sub test_UnityViewGoogleSmartShoppingCampaigns_Success_with_Campaign {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        'AccessToken'                            => 'some-access-token',
        'RefreshToken'                           => 'some-refresh-token',
        'AreGoogleMerchantCenterRequirementsMet' => 1,
        'MerchantCenterAccountID'                => MERCHANT_CENTER_ID,
        'MerchantCenterMultiClientAccountID'     => MULTI_CLIENT_ACCOUNT_ID,
        'GoogleAdsAccountID'                     => CONNECTED_GOOGLE_ADS_ID,
        'AdsCampaignId'                          => ADS_CAMPAIGN_ID,
        'AdsCampaignConversionId'                => CAMPAIGN_CONVERSION_AND_REMARKETING_ID,
        'AdsCampaignConversionLabel'             => CAMPAIGN_CONVERSION_LABEL,
        'AdsCampaignRemarketingId'               => CAMPAIGN_CONVERSION_AND_REMARKETING_ID
    });

    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);

    my $GoogleAdsApiService = $self->_createGoogleAdsApiService({
        'getSmartShoppingCampaigns' => MOCKED_CAMPAIGN_SERVICE_RESPONSE_FUTURE_DATE,
        'createGoogleConversionAction' => 'Error',
        'createGoogleRemarketingAction' => 'Success',
    });

    my $hParams = {
        'AdsApiService' => $GoogleAdsApiService,
    };
    my $SUT = $self->_createSUT($hParams);

    $SUT->UnityViewGoogleSmartShoppingCampaigns($Servlet);

    is_deeply(
        $Servlet->vars('GoogleAdsCampaign'),
        MOCKED_CAMPAIGN_SERVICE_RESPONSE_FUTURE_DATE->[0],
        "$SubTestName - Verify that GoogleAdsCampaign TLE holds existing campaign."
    );
}

sub test_UnityViewGoogleSmartShoppingCampaigns_Success_no_Campaign_with_Products {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken                            => 'some-access-token',
        RefreshToken                           => 'some-refresh-token',
        AreGoogleMerchantCenterRequirementsMet => 1,
        MerchantCenterAccountID                => MERCHANT_CENTER_ID,
        MerchantCenterMultiClientAccountID     => MULTI_CLIENT_ACCOUNT_ID,
        GoogleAdsAccountID                     => CONNECTED_GOOGLE_ADS_ID,
    });

    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);

    my $GoogleAdsApiService = $self->_createGoogleAdsApiService({
        'getSmartShoppingCampaigns' => [],
        'createGoogleConversionAction' => 'Error',
        'createGoogleRemarketingAction' => 'Success',
    });

    my $ExpectedProductsStatistics = {
        TooManyDisapproved  => 0,
        TooManyPending      => 0,
        active              => 1
    };

    my $GoogleApiService = $self->_createGoogleApiService({
        RatedProductsStatistics => $ExpectedProductsStatistics
    });

    my $hParams = {
        'AdsApiService' => $GoogleAdsApiService,
        'GoogleApiService'    => $GoogleApiService,
    };
    my $SUT = $self->_createSUT($hParams);

    $SUT->UnityViewGoogleSmartShoppingCampaigns($Servlet);

    is_deeply(
        $Servlet->vars('RatedProductsStatistics'),
        $ExpectedProductsStatistics,
        "$SubTestName - Verify that RatedProductStatistics was correctly set."
    );
    is($Servlet->vars('CampaignCreationAllowed'), 1, "$SubTestName - Verify campaign creation is allowed");
}

sub test_UnityViewGoogleSmartShoppingCampaigns_Success_no_Campaign_no_Products {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken                            => 'some-access-token',
        RefreshToken                           => 'some-refresh-token',
        AreGoogleMerchantCenterRequirementsMet => 1,
        MerchantCenterAccountID                => MERCHANT_CENTER_ID,
        MerchantCenterMultiClientAccountID  => MULTI_CLIENT_ACCOUNT_ID,
        GoogleAdsAccountID                     => CONNECTED_GOOGLE_ADS_ID,
    });

    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);

    my $GoogleAdsApiService = $self->_createGoogleAdsApiService({
        'getSmartShoppingCampaigns' => [],
        'createGoogleConversionAction' => 'Error',
        'createGoogleRemarketingAction' => 'Success',
    });
    my $ExpectedProductsStatistics = {
        TooManyDisapproved  => 0,
        TooManyPending      => 0,
        IsNotEnoughActive     => 0
    };

    my $GoogleApiService = $self->_createGoogleApiService({
        RatedProductsStatistics => $ExpectedProductsStatistics
    });

    my $hParams = {
        'AdsApiService' => $GoogleAdsApiService,
        'GoogleApiService' => $GoogleApiService,
    };
    my $SUT = $self->_createSUT($hParams);

    $SUT->UnityViewGoogleSmartShoppingCampaigns($Servlet);

    is_deeply(
        $Servlet->vars('RatedProductsStatistics'),
        $ExpectedProductsStatistics,
        "$SubTestName - Verify that HasProductsUploaded was correctly set."
    );
    is(
        $Servlet->vars('CampaignCreationAllowed'), 0,
        "$SubTestName - Verify campaign creation NOT allowed"
    );
}

sub test_UnityViewGoogleSmartShoppingCampaigns_Success_no_Campaign_no_ActiveProducts {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken                            => 'some-access-token',
        RefreshToken                           => 'some-refresh-token',
        AreGoogleMerchantCenterRequirementsMet => 1,
        MerchantCenterAccountID                => MERCHANT_CENTER_ID,
        MerchantCenterMultiClientAccountID  => MULTI_CLIENT_ACCOUNT_ID,
        GoogleAdsAccountID                     => CONNECTED_GOOGLE_ADS_ID,
    });

    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {};
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);

    my $GoogleAdsApiService = $self->_createGoogleAdsApiService({
        'getSmartShoppingCampaigns' => [],
        'createGoogleConversionAction' => 'Error',
        'createGoogleRemarketingAction' => 'Success',
    });
    my $ExpectedProductsStatistics = {
        TooManyDisapproved  => 0,
        TooManyPending      => 0,
        IsNotEnoughActive     => 1,
    };

    my $GoogleApiService = $self->_createGoogleApiService({
        RatedProductsStatistics => $ExpectedProductsStatistics
    });

    my $hParams = {
        'AdsApiService' => $GoogleAdsApiService,
        'GoogleApiService' => $GoogleApiService,
    };
    my $SUT = $self->_createSUT($hParams);

    $SUT->UnityViewGoogleSmartShoppingCampaigns($Servlet);

    is_deeply(
        $Servlet->vars('RatedProductsStatistics'),
        $ExpectedProductsStatistics,
        "$SubTestName - Verify that HasProductsUploaded was correctly set."
    );
    is(
        $Servlet->vars('CampaignCreationAllowed'), 0,
        "$SubTestName - Verify campaign creation NOT allowed"
    );
}

sub test_UnityViewGoogleSmartShoppingCampaigns_Success_Campaign_with_Products {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        AccessToken                            => 'some-access-token',
        RefreshToken                           => 'some-refresh-token',
        AreGoogleMerchantCenterRequirementsMet => 1,
        MerchantCenterAccountID                => MERCHANT_CENTER_ID,
        MerchantCenterMultiClientAccountID     => MULTI_CLIENT_ACCOUNT_ID,
        GoogleAdsAccountID                     => CONNECTED_GOOGLE_ADS_ID,
    });

    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {
        'CampaignId'            => [ ADS_CAMPAIGN_ID ],
        'CampaignStatusEnabled' => [ 'true' ]
    };
    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);

    my $GoogleAdsApiService = $self->_createGoogleAdsApiService({
        'getSmartShoppingCampaigns' => MOCKED_CAMPAIGN_SERVICE_RESPONSE_FUTURE_DATE,
        'createGoogleConversionAction' => 'Error',
        'createGoogleRemarketingAction' => 'Success',
    });


    my $ExpectedProductsStatistics = {
        TooManyDisapproved  => 0,
        TooManyPending      => 0,
        IsNotEnoughActive     => 0
    };

    my $GoogleApiService = $self->_createGoogleApiService({
        RatedProductsStatistics => $ExpectedProductsStatistics
    });

    my $hParams = {
        'AdsApiService' => $GoogleAdsApiService,
        'GoogleApiService'    => $GoogleApiService,
    };
    my $SUT = $self->_createSUT($hParams);

    $SUT->UnityViewGoogleSmartShoppingCampaigns($Servlet);

    is_deeply(
        $Servlet->vars('RatedProductsStatistics'),
        $ExpectedProductsStatistics,
        "$SubTestName - Verify that RatedProductsStatistics was correctly set."
    );
}

sub test_CreateGoogleAdsCampaign_Success {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        'MerchantCenterAccountID' => MERCHANT_CENTER_ID,
        'GoogleAdsAccountID' => CONNECTED_GOOGLE_ADS_ID,
        'AreGoogleMerchantCenterRequirementsMet' => 1,
    });
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);

    my $hFormQueryParameters = {
        'DailyBudget' => [ADS_CAMPAIGN_DAILY_BUDGET_INPUT],
        'CampaignName' => [ADS_CAMPAIGN_NAME]
    };

    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiAdsService = $self->_createGoogleAdsApiService({
        'createGoogleAdsCampaign' => MOCKED_CAMPAIGN_SERVICE_RESPONSE_FUTURE_DATE->[0],
        'createGoogleConversionAction' => 'Success',
        'createGoogleRemarketingAction' => 'Success',
    });
    my $ScriptTagService = $self->_createScriptTagService($Shop);

    my $hParams = {
        'AdsApiService'    => $ApiAdsService,
        'ScriptTagService' => $ScriptTagService
    };
    my $SUT = $self->_createSUT($hParams);

    my $Response = $SUT->CreateGoogleAdsCampaign($Servlet);

    ok(
        WasCalled($SUT, '_validateCreateGoogleAdsCampaignRequest'),
        $SubTestName . ' - Verify that _validateCreateGoogleAdsCampaignRequest was called.'
    );
    ok(
        WasCalled($SUT, '_createCampaign'),
        $SubTestName . ' - Verify that _createCampaign was called.'
    );
    ok(WasCalled($SUT, '_createConversionAction'), $SubTestName . ' - Verify that _createConversionAction was called.');
    ok(
        WasCalled($ApiAdsService, 'createGoogleAdsCampaign'),
        $SubTestName . ' - Verify that service method createGoogleAdsCampaign was called.'
    );
    ok(
        WasCalled($ApiAdsService, 'createGoogleConversionAction'),
        $SubTestName . ' - Verify that service method createGoogleConversionAction was called.'
    );
    ok(
        WasCalled($ApiAdsService, 'createGoogleRemarketingAction'),
        $SubTestName . ' - Verify that service method createGoogleRemarketingAction was called.'
    );
    ok(
        WasCalled($ScriptTagService, 'addScriptTag'),
        $SubTestName . ' - Verify that addScriptTag was called to enable campaign.'
    );

    my $aParams = GetParametersFromMockifyCall($ApiAdsService, 'createGoogleAdsCampaign');

    my $hExpectedCampaignCreationParams = {
        'AdsCampaignName' => SHOP_NAME,
        'AdsSubAccountId' => CONNECTED_GOOGLE_ADS_ID,
        'DailyBudget' => ADS_CAMPAIGN_DAILY_BUDGET_EXPECTED_OUTPUT,
        'MerchantCenterAccountId' => MERCHANT_CENTER_ID
    };
    is_deeply(
        @$aParams[0],
        $hExpectedCampaignCreationParams,
        "$SubTestName - Verify that correct params passed on createGoogleAdsCampaign service call."
    );

    $aParams = GetParametersFromMockifyCall($ApiAdsService, 'createGoogleConversionAction');
    is_deeply(
        $aParams->[0],
        ADS_CAMPAIGN_ID,
        "$SubTestName - Verify that correct params passed on createGoogleConversionAction service call."
    );

    $aParams = GetParametersFromMockifyCall($ApiAdsService, 'createGoogleRemarketingAction');
    is_deeply(
        $aParams->[0],
        ADS_CAMPAIGN_ID,
        "$SubTestName - Verify that correct params passed on createGoogleRemarketingAction service call."
    );

    is($Response, undef, "$SubTestName - Verify that AdsCampaign created and response contains no error");

    is_deeply(
        $GoogleSmartShoppingConfig->get(
            [qw(AdsCampaignId AdsCampaignConversionId AdsCampaignConversionLabel AdsCampaignRemarketingId)]
        ),
        {
            'AdsCampaignId' => ADS_CAMPAIGN_ID,
            'AdsCampaignConversionId' => CAMPAIGN_CONVERSION_AND_REMARKETING_ID,
            'AdsCampaignConversionLabel' => CAMPAIGN_CONVERSION_LABEL,
            'AdsCampaignRemarketingId' => CAMPAIGN_CONVERSION_AND_REMARKETING_ID,
        },
        "$SubTestName - Verify that Ads campaign, conversion and remarketing IDs have been saved."
    );
}

sub test_CreateGoogleAdsCampaign_Fail_ConversionAction {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        'MerchantCenterAccountID' => MERCHANT_CENTER_ID,
        'GoogleAdsAccountID' => CONNECTED_GOOGLE_ADS_ID,
        'AreGoogleMerchantCenterRequirementsMet' => 1,
    });
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);

    my $hFormQueryParameters = {
        'DailyBudget' => [ADS_CAMPAIGN_DAILY_BUDGET_INPUT],
        'CampaignName' => [ADS_CAMPAIGN_NAME]
    };

    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiAdsService = $self->_createGoogleAdsApiService({
        'createGoogleAdsCampaign' => MOCKED_CAMPAIGN_SERVICE_RESPONSE_FUTURE_DATE->[0],
        'createGoogleConversionAction' => 'Error',
        'createGoogleRemarketingAction' => 'Success',
    });
    my $ScriptTagService = $self->_createScriptTagService($Shop);
    my $hParams = {
        'AdsApiService'    => $ApiAdsService,
        'ScriptTagService' => $ScriptTagService
    };
    my $SUT = $self->_createSUT($hParams);
    my $Response = $SUT->CreateGoogleAdsCampaign($Servlet);

    ok(
        WasCalled($SUT, '_validateCreateGoogleAdsCampaignRequest'),
        $SubTestName . ' - Verify that _validateCreateGoogleAdsCampaignRequest was called.'
    );
    ok(
        WasCalled($SUT, '_createCampaign'),
        $SubTestName . ' - Verify that _createCampaign was called.'
    );
    ok(
        WasCalled($ApiAdsService, 'createGoogleAdsCampaign'),
        $SubTestName . ' - Verify that service method createGoogleAdsCampaign was called.'
    );
    ok(
        WasCalled($SUT, '_createConversionAction'),
        $SubTestName . ' - Verify that _createConversionAction was called.'
    );
    ok(
        WasCalled($ApiAdsService, 'createGoogleConversionAction'),
        $SubTestName . ' - Verify that service method createGoogleConversionAction was called.'
    );
    ok(
        !WasCalled($SUT, '_createRemarketingAction'),
        $SubTestName . ' - Verify that _createRemarketingAction was NOT called.'
    );
    ok( !WasCalled($ScriptTagService, 'addScriptTag'), $SubTestName . ' - Verify that addScriptTag was NOT called.' );
    my $aParams = GetParametersFromMockifyCall($ApiAdsService, 'createGoogleAdsCampaign');

    my $hExpectedCampaignCreationParams = {
        'AdsCampaignName' => SHOP_NAME,
        'AdsSubAccountId' => CONNECTED_GOOGLE_ADS_ID,
        'DailyBudget' => ADS_CAMPAIGN_DAILY_BUDGET_EXPECTED_OUTPUT,
        'MerchantCenterAccountId' => MERCHANT_CENTER_ID
    };
    is_deeply(
        @$aParams[0],
        $hExpectedCampaignCreationParams,
        "$SubTestName - Verify that correct params passed on createGoogleAdsCampaign service call."
    );

    $aParams = GetParametersFromMockifyCall($ApiAdsService, 'createGoogleConversionAction');
    is_deeply(
        $aParams->[0],
        ADS_CAMPAIGN_ID,
        "$SubTestName - Verify that correct params passed on createGoogleConversionAction service call."
    );

    is($Response, undef, "$SubTestName - Verify that AdsCampaign created and response contains no error");

    ok(
        WasCalled($ServletForm, 'executeFormError'),
        "$SubTestName - Verify that Form executeFormError was NOT called."
    );

    $aParams = GetParametersFromMockifyCall($ServletForm, 'executeFormError');
    is(
        $aParams->[0]->{'Name'},
        'ConversionActionCreationFailed',
        "$SubTestName - Verify that correct params on executeFormError call."
    );

    is_deeply(
        $GoogleSmartShoppingConfig->get(
            [qw(AdsCampaignId AdsCampaignConversionId AdsCampaignConversionLabel AdsCampaignRemarketingId)]
        ),
        {
            'AdsCampaignId' => undef,
            'AdsCampaignConversionId' => undef,
            'AdsCampaignConversionLabel' => undef,
            'AdsCampaignRemarketingId' => undef,
        },
        "$SubTestName - Verify that Ads campaign, conversion and remarketing IDs have NOT been saved."
    );
}

sub test_CreateGoogleAdsCampaign_Fail_RemarketingAction {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        'MerchantCenterAccountID' => MERCHANT_CENTER_ID,
        'GoogleAdsAccountID' => CONNECTED_GOOGLE_ADS_ID,
        'AreGoogleMerchantCenterRequirementsMet' => 1,
    });
    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);

    my $hFormQueryParameters = {
        'DailyBudget' => [ADS_CAMPAIGN_DAILY_BUDGET_INPUT],
        'CampaignName' => [ADS_CAMPAIGN_NAME]
    };

    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiAdsService = $self->_createGoogleAdsApiService({
        'createGoogleAdsCampaign' => MOCKED_CAMPAIGN_SERVICE_RESPONSE_FUTURE_DATE->[0],
        'createGoogleConversionAction' => 'Success',
        'createGoogleRemarketingAction' => '',
    });
    my $ScriptTagService = $self->_createScriptTagService($Shop);

    my $hParams = {
        'AdsApiService'    => $ApiAdsService,
        'ScriptTagService' => $ScriptTagService
    };
    my $SUT = $self->_createSUT($hParams);

    my $Response = $SUT->CreateGoogleAdsCampaign($Servlet);

    ok(
        WasCalled($SUT, '_validateCreateGoogleAdsCampaignRequest'),
        $SubTestName . ' - Verify that _validateCreateGoogleAdsCampaignRequest was called.'
    );
    ok(
        WasCalled($SUT, '_createCampaign'),
        $SubTestName . ' - Verify that _createCampaign was called.'
    );
    ok(
        WasCalled($ApiAdsService, 'createGoogleAdsCampaign'),
        $SubTestName . ' - Verify that service method createGoogleAdsCampaign was called.'
    );
    ok(
    WasCalled($SUT, '_createConversionAction'),
    $SubTestName . ' - Verify that _createConversionAction was called.'
    );
    ok(
        WasCalled($ApiAdsService, 'createGoogleConversionAction'),
        $SubTestName . ' - Verify that service method createGoogleConversionAction was called.'
    );
    ok(
        WasCalled($SUT, '_createRemarketingAction'),
        $SubTestName . ' - Verify that _createRemarketingAction was called.'
    );
    ok(
        WasCalled($ApiAdsService, 'createGoogleRemarketingAction'),
        $SubTestName . ' - Verify that service method createGoogleRemarketingAction was called.'
    );
    ok( !WasCalled($ScriptTagService, 'addScriptTag'), $SubTestName . ' - Verify that addScriptTag was NOT called.' );
    my $aParams = GetParametersFromMockifyCall($ApiAdsService, 'createGoogleAdsCampaign');

    my $hExpectedCampaignCreationParams = {
        'AdsCampaignName' => SHOP_NAME,
        'AdsSubAccountId' => CONNECTED_GOOGLE_ADS_ID,
        'DailyBudget' => ADS_CAMPAIGN_DAILY_BUDGET_EXPECTED_OUTPUT,
        'MerchantCenterAccountId' => MERCHANT_CENTER_ID,
    };
    is_deeply(
        @$aParams[0],
        $hExpectedCampaignCreationParams,
        "$SubTestName - Verify that correct params passed on createGoogleAdsCampaign service call."
    );

    $aParams = GetParametersFromMockifyCall($ApiAdsService, 'createGoogleConversionAction');
    is_deeply(
        $aParams->[0], ADS_CAMPAIGN_ID,
        "$SubTestName - Verify that correct params passed on createGoogleConversionAction service call."
    );

    $aParams = GetParametersFromMockifyCall($ApiAdsService, 'createGoogleRemarketingAction');
    is_deeply(
        $aParams->[0], ADS_CAMPAIGN_ID,
        "$SubTestName - Verify that correct params passed on createGoogleRemarketingAction service call."
    );

    is($Response, undef, "$SubTestName - Verify that AdsCampaign created and response contains no error");

    ok(
        WasCalled($ServletForm, 'executeFormError'),
        "$SubTestName - Verify that Form executeFormError was NOT called."
    );

    $aParams = GetParametersFromMockifyCall($ServletForm, 'executeFormError');
    is(
        $aParams->[0]->{'Name'},
        'RemarketingActionCreationFailed',
        "$SubTestName - Verify that correct params on executeFormError call."
    );

    is_deeply(
        $GoogleSmartShoppingConfig->get(
            [qw(AdsCampaignId AdsCampaignConversionId AdsCampaignConversionLabel AdsCampaignRemarketingId)]
        ),
        {
            'AdsCampaignId' => undef,
            'AdsCampaignConversionId' => undef,
            'AdsCampaignConversionLabel' => undef,
            'AdsCampaignRemarketingId' => undef,
        },
        "$SubTestName - Verify that Ads campaign, conversion and remarketing IDs have NOT been saved."
    );
}

sub test_GetGoogleAdsCampaign_SuccessPast {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        'MerchantCenterAccountID' => MERCHANT_CENTER_ID,
        'GoogleAdsAccountID'      => CONNECTED_GOOGLE_ADS_ID,
        'AdsCampaignId'           => ADS_CAMPAIGN_ID
    });

    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);

    my $ApiAdsService = $self->_createGoogleAdsApiService({
        'getSmartShoppingCampaigns' => MOCKED_CAMPAIGN_SERVICE_RESPONSE_PAST_DATE
    });
    my $ScriptTagService = $self->_createScriptTagService($Shop);

    my $hParams = {
        'AdsApiService'    => $ApiAdsService,
        'ScriptTagService' => $ScriptTagService
    };
    my $SUT = $self->_createSUT($hParams);

    my $hResponse = $SUT->GetGoogleAdsCampaign($Shop, $GoogleSmartShoppingConfig);

    ok(
        WasCalled($ApiAdsService, 'getSmartShoppingCampaigns'),
        $SubTestName . ' - Verify that service method getSmartShoppingCampaigns was called.'
    );
    my $aParams = GetParametersFromMockifyCall($ApiAdsService, 'getSmartShoppingCampaigns');

    my $hExpectedParams = {
        'AdsSubAccountId' => CONNECTED_GOOGLE_ADS_ID
    };

    is_deeply(
        @$aParams[0],
        $hExpectedParams,
        "$SubTestName - Verify that correct params passed on getSmartShoppingCampaigns service call."
    );

    is_deeply(
        $hResponse,
        MOCKED_CAMPAIGN_SERVICE_RESPONSE_PAST_DATE->[0],
        "$SubTestName - Verify that GetGoogleAdsCampaigns returns expected response"
    );

}

sub test_GetGoogleAdsCampaign_SuccessFuture {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        'MerchantCenterAccountID' => MERCHANT_CENTER_ID,
        'GoogleAdsAccountID' => CONNECTED_GOOGLE_ADS_ID,
        'AdsCampaignId' => ADS_CAMPAIGN_ID
    });

    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);

    my $ApiAdsService = $self->_createGoogleAdsApiService({
        'getSmartShoppingCampaigns' => MOCKED_CAMPAIGN_SERVICE_RESPONSE_FUTURE_DATE
    });
    my $ScriptTagService = $self->_createScriptTagService($Shop);

    my $hParams = {
        'AdsApiService'    => $ApiAdsService,
        'ScriptTagService' => $ScriptTagService
    };
    my $SUT = $self->_createSUT($hParams);

    my $hResponse = $SUT->GetGoogleAdsCampaign($Shop, $GoogleSmartShoppingConfig);

    ok(
        WasCalled($ApiAdsService, 'getSmartShoppingCampaigns'),
        $SubTestName . ' - Verify that service method getSmartShoppingCampaigns was called.'
    );
    my $aParams = GetParametersFromMockifyCall($ApiAdsService, 'getSmartShoppingCampaigns');

    my $hExpectedParams = {
        'AdsSubAccountId' => CONNECTED_GOOGLE_ADS_ID
    };

    is_deeply(
        @$aParams[0], $hExpectedParams,
        "$SubTestName - Verify that correct params passed on getSmartShoppingCampaigns service call."
    );

    is_deeply(
        $hResponse,
        EXPECTED_GET_CAMPAIGNS_RESPONSE_FUTURE_DATE->[0],
        "$SubTestName - Verify that GetGoogleAdsCampaigns returns expected response"
    );

}

sub test_GetGoogleAdsCampaign_Failure_NoCampaignFound {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        'MerchantCenterAccountID' => MERCHANT_CENTER_ID,
        'GoogleAdsAccountID' => CONNECTED_GOOGLE_ADS_ID,
        'AdsCampaignId' => ADS_CAMPAIGN_ID
    });

    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);

    my $ApiAdsService = $self->_createGoogleAdsApiService({
        'getSmartShoppingCampaigns' => []
    });
    my $ScriptTagService = $self->_createScriptTagService($Shop);

    my $hParams = {
        'AdsApiService'    => $ApiAdsService,
        'ScriptTagService' => $ScriptTagService
    };
    my $SUT = $self->_createSUT($hParams);

    my $hResponse = $SUT->GetGoogleAdsCampaign($Shop, $GoogleSmartShoppingConfig);

    ok(
        WasCalled($ApiAdsService, 'getSmartShoppingCampaigns'),
        $SubTestName . ' - Verify that service method getSmartShoppingCampaigns was called.'
    );
    my $aParams = GetParametersFromMockifyCall($ApiAdsService, 'getSmartShoppingCampaigns');

    my $hExpectedParams = {
        'AdsSubAccountId' => CONNECTED_GOOGLE_ADS_ID
    };

    is_deeply(
        @$aParams[0], $hExpectedParams,
        "$SubTestName - Verify that correct params passed on getSmartShoppingCampaigns service call."
    );

    is_deeply(
        $hResponse, { 'id' => ADS_CAMPAIGN_ID },
        "$SubTestName - Verify that GetGoogleAdsCampaigns returns campaign id only."
    );

}

sub test_SwitchGoogleAdsCampaignStatus_to_enabled_Success {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        'MerchantCenterAccountID' => MERCHANT_CENTER_ID,
        'GoogleAdsAccountID' => CONNECTED_GOOGLE_ADS_ID,
    });

    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {
        'CampaignId'            => [ ADS_CAMPAIGN_ID ],
        'CampaignStatusEnabled' => [ 'true' ]
    };

    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiAdsService = $self->_createGoogleAdsApiService({
        'updateGoogleAdsCampaign' => 'Success'
    });
    my $ScriptTagService = $self->_createScriptTagService($Shop);

    my $hParams = {
        'AdsApiService'    => $ApiAdsService,
        'ScriptTagService' => $ScriptTagService
    };
    my $SUT = $self->_createSUT($hParams);

    $SUT->SwitchCampaignStatus($Servlet);

    my $hExpectedSwitchCampaignParams = {
        'CampaignId' => ADS_CAMPAIGN_ID,
        'CampaignStatusEnabled' => 1
    };
    ok(
        WasCalled($ApiAdsService, 'updateGoogleAdsCampaign'),
        $SubTestName . ' - Verify that service method updateGoogleAdsCampaign was called.'
    );

    my $aParams = GetParametersFromMockifyCall($ApiAdsService, 'updateGoogleAdsCampaign');
    is_deeply(
        @$aParams[0],
        $hExpectedSwitchCampaignParams,
        "$SubTestName - Verify that correct params passed on updateGoogleAdsCampaign service call."
    );
    ok(
        WasCalled($ScriptTagService, 'addScriptTag'),
        $SubTestName . ' - Verify that service method to add script tag was called.'
    );
    ok(
        !WasCalled($ScriptTagService, 'deleteScriptTag'),
        $SubTestName . ' - Verify that service method deleteScriptTag was NOT called.'
    );
    ok(
        WasCalled($Servlet, 'writeContentOutput'),
        "$SubTestName - Verify that servlet output writing method was called."
    );
    $aParams = GetParametersFromMockifyCall($Servlet, 'writeContentOutput');
    is_deeply(
        @$aParams[0],
        $hExpectedSwitchCampaignParams,
        "$SubTestName - Verify that correct params passed on writeContentOutput servlet call."
    );
}

sub test_SwitchGoogleAdsCampaignStatus_to_paused_Success {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        'MerchantCenterAccountID' => MERCHANT_CENTER_ID,
        'GoogleAdsAccountID' => CONNECTED_GOOGLE_ADS_ID,
    });

    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {
        'CampaignId'            => [ ADS_CAMPAIGN_ID ],
        'CampaignStatusEnabled' => [ 'false' ]
    };

    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiAdsService = $self->_createGoogleAdsApiService({
        'updateGoogleAdsCampaign' => 'Success'
    });
    my $ScriptTagService = $self->_createScriptTagService($Shop);

    my $hParams = {
        'AdsApiService'    => $ApiAdsService,
        'ScriptTagService' => $ScriptTagService
    };
    my $SUT = $self->_createSUT($hParams);

    $SUT->SwitchCampaignStatus($Servlet);

    my $hExpectedSwitchCampaignParams = {
        'CampaignId' => ADS_CAMPAIGN_ID,
        'CampaignStatusEnabled' => 0
    };
    ok(
        WasCalled($ApiAdsService, 'updateGoogleAdsCampaign'),
        $SubTestName . ' - Verify that service method updateGoogleAdsCampaign was called.'
    );

    my $aParams = GetParametersFromMockifyCall($ApiAdsService, 'updateGoogleAdsCampaign');
    is_deeply(
        @$aParams[0],
        $hExpectedSwitchCampaignParams,
        "$SubTestName - Verify that correct params passed on updateGoogleAdsCampaign service call."
    );
    ok(
        !WasCalled($ScriptTagService, 'addScriptTag'),
        $SubTestName . ' - Verify that service method to add script tag was NOT called.'
    );
    ok(
        !WasCalled($ScriptTagService, 'deleteScriptTag'),
        $SubTestName . ' - Verify that service method deleteScriptTag was NOT called.'
    );
    ok(
        WasCalled($Servlet, 'writeContentOutput'),
        "$SubTestName - Verify that servlet output writing method was called."
    );
    $aParams = GetParametersFromMockifyCall($Servlet, 'writeContentOutput');
    is_deeply(
        @$aParams[0],
        $hExpectedSwitchCampaignParams,
        "$SubTestName - Verify that correct params passed on writeContentOutput servlet call."
    );
}

sub test_SwitchGoogleAdsCampaignStatus_Error {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        'MerchantCenterAccountID' => MERCHANT_CENTER_ID,
        'GoogleAdsAccountID' => CONNECTED_GOOGLE_ADS_ID,
    });

    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {
        'CampaignId'            => [ ADS_CAMPAIGN_ID ],
        'CampaignStatusEnabled' => [ 'true' ]
    };

    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiAdsService = $self->_createGoogleAdsApiService({
        'updateGoogleAdsCampaign' => 'Error'
    });
    my $ScriptTagService = $self->_createScriptTagService($Shop);

    my $hParams = {
        'AdsApiService'    => $ApiAdsService,
        'ScriptTagService' => $ScriptTagService
    };
    my $SUT = $self->_createSUT($hParams);

    $SUT->SwitchCampaignStatus($Servlet);

    ok(
        WasCalled($ApiAdsService, 'updateGoogleAdsCampaign'),
        $SubTestName . ' - Verify that service method updateGoogleAdsCampaign was called.'
    );
    ok(
        !WasCalled($ScriptTagService, 'addScriptTag'),
        $SubTestName . ' - Verify that service method to add script tag was not called because Ads campaign status call failed.'
    );

    my $aParams = GetParametersFromMockifyCall($ApiAdsService, 'updateGoogleAdsCampaign');
    is_deeply(
        @$aParams[0],
        { 'CampaignId' => ADS_CAMPAIGN_ID, 'CampaignStatusEnabled' => 1 },
        "$SubTestName - Verify that correct params and values are passed on updateGoogleAdsCampaign service call."
    );

    ok(
        WasCalled($Servlet, 'writeContentOutput'),
        "$SubTestName - Verify that servlet output writing method was called."
    );
    $aParams = GetParametersFromMockifyCall($Servlet, 'writeContentOutput');
    ok(
        defined $aParams->[0]->{'Error'},
        "$SubTestName - Verify that error param is present on writeContentOutput servlet call."
    );
}


sub test_UpdateGoogleAdsCampaignBudget_Success {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        'MerchantCenterAccountID' => MERCHANT_CENTER_ID,
        'GoogleAdsAccountID' => CONNECTED_GOOGLE_ADS_ID,
        'AreGoogleMerchantCenterRequirementsMet' => 1,
    });

    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {
        'AdsCampaignId' => [ ADS_CAMPAIGN_ID ],
        'DailyBudget' => [ ADS_CAMPAIGN_DAILY_BUDGET ],
        'DailyBudgetCurrency' => [ ADS_CAMPAIGN_DAILY_BUDGET_CURRENCY ]
    };

    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiAdsService = $self->_createGoogleAdsApiService({
        'updateGoogleAdsCampaignBudget' => 'Success'
    });
    my $ScriptTagService = $self->_createScriptTagService($Shop);

    my $hParams = {
        'AdsApiService'    => $ApiAdsService,
        'ScriptTagService' => $ScriptTagService
    };
    my $SUT = $self->_createSUT($hParams);

    $SUT->UpdateGoogleAdsCampaign($Servlet);

    my $hExpectedUpdateCampaignParams = {
        'AdsCampaignId'       => ADS_CAMPAIGN_ID,
        'DailyBudget'         => ADS_CAMPAIGN_DAILY_BUDGET,
        'DailyBudgetCurrency' => ADS_CAMPAIGN_DAILY_BUDGET_CURRENCY
    };
    ok(
        WasCalled($SUT, '_validateUpdateGoogleAdsCampaignRequest'),
        $SubTestName . ' - Verify that _validateUpdateGoogleAdsCampaignRequest was called.'
    );

    ok(
        WasCalled($ApiAdsService, 'updateGoogleAdsCampaignBudget'),
        $SubTestName . ' - Verify that service method updateGoogleAdsCampaignBudget was called.'
    );
    my $aParams = GetParametersFromMockifyCall($ApiAdsService, 'updateGoogleAdsCampaignBudget');
    is_deeply(
        @$aParams[0],
        $hExpectedUpdateCampaignParams,
        "$SubTestName - Verify that correct params passed on updateGoogleAdsCampaignBudget service call."
    );

}

sub test_DeleteAdsCampaign_Success {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        'MerchantCenterAccountID' => MERCHANT_CENTER_ID,
        'GoogleAdsAccountID' => CONNECTED_GOOGLE_ADS_ID,
        'AdsCampaignId' => ADS_CAMPAIGN_ID,
        'AreGoogleMerchantCenterRequirementsMet' => 1,
    });

    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {
        'AdsCampaignId' => [ ADS_CAMPAIGN_ID ],
    };

    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiAdsService = $self->_createGoogleAdsApiService({
        'deleteAdsCampaign' => 'Success'
    });
    my $ScriptTagService = $self->_createScriptTagService($Shop);


    my $hParams = {
        'AdsApiService'    => $ApiAdsService,
        'ScriptTagService' => $ScriptTagService
    };
    my $SUT = $self->_createSUT($hParams);

    $SUT->DeleteAdsCampaign($Servlet);

    ok(
        WasCalled($GoogleSmartShoppingConfig, 'deleteAdsCampaign'),
        $SubTestName . ' - Verify that $GoogleSmartShoppingConfig->deleteAdsCampaign was called.'
    );
    ok(
        WasCalled($ApiAdsService, 'deleteAdsCampaign'),
        $SubTestName . ' - Verify that service method deleteAdsCampaign was called.'
    );

    my $aParams = GetParametersFromMockifyCall($ApiAdsService, 'deleteAdsCampaign');
    ok(
        ! WasCalled($ServletForm, 'executeFormError'),
        "$SubTestName - Verify that Form executeFormError was NOT called."
    );
    is(
        $aParams->[0],
        ADS_CAMPAIGN_ID,
        "$SubTestName - Verify that correct params passed on deleteAdsCampaign service call."
    );
}

sub test_DeleteAdsCampaign_Failure_CampaignID {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        'MerchantCenterAccountID' => MERCHANT_CENTER_ID,
        'GoogleAdsAccountID' => CONNECTED_GOOGLE_ADS_ID,
        'AdsCampaignId' => CONNECTED_GOOGLE_ADS_ID,
        'AreGoogleMerchantCenterRequirementsMet' => 1,
    });

    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {
        'AdsCampaignId' => [ ADS_CAMPAIGN_ID ],
    };

    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiAdsService = $self->_createGoogleAdsApiService({
        'deleteAdsCampaign' => 'Success'
    });
    my $ScriptTagService = $self->_createScriptTagService($Shop);
    my $hParams = {
        'AdsApiService'    => $ApiAdsService,
        'ScriptTagService' => $ScriptTagService
    };
    my $SUT = $self->_createSUT($hParams);

    $SUT->DeleteAdsCampaign($Servlet);

    ok(
        WasCalled($ApiAdsService, 'deleteAdsCampaign'),
        $SubTestName . ' - Verify that service method deleteAdsCampaign was called.'
    );

    ok(
        WasCalled($ServletForm, 'executeFormError'),
        "$SubTestName - Verify that Form executeFormError was called."
    );

    my $aDeleteAdsCampaignParams = GetParametersFromMockifyCall($ApiAdsService, 'deleteAdsCampaign');
    is(
        $aDeleteAdsCampaignParams->[0],
        ADS_CAMPAIGN_ID,
        "$SubTestName - Verify that correct params passed on deleteAdsCampaign service call."
    );

    my $aFormErrorParams = GetParametersFromMockifyCall($ServletForm, 'executeFormError');
    is_deeply(
        $aFormErrorParams->[0],
        {
            'Name'   => 'DeletionFailed',
            'Reason' => 'Validation error'
        },
        "$SubTestName - Verify that correct params passed on deleteAdsCampaign service call."
    );
}


sub test_DeleteAdsCampaign_Failure_Api {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $GoogleSmartShoppingConfig = $self->_createGoogleSmartShoppingConfig({
        'MerchantCenterAccountID' => MERCHANT_CENTER_ID,
        'GoogleAdsAccountID' => CONNECTED_GOOGLE_ADS_ID,
        'AdsCampaignId' => ADS_CAMPAIGN_ID,
        'AreGoogleMerchantCenterRequirementsMet' => 1,
    });

    my $AlreadyExistingConfig = 1;
    my $Shop = $self->_createShop($GoogleSmartShoppingConfig, $AlreadyExistingConfig);
    my $hFormQueryParameters = {
        'AdsCampaignId' => [ ADS_CAMPAIGN_ID ],
    };

    my $ServletForm = $self->_createServletForm($hFormQueryParameters);
    my $Servlet = $self->_createServlet($Shop, $ServletForm);
    my $ApiAdsService = $self->_createGoogleAdsApiService({
        'deleteAdsCampaign' => 'Failure'
    });
    my $ScriptTagService = $self->_createScriptTagService($Shop);

    my $hParams = {
        'AdsApiService'    => $ApiAdsService,
        'ScriptTagService' => $ScriptTagService
    };
    my $SUT = $self->_createSUT($hParams);

    $SUT->DeleteAdsCampaign($Servlet);

    ok(
        WasCalled($ApiAdsService, 'deleteAdsCampaign'),
        $SubTestName . ' - Verify that service method deleteAdsCampaign was called.'
    );

    ok(
        WasCalled($ServletForm, 'executeFormError'),
        "$SubTestName - Verify that Form executeFormError was called."
    );

    my $aDeleteAdsCampaignParams = GetParametersFromMockifyCall($ApiAdsService, 'deleteAdsCampaign');
    is(
        $aDeleteAdsCampaignParams->[0],
        ADS_CAMPAIGN_ID,
        "$SubTestName - Verify that correct params passed on deleteAdsCampaign service call."
    );

    my $aFormErrorParams = GetParametersFromMockifyCall($ServletForm, 'executeFormError');
    is_deeply(
        $aFormErrorParams->[0],
        {
            'Name'   => 'DeletionFailed',
            'Reason' => 'unknown_error'
        },
        "$SubTestName - Verify that correct params passed on deleteAdsCampaign service call."
    );
}

#----------------------------------------------------------------------------------------
# Mockery
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;
    my ($hParams ) = @_;

    my $MockifySut = Test::Mockify::Sut->new('DE_EPAGES::GoogleSmartShopping::UI::Campaigns');

    $MockifySut->mockImported('DE_EPAGES::GoogleSmartShopping::API::Tools', 'LogGoogleSmartShopping')
        ->whenAny()
        ->thenReturnUndef();

    $MockifySut->spy('_validateCreateGoogleAdsCampaignRequest')->whenAny();
    $MockifySut->spy('_validateUpdateGoogleAdsCampaignRequest')->whenAny();
    $MockifySut->spy('_validateDeleteAdsCampaignRequest')->whenAny();
    $MockifySut->spy('_createCampaign')->whenAny();
    $MockifySut->spy('_createConversionAction')->whenAny();
    $MockifySut->spy('_createRemarketingAction')->whenAny();
    $MockifySut->spy('_isCampaignCreationAllowed')->whenAny();

    if (defined $hParams->{'AdsApiService'}) {
        $MockifySut->mockStatic('DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService::new')
            ->when(String(), HashRef())
            ->thenReturn($hParams->{'AdsApiService'});
    }

    if (defined $hParams->{'GoogleApiService'}) {
        $MockifySut->mockStatic('DE_EPAGES::GoogleSmartShopping::API::GoogleApiService::new')
            ->when(String(), HashRef())
            ->thenReturn($hParams->{'GoogleApiService'});
    }

    if (defined $hParams->{'ScriptTagService'}) {
        $MockifySut->mockStatic('DE_EPAGES::GoogleSmartShopping::API::ScriptTagService::new')
            ->when(String(), HashRef())
            ->thenReturn($hParams->{'ScriptTagService'});
    }

    $MockifySut->mock('_getPortalConfig')
        ->when(Object())
        ->thenReturnUndef();

    return $MockifySut->getMockObject();
}

sub _createScriptTagService {
    my $self = shift;
    my ($Shop) = @_;

    my $aParameterList = [{'Shop' => $Shop}];

    my $Mockify = Test::Mockify->new(
                   'DE_EPAGES::GoogleSmartShopping::API::ScriptTagService',
                   $aParameterList
              );
    $Mockify->mock('addScriptTag')->when()->thenReturnUndef();
    $Mockify->mock('deleteScriptTag')->when()->thenReturnUndef();

    return $Mockify->getMockObject();
}

sub _createGoogleSmartShoppingConfig {
    my $self = shift;
    my ($hConfig) = @_;

    $hConfig->{'AreGoogleMerchantCenterRequirementsMet'} //= 0;

    my $hAttributes = {
        ChecklistBillingTOC                => $hConfig->{'AreGoogleMerchantCenterRequirementsMet'},
        ChecklistContactInfo               => $hConfig->{'AreGoogleMerchantCenterRequirementsMet'},
        ChecklistGoogleAdsPolicies         => $hConfig->{'AreGoogleMerchantCenterRequirementsMet'},
        ChecklistCookieNotice              => $hConfig->{'AreGoogleMerchantCenterRequirementsMet'},
        ChecklistReturnPolicy              => $hConfig->{'AreGoogleMerchantCenterRequirementsMet'},
        MerchantCenterAccountID            => $hConfig->{'MerchantCenterAccountID'} // undef,
        GoogleAdsAccountID                 => $hConfig->{'GoogleAdsAccountID'} // undef,
        AdsCampaignId                      => $hConfig->{'AdsCampaignId'} // undef,
        MerchantCenterMultiClientAccountID => $hConfig->{'MerchantCenterMultiClientAccountID'} // undef,
        AreMerchantCenterAndAdsLinked      => $hConfig->{'AreMerchantCenterAndAdsLinked'} // 1,
        AdsCampaignConversionId            => $hConfig->{'AdsCampaignConversionId'} // undef,
        AdsCampaignConversionLabel         => $hConfig->{'AdsCampaignConversionLabel'} // undef,
        AdsCampaignRemarketingId           => $hConfig->{'AdsCampaignRemarketingId'} // undef,

    };

    my $MockifyConfig = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::GoogleSmartShopping::API::Object::GoogleSmartShoppingConfig', $hAttributes
    );

    $MockifyConfig->addMockWithReturnValue('deleteAdsCampaign', undef);

    return $MockifyConfig->getMockObject();
}

sub _createGoogleAdsApiService {
    my $self = shift;
    my ($hServiceResponse) = @_;

    my $Mockify = Test::Mockify->new('DE_EPAGES::GoogleSmartShopping::API::GoogleAdsApiService');
    $Mockify->mock('createGoogleAdsCampaign')
        ->when(HashRef())
        ->thenReturn($hServiceResponse->{'createGoogleAdsCampaign'} // {});

    $Mockify->mock('getSmartShoppingCampaigns')
        ->when(HashRef())
        ->thenReturn($hServiceResponse->{'getSmartShoppingCampaigns'} // {});

    if (defined $hServiceResponse->{'createGoogleConversionAction'}) {
        if ($hServiceResponse->{'createGoogleConversionAction'} eq 'Success') {
            $Mockify->mock('createGoogleConversionAction')
                ->when(Number())
                ->thenReturn(MOCKED_CAMPAIGN_CONVERSION_ACTION_SERVICE_RESPONSE);
        } else {
            $Mockify->mock('createGoogleConversionAction')
                ->when(Number())
                ->thenThrowError(ADS_CAMPAIGN_RESPONSE_ERROR);
        }
    }

    if (defined $hServiceResponse->{'updateGoogleAdsCampaign'}) {
        if ($hServiceResponse->{'updateGoogleAdsCampaign'} eq 'Success') {
            $Mockify->mock('updateGoogleAdsCampaign')
                ->when(HashRef())
                ->thenReturnUndef();
        } else {
            $Mockify->mock('updateGoogleAdsCampaign')
                ->when(HashRef())
                ->thenThrowError(ADS_CAMPAIGN_RESPONSE_ERROR);
        }
    }

    if (defined $hServiceResponse->{'updateGoogleAdsCampaignBudget'}) {
        if ($hServiceResponse->{'updateGoogleAdsCampaignBudget'} eq 'Success') {
            $Mockify->mock('updateGoogleAdsCampaignBudget')
                ->when(HashRef())
                ->thenReturnUndef();
        } else {
            $Mockify->mock('updateGoogleAdsCampaignBudget')
                ->when(HashRef())
                ->thenThrowError(ADS_CAMPAIGN_RESPONSE_ERROR);
        }
    }

    if (defined $hServiceResponse->{'deleteAdsCampaign'}) {
        if ($hServiceResponse->{'deleteAdsCampaign'} eq 'Success') {
            $Mockify->mock('deleteAdsCampaign')
                ->when(Number())
                ->thenReturnUndef();
        } else {
            $Mockify->mock('deleteAdsCampaign')
                ->when(Number())
                ->thenThrowError(ADS_CAMPAIGN_RESPONSE_ERROR);
        }
    }

    if (defined $hServiceResponse->{'createGoogleRemarketingAction'}) {
        if ($hServiceResponse->{'createGoogleRemarketingAction'} eq 'Success') {
            $Mockify->mock('createGoogleRemarketingAction')
                ->when(Number())
                ->thenReturn(MOCKED_CAMPAIGN_REMARKETING_ACTION_SERVICE_RESPONSE);
        } else {
            $Mockify->mock('createGoogleRemarketingAction')
                ->when(Number())
                ->thenThrowError(ADS_CAMPAIGN_RESPONSE_ERROR);
        }
    }

    return $Mockify->getMockObject();
}

sub _createServlet {
    my $self = shift;
    my ($Shop, $ServletForm) = @_;

    my $MockifyServlet = DE_EPAGES::Test::Mock::MockifyServlet->new($Shop);
    $MockifyServlet->addMockWithReturnValue('form', $ServletForm);
    $MockifyServlet->addMockWithReturnValue('response', $self->_createServletResponse());
    $MockifyServlet->addMockWithReturnValueAndParameterCheck('writeContentOutput', undef, ['hashref']);

    return $MockifyServlet->getMockObject();
}

sub _createShop {
    my $self = shift;
    my ($GoogleSmartShoppingConfig, $AlreadyExistingConfig) = @_;
    my $hAttributes = {
        'ID'                          => undef,
        'GoogleSiteVerificationToken' => 'VerificationToken',
        'GoogleSmartShoppingConfig'   => $GoogleSmartShoppingConfig,
        'LanguageID'                  => 'DE',
        'NameOrAlias'                 => SHOP_NAME,
        'CurrencyID'                  => 'EUR',
        'WebUrlUnitySSL'              => SHOP_URL
    };
    my $MockifyShop = DE_EPAGES::Test::Mock::MockifyEpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);
    $MockifyShop->addMockWithReturnValueAndParameterCheck(
        'existsChild', $AlreadyExistingConfig, [ {'string' => 'GoogleSmartShoppingConfig'} ]
    );
    $MockifyShop->addMockWithReturnValueAndParameterCheck(
        'insertChild', $GoogleSmartShoppingConfig, [ {'string' => 'GoogleSmartShoppingConfig'}, 'hashref' ]
    );

    return $MockifyShop->getMockObject();
}

sub _createServletForm {
    my $self = shift;
    my ($hQueryParameters) = @_;

    my $Mockify = Test::Mockify->new( 'DE_EPAGES::Presentation::API::Form');
    $Mockify->mock('values')
        ->when()
        ->thenReturn($hQueryParameters);

    $Mockify->mock('formValues')
        ->when()
        ->thenReturn($hQueryParameters);

    $Mockify->mock('field')
        ->whenAny()
        ->thenReturn(ADS_CAMPAIGN_DAILY_BUDGET_EXPECTED_OUTPUT);

    $Mockify->mock('executeFormError')
        ->when()
        ->thenReturnUndef();

    $Mockify->mock('executeFormError')
        ->when(HashRef())
        ->thenReturnUndef();

    $Mockify->mock('addFormError')
        ->when(HashRef())
        ->thenReturnUndef();

    return $Mockify->getMockObject();
}

sub _createServletResponse {
    my $self = shift;

    my $Mockify = Test::Mockify->new( 'HTTP::Response');
    $Mockify->mock('code')
        ->whenAny()
        ->thenReturnUndef();

    return $Mockify->getMockObject();
}

sub _createGoogleApiService {
    my $self = shift;
    my ($hServiceResponse) = @_;

    my $Mockify = Test::Mockify->new('DE_EPAGES::GoogleSmartShopping::API::GoogleApiService');

    $Mockify->mock('getRatedProductsStatistics')
        ->when()
        ->thenReturn($hServiceResponse->{'RatedProductsStatistics'} // {}
    );
    return $Mockify->getMockObject();

}

__PACKAGE__->runScript();

