#========================================================================================
# package      DE_EPAGES::Order::API::TLE::OrderHandler
# state        public
#========================================================================================
package DE_EPAGES::Order::API::TLE::OrderHandler;

use strict;

use DateTime;
use DE_EPAGES::Core::API::Log qw( GetLog );
use DE_EPAGES::Core::API::PerlTools qw( GetCurrentDateTime );
use DE_EPAGES::Core::API::TestParameter qw( TestObject TestInteger );
use DE_EPAGES::Customer::API::Shop qw( GetCustomerForUser );
use DE_EPAGES::Object::API::Factory qw( LoadRootObject LoadClassByAlias );
use DE_EPAGES::Object::API::Table::Currency qw( GetInfoCurrency );
use DE_EPAGES::Object::API::Table::Locale qw( GetInfoLocale );
use DE_EPAGES::Order::API::Constants qw ( DELIVERY_TRESHOLD );
use DE_EPAGES::Order::API::SalesVolume qw ( GetSalesVolumeByTaxModel );
use DE_EPAGES::Order::DAL::Order qw( DAL_UnseenOrderCount );
use DE_EPAGES::Presentation::API::LocaleHandler qw ( AddLanguageToURL );
use DE_EPAGES::Presentation::API::SiteTLEProcessor qw( ReplaceTLEs  );
use DE_EPAGES::Presentation::API::Template qw( Translate );
use DE_EPAGES::Search::API::SearchDefinition qw( GetSearchDefinition );

use feature 'switch';

#========================================================================================
# function     RegisterHandlerProc
# state        private
#----------------------------------------------------------------------------------------
# syntax       $RegisterHandlerProc( $hParams );
#----------------------------------------------------------------------------------------
# description  this function registers this attribute handle on the processor via hook
#               function on hook TLEProcessorRegistration
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments keys :
#               <ul>
#                   <li>Processor - tle processor - object
#               </ul> | ref.hash
#========================================================================================
sub RegisterHandlerProc {
    my ($Params) = @_;
    GetLog->debug( 'OrderHandler.RegisterHandlerProc' );

    __PACKAGE__->new()->register( $Params->{'Processor'} );
    return;
}

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler = new DE_EPAGES::Object::API::TLE::OrderHandler;
# example      my $PrefixHandler  = new DE_EPAGES::Object::API::TLE::OrderHandler;
#----------------------------------------------------------------------------------------
# description  create a new handler
#----------------------------------------------------------------------------------------
# return       $Handler | blessed object | object
#========================================================================================
sub new { bless {}, shift; }

#========================================================================================
# function     register
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->register( $Processor );
#----------------------------------------------------------------------------------------
# description  this function register this handle on the processor
#----------------------------------------------------------------------------------------
# input        $Processor | tle processor | object
#========================================================================================
sub register {
    my $self = shift;
    my ($Processor) = @_;
    GetLog->debug( 'OrderHandler.register' );

    $Processor->registerHandler('FunctionHandler', $self, 'CANAddToBasket');
    $Processor->registerHandler('FunctionHandler', $self, 'ORDERCONFIRMATIONTEXT');
    $Processor->registerHandler('FunctionHandler', $self, 'BASKETBELOWTEXT');
    $Processor->registerHandler('FunctionHandler', $self, 'BASKETOVERTEXT');
    $Processor->registerHandler('FunctionHandler', $self, 'ORDERPROCESSCHECKTEXT');
    $Processor->registerHandler('FunctionHandler', $self, 'ACCEPTTACTEXT');
    $Processor->registerHandler('FunctionHandler', $self, 'ACCEPTCUSTOMERINFORMATIONTEXT');
    $Processor->registerHandler('FunctionHandler', $self, 'SALESVOLUME');
    $Processor->registerHandler('FunctionHandler', $self, 'GetLastOrders');
    $Processor->registerHandler('FunctionHandler', $self, 'GetLastCustomerOrderAndOrderCount');
    $Processor->registerHandler('FunctionHandler', $self, 'GetUnseenCustomerOrderCount');
    $Processor->registerHandler('FunctionHandler', $self, 'THRESHOLDLIMIT');

    return;
}

#========================================================================================
# function     CANAddToBasket
# state        public
#----------------------------------------------------------------------------------------
# syntax       #CANAddToBasket(#LineItemContainer, #Element)
# syntax       #IF( #CANAddToBasket( #Basket.LineItemContainer.Object, #ShippingMethod.Object ) )
#                 Set shipping to: #ShippingMethod.NameOrAlias
#               #ENDIF
#----------------------------------------------------------------------------------------
# description  Returns true if the element (for example payment method, or shipping
#               method) can added to the basket. The element
#               must implement the method canAddToBasket($hVars) like
#               <function DE_EPAGES::Shipping::API::Object::ShippingMethod::canAddToBasket>
#               <function DE_EPAGES::Payment::API::Object::PaymentMethod::canAddToBasket>.
#               TLE function adds Container,CurrencyID and TaxModel to $hVars.
#----------------------------------------------------------------------------------------
# input        #LineItemContainer | line item container | object
# input        #Element | shipping method or payment method | object
#========================================================================================
sub CANAddToBasket {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($Container, $Element) = @$aParams;

    return $Element->canAddToBasket({
            'Container'  => $Container,
            'CurrencyID' => $Container->get('CurrencyID'),
            'TaxModel'   => $Container->get('TaxModel'),
        });
}

#========================================================================================
# function     ORDERCONFIRMATIONTEXT
# state        public
#----------------------------------------------------------------------------------------
# syntax       #ORDERCONFIRMATIONTEXT(#Order)
#----------------------------------------------------------------------------------------
# description  Returns the tle replaced orderconfirmation text.
#----------------------------------------------------------------------------------------
# input        #Order | order object | object
#========================================================================================
sub ORDERCONFIRMATIONTEXT {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($CustomerOrder) = @$aParams;
    TestObject('CustomerOrder', $CustomerOrder, 'DE_EPAGES::Order::API::Object::CustomerOrder');

    my $Site = $CustomerOrder->getSite();
    my $TLEText = $Site->get('OrderConfirmationText');

    # use locale and language from the original processor (important for formatters)
    my $hContextVars;
    $hContextVars->{'Locale'} = $Processor->tle('Locale') // $Processor->tle('INPUT.Locale') ;

    # use the SiteTLEProcessor in the context of the $Processor for replacing text
    my $Text;
    $self->_tleContextOrder( $CustomerOrder, $hContextVars, sub {
            my ($hVarsObject, $LanguageID) = @_;
            $Text = ReplaceTLEs($TLEText, $hVarsObject);
        });
    return $Text;
}

#========================================================================================
# function     BASKETBELOWTEXT
# state        public
#----------------------------------------------------------------------------------------
# syntax       #BASKETBELOWTEXT(#BasketOrOrder)
#----------------------------------------------------------------------------------------
# description  Returns the tle replaced text below the checkout basket.
#----------------------------------------------------------------------------------------
# input        #BasketOrOrder | Basket or Customer Order object | object
#========================================================================================
sub BASKETBELOWTEXT {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($BasketOrOrder) = @$aParams;
    TestObject('BasketOrOrder', $BasketOrOrder);

    my $Shop = $BasketOrOrder->getSite();
    # if parameter is no customer order or basket, or for a unity shop a shop or a user
    # (password lost functionality - UNITY-3939)
    return unless ($BasketOrOrder->instanceOf('CustomerOrder') or $BasketOrOrder->instanceOf('Basket')
        or ($Shop->get('IsUnityRegistered') and ($BasketOrOrder->instanceOf('Shop')
            or $BasketOrOrder->instanceOf('User'))));

    my $TLEText = $Shop->get('BasketBelowText');

    # use locale and language from the original processor (important for formatters)
    my $hContextVars;
    $hContextVars->{'Locale'} = $Processor->tle('Locale') // $Processor->tle('INPUT.Locale') ;

    # use the SiteTLEProcessor in the context of the $Processor for replacing text
    my $Text;
    $self->_tleContextBasketOrOrder($BasketOrOrder, $hContextVars, sub {
            my ($hVarsObject, $LanguageID) = @_;
            $Text = ReplaceTLEs($TLEText, $hVarsObject);
        });
    return $Text;
}

#========================================================================================
# function     BASKETOVERTEXT
# state        public
#----------------------------------------------------------------------------------------
# syntax       #BASKETOVERTEXT(#BasketOrOrder)
#----------------------------------------------------------------------------------------
# description  Returns the tle replaced text over the checkout basket.
#----------------------------------------------------------------------------------------
# input        #BasketOrOrder | Basket or Customer Order object | object
#========================================================================================
sub BASKETOVERTEXT {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($BasketOrOrder) = @$aParams;
    TestObject('BasketOrOrder', $BasketOrOrder);

    # if parameter is no customer order or basket
    return unless ($BasketOrOrder->instanceOf('CustomerOrder') or $BasketOrOrder->instanceOf('Basket'));

    my $Shop = $BasketOrOrder->getSite();
    my $TLEText = $Shop->get('BasketOverText');

    # use locale and language from the original processor (important for formatters)
    my $hContextVars;
    $hContextVars->{'Locale'} = $Processor->tle('Locale') // $Processor->tle('INPUT.Locale') ;

    # use the SiteTLEProcessor in the context of the $Processor for replacing text
    my $Text;
    $self->_tleContextBasketOrOrder($BasketOrOrder, $hContextVars, sub {
            my ($hVarsObject, $LanguageID) = @_;
            $Text = ReplaceTLEs($TLEText, $hVarsObject);
        });
    return $Text;
}

#========================================================================================
# function     ORDERPROCESSCHECKTEXT
# state        public
#----------------------------------------------------------------------------------------
# syntax       #ORDERPROCESSCHECKTEXT(#BasketOrOrder)
#----------------------------------------------------------------------------------------
# description  Returns the tle replaced text on the last checkout page.
#----------------------------------------------------------------------------------------
# input        #BasketOrOrder | Basket or Customer Order object | object
#========================================================================================
sub ORDERPROCESSCHECKTEXT {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($BasketOrOrder) = @$aParams;
    TestObject('BasketOrOrder', $BasketOrOrder);

    # if parameter is no customer order or basket
    return unless ($BasketOrOrder->instanceOf('CustomerOrder') or $BasketOrOrder->instanceOf('Basket'));

    my $Shop = $BasketOrOrder->getSite();
    my $TLEText = $Shop->get('OrderProcessCheckText');

    # use locale and language from the original processor (important for formatters)
    my $hContextVars;
    $hContextVars->{'Locale'} = $Processor->tle('Locale') // $Processor->tle('INPUT.Locale') ;

    # use the SiteTLEProcessor in the context of the $Processor for replacing text
    my $Text;
    $self->_tleContextBasketOrOrder($BasketOrOrder, $hContextVars, sub {
            my ($hVarsObject, $LanguageID) = @_;
            $Text = ReplaceTLEs($TLEText, $hVarsObject);
        });
    return $Text;
}

#========================================================================================
# function     ACCEPTTACTEXT
# state        public
#----------------------------------------------------------------------------------------
# syntax       #ACCEPTTACTEXT(#BasketOrOrder)
#----------------------------------------------------------------------------------------
# description  Returns the tle replaced text for the "accept the terms and conditions"
#               text.
#----------------------------------------------------------------------------------------
# input        #BasketOrOrder | Basket or Customer Order object | object
#========================================================================================
sub ACCEPTTACTEXT {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($BasketOrOrder) = @$aParams;
    TestObject('BasketOrOrder', $BasketOrOrder);

    # if parameter is no customer order or basket
    return unless ($BasketOrOrder->instanceOf('CustomerOrder') or $BasketOrOrder->instanceOf('Basket'));

    my $Shop = $BasketOrOrder->getSite();
    my $TLEText = $Shop->get('AcceptTacText');

    # use default text if merchant does not have entered something
    unless (defined $TLEText) {
        $TLEText = Translate('AcceptTAC', undef, "DE_EPAGES::Order");
    }

    # use locale and language from the original processor (important for formatters)
    my $hContextVars;
    $hContextVars->{'Locale'} = $Processor->tle('Locale') // $Processor->tle('INPUT.Locale');

    # use the SiteTLEProcessor in the context of the $Processor for replacing text
    my $Text;
    $self->_tleContextBasketOrOrder($BasketOrOrder, $hContextVars, sub {
            my ($hVarsObject, $LanguageID) = @_;
            $Text = ReplaceTLEs($TLEText, $hVarsObject);
        });
    return $Text;
}

#========================================================================================
# function     ACCEPTCUSTOMERINFORMATIONTEXT
# state        public
#----------------------------------------------------------------------------------------
# syntax       #ACCEPTCUSTOMERINFORMATIONTEXT(#BasketOrOrder)
#----------------------------------------------------------------------------------------
# description  Returns the tle replaced text for the "accept customer information" text.
#----------------------------------------------------------------------------------------
# input        #BasketOrOrder | Basket or Customer Order object | object
#========================================================================================
sub ACCEPTCUSTOMERINFORMATIONTEXT {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($BasketOrOrder) = @$aParams;
    TestObject('BasketOrOrder', $BasketOrOrder);

    # if parameter is no customer order or basket
    return unless ($BasketOrOrder->instanceOf('CustomerOrder') or $BasketOrOrder->instanceOf('Basket'));

    my $Shop = $BasketOrOrder->getSite();
    my $TLEText = $Shop->get('AcceptCustomerInformationText');

    # use default text if merchant does not have entered something
    unless (defined $TLEText) {
        $TLEText = Translate('AcceptCustomerInformationDefaultText', undef, "DE_EPAGES::Order");
    }

    # use locale and language from the original processor (important for formatters)
    my $hContextVars;
    $hContextVars->{'Locale'} = $Processor->tle('Locale') // $Processor->tle('INPUT.Locale');

    # use the SiteTLEProcessor in the context of the $Processor for replacing text
    my $Text;
    $self->_tleContextBasketOrOrder($BasketOrOrder, $hContextVars, sub {
            my ($hVarsObject, $LanguageID) = @_;
            $Text = ReplaceTLEs($TLEText, $hVarsObject);
        });

    return $Text;
}

#========================================================================================
# function     _tleContextOrder
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Order->_tleContextOrder($Order, $hRegionalParams, $Sub);
#----------------------------------------------------------------------------------------
# description  Initializes the TLE hash and sets the language context for the TLE
#               processor to the language given in $hRegionalParams->{Language}.
#----------------------------------------------------------------------------------------
# input        $Order | order object | object
# input        $hRegionalParams | see <function subject> | ref.hash
# input        $Sub | code to execute in the language context. The code is called
#                   with 2 parameters:
#                   <ul>
#                       <li>$hVarsObject - TLE hash, including regional settings - ref.hash</li>
#                       <li>$LanguageID - language id - int</li>
#                   </ul> | ref.code
#========================================================================================
sub _tleContextOrder {
    my $self = shift;
    my ($Order, $hRegionalParams, $Sub) = @_;

    my $hVarsObject = $self->_getOrderVars($Order, $hRegionalParams);
    my $LanguageID = GetInfoLocale($hRegionalParams->{'Locale'})->{'LanguageID'};
    local $DE_EPAGES::Object::API::Language::LANGUAGEID = $LanguageID;
    $Sub->( $hVarsObject, $LanguageID );

    return;
}

#========================================================================================
# function     _tleContextBasketOrOrder
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Order->_tleContextBasketOrOrder($BasketOrOrder, $hRegionalParams, $Sub);
#----------------------------------------------------------------------------------------
# description  Initializes the TLE hash and sets the language context for the TLE
#               processor to the language given in $hRegionalParams->{Language}.
#----------------------------------------------------------------------------------------
# input        $BasketOrOrder | Basket or Customer Order object | object
# input        $hRegionalParams | see <function subject> | ref.hash
# input        $Sub | code to execute in the language context. The code is called
#                   with 2 parameters:
#                   <ul>
#                       <li>$hVarsObject - TLE hash, including regional settings - ref.hash</li>
#                       <li>$LanguageID - language id - int</li>
#                   </ul> | ref.code
#========================================================================================
sub _tleContextBasketOrOrder {
    my $self = shift;
    my ($BasketOrOrder, $hRegionalParams, $Sub) = @_;

    my $hVarsObject = $self->_getBasketOrOrderVars($BasketOrOrder, $hRegionalParams);
    my $LanguageID = GetInfoLocale($hRegionalParams->{'Locale'})->{'LanguageID'};
    local $DE_EPAGES::Object::API::Language::LANGUAGEID = $LanguageID;
    $Sub->( $hVarsObject, $LanguageID );

    return;
}

#========================================================================================
# function     _getBasketOrOrderVars
# state        private
#----------------------------------------------------------------------------------------
# syntax       $OrderHandler->_getBasketOrOrderVars($BasketOrOrder, $hRegionalParams);
#----------------------------------------------------------------------------------------
# description  Provides TLE vars for basket or order.
#----------------------------------------------------------------------------------------
# input        $BasketOrOrder | Basket or Customer Order object | object
# input        $hRegionalParams | see <function subject> | ref.hash
# return       $hVars | TLE vars | ref.hash
#========================================================================================
sub _getBasketOrOrderVars {
    my $self = shift;
    my ($BasketOrOrder, $hRegionalParams) = @_;

    my $Shop                     = $BasketOrOrder->getSite();

    # only TLEs in keys($%hMap) are processed
    # values are only valid for unity. For epages6 the Shop attributes are used
    my $hMap = {
        'CustomerInformation'           => '/l/withdrawal',
        'Contact'                       => '/l/contact',
        'TermsAndConditions'            => '/l/tac',
        'PrivacyPolicy'                 => '/l/privacy',
        'ShippingInformation'           => '/l/shipping',
        'ShippingAndPaymentInformation' => '/l/shippingandpayment',
    };

    # get Attributes used in epages6
    my $hShopVars   = $Shop->get([qw(IsUnityRegistered WebUrlUnitySSL CustomerInformation PrivacyPolicy TermsAndConditions ShippingAndPaymentInformation)]);
    my $hVarsObject = {};

    # add the language to the url like <url>/ru/cart
    my $UnityShopURL = AddLanguageToURL($hShopVars->{'WebUrlUnitySSL'}, $DE_EPAGES::Object::API::Language::LANGUAGEID, 0);

    my $UseUnitySubdomainRouting = $hShopVars->{'IsUnityRegistered'} ? LoadRootObject()->get('UnityUseSubdomainRouting') : undef;

    # loop over all attibutes to provide also all unity links for legal pages
    foreach my $AttributeName (keys(%$hMap)){
        if($hShopVars->{'IsUnityRegistered'}){
            # we do not use the legal stored in epages6 for unity, so we have to build the unity urls
            # rebuild URL like scheme: http://unityserver.epages.com:7000/l/shipping?shop=xyzshop
            # (with subdomainrouting: http://xyzshop.unityserver.epages.com:7000/l/shipping)
            if ($UseUnitySubdomainRouting) {
                $hVarsObject->{$AttributeName} = $UnityShopURL . $hMap->{$AttributeName};
            } else {
                my ($BaseURL,$ShopName) = split(/\?/,$UnityShopURL);
                $hVarsObject->{$AttributeName} = $BaseURL . $hMap->{$AttributeName} . '?' . $ShopName ;
            }
        } else {
            # use urls from epages legal pages if we have not a unity shop
            # add Viewaction for fix AD-5898
            $hVarsObject->{$AttributeName}  = defined $hShopVars->{$AttributeName}
                ? $hShopVars->{$AttributeName}->get('WebUrlSSL').'&amp;ViewAction=ViewPopup' : '';
        }
    }

    # ImprintWebUrl and ShippingInformationWebUrl are only filled for unity
    return {
        'CustomerInformationWebUrl'             => $hVarsObject->{CustomerInformation},
        'PrivacyPolicyWebUrl'                   => $hVarsObject->{PrivacyPolicy},
        'TermsAndConditionsWebUrl'              => $hVarsObject->{TermsAndConditions},
        'ContactWebUrl'                         => $hVarsObject->{Contact},
        'ShippingInformationWebUrl'             => $hVarsObject->{ShippingInformation},
        'ShippingAndPaymentInformationWebUrl'   => $hVarsObject->{ShippingAndPaymentInformation},
        'INPUT'                                 => {
            'ObjectID'                  => $BasketOrOrder->id,
            %$hRegionalParams
        },
    };
}

#========================================================================================
# function     _getOrderVars
# state        private
#----------------------------------------------------------------------------------------
# syntax       $OrderHandler->_getOrderVars($Order, $hRegionalParams);
#----------------------------------------------------------------------------------------
# description  Provides TLE vars for order.
#----------------------------------------------------------------------------------------
# input        $Order | Customer Order object | object
# input        $hRegionalParams | see <function subject> | ref.hash
# return       $hVars | TLE vars | ref.hash
#========================================================================================
sub _getOrderVars {
    my $self = shift;
    my ($Order, $hRegionalParams) = @_;

    # we have only a TLE WhiteList here, all other TLEs are ignored
    # CustomerNumber OrderNumber OrderTotalAmountGross OrderTotalAmountNet CountOfProducts CurrencySymbol CurrencyCode
    # CustomerInformationWebUrl PrivacyPolicyWebUrl TermsAndConditionWebUrl

    my $hVars = $self->_getBasketOrOrderVars($Order, $hRegionalParams);
    my $CurrencySymbol = GetInfoCurrency($Order->container->get('CurrencyID'))->{'Symbol'};

    return {
        'CustomerNumber'            => $Order->get('Customer')->alias,
        'OrderNumber'               => $Order->alias,
        'OrderTotalAmountGross'     => $Order->container->get('GrandTotal'),
        'OrderTotalAmountNet'       => $Order->container->get('TotalBeforeTax'),
        'CountOfProducts'           => scalar(@{$Order->container->get('LineItems')}),
        'CurrencySymbol'            => $CurrencySymbol,
        'CurrencyCode'              => $Order->container->get('CurrencyID'),
        'INPUT'                     => {
            'ObjectID' => $Order->id,
            %$hRegionalParams
        },
        %$hVars
    };
}

#========================================================================================
# function     SALESVOLUME
# state        public
#----------------------------------------------------------------------------------------
# syntax       #SALESVOLUME(#Shop)
#----------------------------------------------------------------------------------------
# description  Returns the sales volume for the given duration (month or all)
#----------------------------------------------------------------------------------------
# input        #Shop | Shop object | object
# input        #TaxModel | TaxModel | int
# input        #Duration | (optional, default: 'month') can be 'month' or 'all' | string
# input        #CustomerID | (optional, default: undef) ID of the customer | string
#========================================================================================
sub SALESVOLUME {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($Shop, $TaxModel, $Duration, $CustomerID) = @$aParams;

    $Duration = 'month' unless defined $Duration;

    TestObject('Shop', $Shop);
    TestInteger('TaxModel', $TaxModel, 0, 1);

    # use the shop default currency
    my $CurrencyID = $Shop->get('CurrencyID');

    # reset today to 00:00:00
    my $Now = GetCurrentDateTime();
    my $StartToday = $Now->clone->truncate( to => 'day' );

    given ( $Duration ) {
        when ('month') {
            # reset today to 1st day of the current month
            my $StartMonth = $StartToday->clone->truncate( to => 'month' );
            # orders from this month
            my ($SalesVolumeMonth) = GetSalesVolumeByTaxModel($Shop,$StartMonth,$Now,$CurrencyID,$TaxModel,$CustomerID);
            return $SalesVolumeMonth;
        } when ('all') {
            # we start at the beginning of counting the time
            my $StartDate = DateTime->new( year => 1970, month => 1, day => 1);
            my ($SalesVolumeAll) = GetSalesVolumeByTaxModel($Shop,$StartDate,$Now,$CurrencyID,$TaxModel,$CustomerID);
            return $SalesVolumeAll;
        }
    }

    return undef;
}

#========================================================================================
# function     THRESHOLDLIMIT
# state        public
#----------------------------------------------------------------------------------------
# syntax       #THRESHOLDLIMIT(#Shop)
#----------------------------------------------------------------------------------------
# description  Returns the delivery threshold which is used to determine the limit
#               of the delivery threshold for deliveries in EU country's
#----------------------------------------------------------------------------------------
# return       $ThresholdLimit | ThresholdLimit for EU country's | int
#========================================================================================
sub THRESHOLDLIMIT {
    my $self = shift;
    my ($Processor, $aParams) = @_;

    my $hShop = $aParams->[0];
    my $CurrencyID = $hShop->{'CurrencyID'};

    #-- Whatever the default currency is in the MBO, convert our constant threshold limit (always in EURO) to that currency,
    #-- provided that we have an exchange rate for it (we only care about 'EU' currencies')
    #--
    #-- See also DE_EPAGES::Tax::UI::Shop::ViewTaxMatrix
    if ( $CurrencyID ) {
        my $System = LoadRootObject();
        my $hExchangeRates;
        eval {
           $hExchangeRates = $System->get('ExchangeRates');
        };
        $hExchangeRates = undef if $@;

        if ( ref $hExchangeRates eq 'HASH' && $hExchangeRates->{$CurrencyID} ) {
            return DELIVERY_TRESHOLD * $hExchangeRates->{$CurrencyID};
        }
    }

    return DELIVERY_TRESHOLD;
}

#========================================================================================
# function     GetLastOrders
# state        public
#----------------------------------------------------------------------------------------
# syntax       #GetLastOrders(#User, #Number)
# example      #GetLastOrders(#Object, 2)
#----------------------------------------------------------------------------------------
# description  Returns the last orders of a user
#----------------------------------------------------------------------------------------
# input        #User | User object | object
# input        #Number | number of orders | int
#========================================================================================
sub GetLastOrders {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($User, $Number) = @$aParams;

    TestObject('User', $User, 'DE_EPAGES::Permission::API::Object::User');
    TestInteger('Number', $Number, 0);

    my $Shop = $User->getSite();
    my $Customer = GetCustomerForUser($User, $Shop);

    return [] unless defined $Customer;

    my $SearchDefinition = GetSearchDefinition(LoadClassByAlias('CustomerOrder'), 'LastOrderSearch');

    my %Options = (
        'ObjectID' => $User->id,
        'SearchDefinition' => $SearchDefinition,
        'Parameters' => {
            'CustomerID' => $Customer->id,
        },
        'OrderBy'    => 'CreationDate',
        'OrderDesc'  => 1,
        'PageSize'   => $Number,
        'Page'       => 1,
    );
    my $Search = DE_EPAGES::Search::API::Object::PagerSearch->new(\%Options);

    return [map { $_->tleHash } @{$Search->items(1)}];
}
#========================================================================================
# function     GetLastCustomerOrderAndOrderCount
# state        public
#----------------------------------------------------------------------------------------
# syntax       #GetLastCustomerOrderAndOrderCount(#Customer)
# example      #GetLastCustomerOrderAndOrderCount(#Customer)
#----------------------------------------------------------------------------------------
# description  Returns the last order and the count of orders of a customer
#----------------------------------------------------------------------------------------
# input        #Customer | Customer object | object
#========================================================================================
sub GetLastCustomerOrderAndOrderCount {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($Customer) = @$aParams;

    TestObject('Customer', $Customer, 'DE_EPAGES::Customer::API::Object::Customer');

    my $SearchDefinition = GetSearchDefinition(LoadClassByAlias('CustomerOrder'), 'LastOrderSearch');
    my %Options = (
        'ObjectID' => $Customer->id,
        'SearchDefinition' => $SearchDefinition,
        'Parameters' => {
            'CustomerID' => $Customer->id,
        },
        'OrderBy'    => 'CreationDate',
        'OrderDesc'  => 1,
        'PageSize'   => 1,
        'Page'       => 1,
    );
    my $Search = DE_EPAGES::Search::API::Object::PagerSearch->new(\%Options);

    # get only the 1st item
    my $FoundItem = @{$Search->items(1)}[0];

    my $hItem = {};
    $hItem = $FoundItem->tleHash if defined $FoundItem;

    # add the count of all orders
    my $ItemCount = scalar(@{$Search->itemIDs(0)});
    $hItem->{'CustomerOrderCount'} = $ItemCount;

    return $hItem ;
}

#========================================================================================
# function     GetUnseenCustomerOrderCount
# state        public
#----------------------------------------------------------------------------------------
# syntax       #GetUnseenCustomerOrderCount()
# example      #GetUnseenCustomerOrderCount()
#----------------------------------------------------------------------------------------
# description  Returns the count of unseen orders
#----------------------------------------------------------------------------------------
# input        #Shop | Shop object | object
#========================================================================================
sub GetUnseenCustomerOrderCount {
    my $self = shift;
    my ($Processor, $aParams) = @_;
    my ($Shop) = @$aParams;

    return DAL_UnseenOrderCount($Shop->id);
}

1;
