#========================================================================================
# §description      Download an exchange rate file from ECB Europa for use internally
#========================================================================================

use strict;
use Getopt::Long;
use Time::HiRes qw (time);
use LWP::UserAgent;
use XML::Simple;
use Readonly;
use Data::Dumper;
use JSON;
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Warning qw (Warning);
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf );
use DE_EPAGES::Core::API::File qw ( WriteFile GetFileContent );
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );


Readonly my $MAX_FILE_AGE_HOURS => 23;
Readonly my $LOCAL_FILE => 'ExchangeRates.json';
Readonly my $URL => 'https://www.ecb.europa.eu/stats/eurofxref/eurofxref-daily.xml';
Readonly my @EU_CURRENCIES => ( 'EUR', 'BGN', 'HRK', 'CZK', 'DKK', 'HUF', 'PLN', 'RON', 'SEK', 'CHF', 'TRY' );


sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $Help;
    my $NoOutput;

    GetOptions(
        'help'        => \$Help,
        'nooutput'    => \$NoOutput,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
    );

    usage() if $Help;

    unless ( defined $StoreName ) {
        print STDERR "Missing parameter -storename.\n\n";
        usage();
    }
    my $time1 = time;

    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub {
            my $rContent = _getExchangeRates($NoOutput);

            if ( defined $rContent ) {
                print "\n[Setting 'ExchangeRates' to]\n" . $$rContent . "\n" unless $NoOutput;

                my $System = LoadRootObject();
                $System->set({
                    'ExchangeRates' => from_json $$rContent,
                });

                eval { SynchronizeCache(); };
                Warning(GetError()) if ExistsError();
            } else {
                print "\n[ERROR]\nUnable to set exchange rates\n" unless $NoOutput;
            }
        },
    );

    my $time2 = time;
    printf "need %.1f seconds\n", $time2 - $time1 if not $NoOutput;
}

#==============================
#
# Download the remote exchange rates file and save it into the secure WebRoot folder
#   - Use this file if it exists and is less than 23 hours old
#   - Otherwise, redownload it
#   - return its contents (ref.string), or undef in case of failure
#
#============================================
sub _getExchangeRates {
    my ($NoOutput) = @_;

    my $FullPath = $ENV{EPAGES_WEBSECURE} . '/' . $LOCAL_FILE;

    my $DoDownloadFile = 0;

    #-- File exists, try and stat it
    if ( -f $FullPath ) {
        my $FileMTime;
        eval {
            $FileMTime = (stat($FullPath))[9];
        };
        if ( ExistsError() ) {
            Warning(GetError());
            return undef;
        }

        my $Now = time();
        my $DifferenceInSecs = $Now - $FileMTime;

        #-- File is older than X hours, redownload
        if ( $DifferenceInSecs > $MAX_FILE_AGE_HOURS*60*60 ) {
            $DoDownloadFile = 1;
        }
    } else {
        #-- File does not exist, let's download it
        $DoDownloadFile = 1;
    }

    if ( $DoDownloadFile ) {
        eval {
            my $Ua = LWP::UserAgent->new;
            my $Req = HTTP::Request->new(GET => $URL);

            my $Res = $Ua->request($Req);

            if ($Res->is_success) {
                my $XMLContent = $Res->content;

                print "[GET: $URL]\n" . $XMLContent . "\n" unless $NoOutput;

                my $Xml = XML::Simple->new();
                my $hXML = $Xml->XMLin( $XMLContent );

                print "\n[Parse XML result]\n" . Dumper($hXML) . "\n" unless $NoOutput;

                my $hJson = {};
                foreach ( @{$hXML->{'Cube'}->{'Cube'}->{'Cube'}} ) {
                    $hJson->{$_->{'currency'}} = $_->{'rate'} if ElementOf( $_->{'currency'}, @EU_CURRENCIES);
                }

                my $JsonString = to_json $hJson;
                print "\n[Final JSON]\n" . $JsonString . "\n" unless $NoOutput;

                WriteFile($FullPath, \$JsonString);
            }
            else {
                die $Res->status_line, "\n";
            }
        };
        if ( ExistsError() ) {
            Warning(GetError());
            return undef;
        }
    }

    my $rContent;
    eval {
        $rContent = GetFileContent( $FullPath );
    };
    if ( ExistsError() ) {
        Warning(GetError());
        return undef;
    }

    return $rContent;
}

sub usage {
    print <<END_USAGE;
Description:
    Download an exchange rates file from ECB Europa for use internally

Usage:
    perl $0 [options] [flags]

options:
    -passwd     database user password                          ()
    -storename  name of store                                   ()
flags:
    -help       show the command line options
    -nooutput   do not print any output

Example:
    perl $0 -storename Store
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main );
