#========================================================================================
# package      DE_EPAGES::ThirdPartyConfig::API::Log
# state        public
#----------------------------------------------------------------------------------------
# description  Log functions for external payments.
#========================================================================================
package DE_EPAGES::ThirdPartyConfig::API::Log;
use base qw (Exporter);
use strict;

our @EXPORT_OK = qw (Log3rdParty);

use Scalar::Util qw (blessed);
use DE_EPAGES::Core::API::Log qw (LogCommunication);

#========================================================================================
# function     Log3rdParty
#----------------------------------------------------------------------------------------
# syntax       Log3rdParty($Name, $Debug, $Section);
#----------------------------------------------------------------------------------------
# example      Log3rdParty('SendRequestHash', \%Hash, 'Send');
#----------------------------------------------------------------------------------------
# description  Logs a structure to the communication.log.
#               epages objects are logged in the form '<objectid> <objectpath'
#               datetime objects are logged in 'yyyy-mm-dd hh:mm:ss'
#----------------------------------------------------------------------------------------
# input        $Name | name of data | string
# input        $Debug | data to be logged | ref.hash, ref.array, string, object
# input        $Section | name of the section to be logged (optional - defaults to name of calling sub) | string
# input        $aFilter | (optional) filter (perl regular expression for
#                       s/$Filter[^\n]+\n/\n/gis) | ref.array.string
# input        $DataMaxDepth | maximum depth of perl structures to be logged out | int
#========================================================================================
sub Log3rdParty($$;$$$) {
    my ($Name, $DebugStuff, $Section, $aFilter, $DataMaxDepth) = @_;

    $Section //= (caller(1))[3];
    $aFilter //= [];

    $DebugStuff = flattenObjects($DebugStuff);
    LogCommunication($Name, $DebugStuff, $Section, $aFilter, 2, $DataMaxDepth);
    return;
}

#========================================================================================
# function     flattenObjects
#----------------------------------------------------------------------------------------
# syntax       my $Data = flattenObjects($Data);
#----------------------------------------------------------------------------------------
# description  Converts all epages objects to a string containing objectid and pathString,
#               all datetime objects to a string of format '%Y-%m-%d %H:%M:%S %Z'.
#----------------------------------------------------------------------------------------
# input        $LogData | data | scalar, object, or reference
# return       $LogData | data | scalar, object, or reference
#========================================================================================
sub flattenObjects {
    my ($LogData) = @_;

    return undef unless defined $LogData;

    if (blessed($LogData)) {
        if (blessed($LogData) eq 'DateTime') {
            return $LogData->strftime('%Y-%m-%d %H:%M:%S %Z');
        }
        elsif ($LogData->can('id') && $LogData->can('pathString')) { # it's an epages object
            return sprintf('%d (%s)', $LogData->id, $LogData->pathString);
        }
    }
    elsif (ref($LogData) eq 'HASH') {
        return {map { $_ => flattenObjects($LogData->{$_}) } keys %$LogData};
    }
    elsif (ref($LogData) eq 'ARRAY') {
        return [map { flattenObjects($_) } @$LogData];
    }
    return $LogData;
}

1;
