#========================================================================================
# package      DE_EPAGES::Zettle::Hooks::Product
# state        public
# description  export stock level updates to Zettle
#========================================================================================
package DE_EPAGES::Zettle::Hooks::Product;

use strict;

use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Core::API::PerlTools qw ( Compare );
use DE_EPAGES::Object::API::Factory qw( LoadClassByAlias );
use DE_EPAGES::Zettle::API::Constants qw ( ZETTLE_ATTRIBUTES_PROFILE IMAGE_LARGE_PROFILE ZETTLE_OBJECT_ALIAS );
use DE_EPAGES::Zettle::API::WebService::InventoryService;
use DE_EPAGES::Zettle::API::ZettleService;
use DE_EPAGES::Zettle::DAL::Zettle qw( DAL_SetZettleDeleted );


#========================================================================================
# function     OnBeforeUpdateProduct
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnBeforeUpdateProduct( $hParams );
#----------------------------------------------------------------------------------------
# description  Export stock level updates to Zettle and update Zettle-related LastUpdateProfiles if necessary.
#               (Hook OBJ_BeforeUpdateProduct)
#----------------------------------------------------------------------------------------
# input        $hParams | objects
#               <ul>
#                   <li>Object - object - DE_EPAGES::Product::API::Object::Product
#                   <li>Values - new product values - ref.hash
#               </ul> | ref.hash.*
#========================================================================================
sub OnBeforeUpdateProduct {
    my ($hParams) = @_;

    my $Product = $hParams->{'Object'};
    my $Shop = $Product->getSite();

    return unless $Shop->featureMaxValue('Zettle');

    return unless $Shop->existsChild(ZETTLE_OBJECT_ALIAS);

    # skip if merchant disconnected from Zettle
    my $Environment = $Shop->child(ZETTLE_OBJECT_ALIAS);
    return if $Environment->requiresCleanup();

    my $ZettleService = DE_EPAGES::Zettle::API::ZettleService->new($Environment);
    return unless $ZettleService->isInitialExportComplete();

    _ExportStock($hParams);

    _UpdateProfiles($hParams);

    return;
}

#========================================================================================
# function     OnDeleteProduct
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnDeleteProduct( $hValues );
#----------------------------------------------------------------------------------------
# description  OnDeleteProduct is triggered when a product is deleted and we have to:
#               <ul>
#                   <li>disable Zettle inventory tracking for the product deleted, if required.
#                   <li>store ZettleUUID in zettledelete table.
#               </ul>
#               (Hook OBJ_DeleteProduct)
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                 <li>Object        - product object       - object
#                 <li>ProductID     - product identifier   - int
#               </ul> | ref.hash.*
#========================================================================================
sub OnDeleteProduct {
    my ($hParams) = @_;

    my $Product = $hParams->{'Object'};

    my $Shop = $Product->getSite();

    return unless $Shop->featureMaxValue('Zettle');

    return unless $Shop->existsChild(ZETTLE_OBJECT_ALIAS);

    # skip if merchant disconnected from Zettle
    my $Environment = $Shop->child(ZETTLE_OBJECT_ALIAS);
    return if $Environment->requiresCleanup();

    my $ZettleUUID = $Product->get('ZettleUUID');
    return unless defined $ZettleUUID;

    # Disable inventory tracking for superproducts whose ZettleTracking is enabled
    my $TrackingEnabled = $Product->get('ZettleTrackingEnabled');
    my $SuperProduct = $Product->superProduct();
    if($TrackingEnabled && ! defined $SuperProduct){
        eval{
            my $InventoryService = DE_EPAGES::Zettle::API::WebService::InventoryService->new($Environment);
            $InventoryService->disableProductTracking($ZettleUUID);
        };
        if(ExistsError()){
            my $Error = GetError();
            LogError($Error);
        }
    }

    # Register ZettleUUID in zettledelete table:
    # this data is used during scheduled or manual product export to send deletions to Zettle
    eval{
        DAL_SetZettleDeleted({
            GUID => $Product->guid,
            ClassID => LoadClassByAlias('Product')->id,
            SiteID => $Shop->id,
            PathFromSite => $Product->get('RelativePath'),
            ZettleUUID => $ZettleUUID,
        });
    };
    if(ExistsError()){
        my $Error = GetError();
        LogError($Error);
    }
}

#========================================================================================
# function     _ExportStock
# state        private
#----------------------------------------------------------------------------------------
# syntax       _ExportStock( $hParams );
#----------------------------------------------------------------------------------------
# description  Export stock level updates to Zettle
#----------------------------------------------------------------------------------------
# input        $hParams | objects
#               <ul>
#                   <li>Object - object - DE_EPAGES::Product::API::Object::Product
#                   <li>Values - new product values - ref.hash
#               </ul> | ref.hash.*
#========================================================================================
sub _ExportStock {
    my ($hParams) = @_;

    # only react if stocklevel changed.
    return unless defined $hParams->{'Values'};
    return unless defined $hParams->{'Values'}->{'StockLevel'};

    # Prevent incoming stock updates (Zettle webhooks) to be sent back to Zettle
    if(defined $hParams->{'AdditionalOptions'}->{'IgnoreZettleUpdate'} && $hParams->{'AdditionalOptions'}->{'IgnoreZettleUpdate'} == 1){
        delete $hParams->{'AdditionalOptions'}->{'IgnoreZettleUpdate'};
        return;
    }

    my $Product = $hParams->{'Object'};
    my $hNewValues = $hParams->{'Values'};
    my $NewStockLevel = $hNewValues->{'StockLevel'};

    my $Shop = $Product->getSite();
    my $Environment = $Shop->child(ZETTLE_OBJECT_ALIAS);
    my $InventoryService = DE_EPAGES::Zettle::API::WebService::InventoryService->new($Environment);
    my $OldStockLevel = $Product->get('StockLevel');

    my $SuperProduct = $Product->superProduct;
    my $ProductUuid = (defined $SuperProduct) ? $SuperProduct->get('ZettleUUID') : $Product->get('ZettleUUID');
    my $VariantUuid = $Product->get('ZettleUUID');

    # Avoid sending stock updates when a product is cloned (apparently set({'StockLevel' ...}) is called and triggers this call)
    return unless defined $ProductUuid && defined $VariantUuid;

    eval{
        my $BalanceChange = $NewStockLevel-$OldStockLevel;
        #TODO: process ProductBalanceDTO returned and log a warning if Zettle balance doesn't match $NewStockLevel
        $InventoryService->updateProductBalance($ProductUuid, $VariantUuid, $BalanceChange) if $BalanceChange != 0;
    };
    if(ExistsError()){
        my $Error = GetError();
        LogError($Error);
    }

    return;
}

#========================================================================================
# function     _UpdateProfiles
# state        private
#----------------------------------------------------------------------------------------
# syntax       _UpdateProfiles( $hParams );
#----------------------------------------------------------------------------------------
# description  Update Zettle LastUpdateProfiles if any attribute of interest was modified.
#               Register changes with regards to two different profiles:
#               <ul>
#                   <li>ZettleAttributes - changes in basic product attributes of interest to Zettle export
#                   <li>ImageLarge - changes in product image
#               </ul>
#               A separate ImageLarge profile is used since images are handled and uploaded in different API endpoints (ImageService vs ProductService)
#----------------------------------------------------------------------------------------
# input        $hParams | objects
#               <ul>
#                   <li>Object - object - DE_EPAGES::Product::API::Object::Product
#                   <li>Values - new product values - ref.hash
#               </ul> | ref.hash.*
#========================================================================================
sub _UpdateProfiles {
    my ($hParams) = @_;

    my $Product = $hParams->{'Object'};
    my $Shop = $Product->getSite();
    my $LanguageID = $Shop->get('LanguageID');
    my $hNewValues = $hParams->{'Values'};

    # Register changes in basic product attributes
    my @ZettleAttributes = qw( Name Description ManufacturerSKU UPCEAN TaxClass );
    if (my @UpdatedAttributes = grep { exists $hNewValues->{$_} } @ZettleAttributes) {
        my $hOldValues = $Product->get(\@UpdatedAttributes, $LanguageID);
        if ( grep { Compare('String', $hNewValues->{$_}, $hOldValues->{$_}) != 0} @UpdatedAttributes) {
            my $ZettleAttributesProfile = LoadClassByAlias('Product')->lastUpdateProfile(ZETTLE_ATTRIBUTES_PROFILE);
            $Product->setUpdated( $ZettleAttributesProfile );
        }
    }

    # Register changes in product image (ImageLarge)
    my $ImageAttribute = 'ImageLarge';
    if (exists $hNewValues->{$ImageAttribute}) {
        my $imageOldValue = $Product->get($ImageAttribute);
        if (Compare('String', $hNewValues->{$ImageAttribute}, $imageOldValue) != 0) {
            my $ZettleImageProfile = LoadClassByAlias('Product')->lastUpdateProfile(IMAGE_LARGE_PROFILE);
            $Product->setUpdated( $ZettleImageProfile );
        }
    }
}

1;
