use strict;

use Getopt::Long;
use Time::HiRes qw (time);

use DE_EPAGES::Cartridge::API::Table::Cartridge qw ( ExistsCartridgeByPackage );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::Warning qw ( Warning BlockWarnings );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Database::API::Connection qw ( RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( ExistsClassByAlias LoadClassByAlias LoadObjectByPath );
use DE_EPAGES::Shop::API::Feature qw( LoadFeatureByAlias );
use DE_EPAGES::WebInterface::API::MessageCenter qw ( SynchronizeCache );
use DE_EPAGES::Zettle::API::Constants qw ( ZETTLE_ENVIRONMENT_CLASS ZETTLE_OBJECT_ALIAS );
use DE_EPAGES::Zettle::API::WebService::ProductService;
use DE_EPAGES::Zettle::API::Tools qw ( LoadProductByZettleUUID );

# Store to be used if 'storename' argument is not present
use constant DEFAULT_STORE => 'Store';
use constant OLD_FAKE_OPTION_NAME => 'FakeOption';

my $ShopAlias;

# Fix Variations that have fake options in old format. These fake option names and values will be updated to the new format.
sub Main {

    local $| = 1;

    my ($Help, $NoOutput, $Password, $StoreName);

    GetOptions(
        'help'        => \$Help,
        'nooutput'    => \$NoOutput,
        'passwd=s'    => \$Password,
        'storename=s' => \$StoreName,
        'shop=s'      => \$ShopAlias,
    );

    $StoreName //= DEFAULT_STORE;

    usage() if $Help;

    my $time1 = time;

    RunOnStore(
        Store      => $StoreName,
        DBPassword => $Password,
        Sub        => sub {
            # make sure that the cartridge is installed
            unless(ExistsCartridgeByPackage('DE_EPAGES::Zettle')) {
                printf STDERR ("[%s] WARN: Cartridge Zettle is not installed.\n", $StoreName) unless $NoOutput;
                return;
            }

            my @EnvironmentsForFixingFakeOptions = ();
            # Run script on a specific shop if 'shop' argument is present (and different from 'all')
            if(defined $ShopAlias && $ShopAlias ne 'all'){
                my $Shop;
                eval {
                    $Shop = LoadObjectByPath('/Shops')->child( $ShopAlias );
                };
                if(ExistsError()){
                    printf STDERR ("[%s] WARN: Shop '%s' does not exist.\n", $StoreName, $ShopAlias);
                    return;
                }

                return unless $Shop->existsChild(ZETTLE_OBJECT_ALIAS);

                my $ZettleEnvironment = $Shop->child(ZETTLE_OBJECT_ALIAS);

                push (@EnvironmentsForFixingFakeOptions, $ZettleEnvironment);
            } else {
                unless(ExistsClassByAlias(ZETTLE_ENVIRONMENT_CLASS)){
                    printf STDERR ("[%s] WARN: Class '%s' does not exist.\n", $StoreName, ZETTLE_ENVIRONMENT_CLASS);
                    return;
                }

                # Retrieve all Zettle instances from store
                my $aZettleEnvironments = LoadClassByAlias(ZETTLE_ENVIRONMENT_CLASS)->objects;
                
                foreach my $ZettleEnvironment ( @{ $aZettleEnvironments }){
                    push (@EnvironmentsForFixingFakeOptions, $ZettleEnvironment) ;
                }
            }

            FixFakeVariationOptionsJob(\@EnvironmentsForFixingFakeOptions, $StoreName, $NoOutput);
        },
    );

    my $time2   = time;
    my $seconds = $time2 - $time1;
    my $timemsg = sprintf( 'need %.3f seconds', $seconds );
    print $timemsg. "\n" unless $NoOutput;

    eval { SynchronizeCache(); };
    Warning( GetError() ) if ExistsError();
}

sub FixFakeVariationOptionsJob {
    my ($Environments, $StoreName, $NoOutput) = @_;
    
    return unless scalar @{$Environments};

    my $Feature = LoadFeatureByAlias('Zettle');
    foreach my $Environment ( @{$Environments} ) {
        my $EnvPath = $Environment->pathString;
        my $Shop = $Environment->getSite();

        # skip export if Zettle feature is disabled for this shop
        my $FeatureValue = $Shop->featureMaxValue($Feature);
        next if ( not $FeatureValue );

        # skip if shop is ClosedByProvider
        next if ($Shop->get('ClosedByProvider'));

        my $aWarnings = BlockWarnings(
            sub {
                eval {
                    my $EnvironmentShopAlias = $Shop->get('Alias');

                    # Get products from zettle
                    my $ProductService = DE_EPAGES::Zettle::API::WebService::ProductService->new($Environment);
                    my $aProductResponseDTO = $ProductService->getAllProducts();

                    my @aProductsWithVariationsWithFakeOptions = ();

                    # Find variations with fake option value
                    foreach my $ProductResponseDTO (@{$aProductResponseDTO}) {
                        my $SuperProductUUID = $ProductResponseDTO->uuid;
                        my $Variations = $ProductResponseDTO->variations;

                        # Must have only one variation
                        next if (scalar(@{$Variations}) != 1);
                        
                        my $hVariation = $Variations->[0];
                        # The variation must have the same uuid as the superproduct
                        next if ($SuperProductUUID ne $hVariation->{'uuid'});
                        
                        my $VariationOptions = $hVariation->{'options'};
                        # The variation must have only one option
                        next if (scalar(@{$VariationOptions}) != 1);

                        my $hOption = $VariationOptions->[0];
                        # Option name must match 'FakeOption'
                        next if ($hOption->{'name'} ne OLD_FAKE_OPTION_NAME);

                        eval {
                            my $SuperProduct = LoadProductByZettleUUID($Shop, $SuperProductUUID);
                            printf("[%s] Shop '%s' - Product with old fake option found in Zettle: ZettleUUID = '%s'\n",
                                    $StoreName, $EnvironmentShopAlias, $SuperProductUUID);
                            push @aProductsWithVariationsWithFakeOptions, $SuperProduct;
                        };
                        if (ExistsError()) {
                            printf("[%s] WARN: Shop '%s' - Product with old fake option missing from the shop: ZettleUUID = '%s'\n",
                                    $StoreName, $EnvironmentShopAlias, $SuperProductUUID);
                        }
                    }

                    my $ExportCount = scalar(@aProductsWithVariationsWithFakeOptions);
                    if($ExportCount > 0) {
                        $ProductService->exportProducts(\@aProductsWithVariationsWithFakeOptions, 0, undef);
                        printf ("[%s] Shop '%s' - '%d' products exported\n", $StoreName, $EnvironmentShopAlias, $ExportCount);
                    }
                };
                if(ExistsError()){
                    my $Error = GetError();
                    $Error->addVars({ 'Path' => $EnvPath });
                    Warning($Error);
                }
            }
        );

        foreach my $Warning ( @{$aWarnings} ) {
            LogError($Warning);

            printf ("[%s] WARN: %s: %s\n",
                            $StoreName,
                            $Warning->vars->{Path}//'path unknown' ,
                            substr( $Warning->shortMessage, 0, 4096 ) ); ## no critic (ProhibitMagicNumbers)
        }
    }

    return;
}

sub usage {
    print <<END_USAGE;
Description:
    Fix Variations that have fake options in old format. These fake option names and values will be updated to the new format.

Usage:
    perl $0 [options] [flags]

options:
    -passwd       database user password              ()
    -storename    name of store                       (Default: Store)
    -shop         shop alias             (optional)   (Default: all)
flags:
    -nooutput   do not print any output
    -help       show the command line options

Example:
    perl $0 -shop DemoShop
    perl $0 -storename Store -shop DemoShop
END_USAGE
    exit 2;
}

RunScript( 'Sub' => \&Main );
